function amalgamatefomUTLC

% Written by S.R. Jim, (C) October 2013
% Glancing Angle Deposition (GLAD) Laboratory, PI: M.J. Brett
% University of Alberta, Edmonton, Canada
% 
% Purpose:  This script is designed to amalgamate the figure of merit *.mat
% files output by the UTLC video analysis script fomUTLC.m.  It loads data
% arrays in order into a cell array in which each cell is a structure:
% 
% FOMs produced by combinePlateUTLCfom.m (See Note 1):
%   fom*, group, nfom, plateref, subset 
% 
% Basic FOMs produced by fomUTLC.m:
%   ampSpot, chi, firstFrame, frameRate, frameRes, hueRange1,
%   hueRangeLabel1, intLength, lastFrame, LODtVect, middleFrame, nFrames,
%   noisefloorLOG, nSpots1, nTracks1, RealPH, RealPN, relConc, resSpot,
%   rfLimits1, sigSpot, SN, SNfit, SNsVect, spotFWHM, spotHRF, spotLOD,
%   spotLOD0, spotLocationX1, spotLocationY1, stdevBlank, stdevBlankScaled,
%   stdevBlankScaledLOG, stdevBlankTrk, spotMass1, spotPH, spotPN, spotSNR,
%   spotVolume1, stdResiduals, stdSpot, sVect, t, tVect, undilutedConc,
%   zf1, zSpot
% 
% Averaged FOMs from fomUTLCavg.m:
%   avgSigTrk, avgSigName, avg_*, stdev_*
% 
% Note 1:
% It also can be used to amalgamate data produced by combinePlateUTLCfom.m.
% These data include:
%    fom* : All of the above data produced in a first pass through this
%       script.  Note that combinePlateUTLCfom.m requires an amalgamation
%       file produced by this script since it requires fom{}.variable
%       arrays.  To access data extracted during the first pass, use
%       "fom{}.fom{}.variable" (ie. nested structures.)
%    group : Relevant foms from all fom{} cells combined together.
%    nfom : Number of fom{} cells used by combinePlateUTLCfom.m
%    plateref : Vector used to record plate from which each track's data
%       was extracted.
%    subset : Array used to select data from fom{} cells.
% If present, these variables will be listed at beginning of new fom{}
% structure.
% 
% Note 2:
% If they exist, the script can import averaged FOMs produced by
% fomUTLCavg.m.  (Files output by fomUTLCavg.m also contain all those from
% fomUTLC.m.)  These data signals are imported if 'script_fomUTLC_avg'
% exists.

%% Import old (if desired) or define new amalgamation files
clear all; close all;
% Select old recipefile
disp(' ')
disp('PART 0: FILE DIRECTORIES')
loadold = input('Load old FOM amalgamation file (y/n): ','s');
if strcmp(loadold,'y')
    [oldAmalgam0,amalgamPath0] = uigetfile('*.mat', ...
        'Select old FOM amalgamation file');
    amalgamPath0 = [amalgamPath0 '\'];
    load([amalgamPath0 oldAmalgam0]);
    oldAmalgam = oldAmalgam0; amalgamPath = amalgamPath0;
    clear oldAmalgam0 amalgamPath0;
    disp(['Loaded FOM amalgamation file ''' oldAmalgam '''.'])
else
    amalgamPath = ''; loadold = 'n';
    disp('Creating entirely new recipe file.')
end

% Specify output recipefile name
disp('Define location and name of new FOM amalgamation file.')
amalgamPath = uigetdir(amalgamPath,['Select directory in which to '...
    'place new FOM amalgamation file.']);
amalgamPath = [amalgamPath '\'];
newAmalgam = ...
    input('Enter output FOM amalgamtion file name (name.mat): ','s');

% Variable to record the script name and revision
script_amalgamFOM = 'amalgamatefomUTLC.m rev October 2013';

% Initialize output recipe file (verify that save is permitted)
save([amalgamPath newAmalgam]);

%% Create desired cell arrays
disp(' ')
disp('PART 1: DEFINE CELL ARRAY DIMENSIONS')
goodcell = exist('fomFile','var') && exist('fomPath','var') && ...
    exist('fomName','var') && exist('fom','var');
if strcmp(loadold,'y') && goodcell
    celldim = [size(fom,1) size(fom,2) size(fom,3)];
    disp('Existing cell arrays have dimensions of:')
    disp(celldim)
else
    celldim = input('Desired cell dimensions as 3-element row vector: ');
    fomName = cell(celldim);    % Label assigned to each structure
    fomFile = cell(celldim);    % Name of FOM *.mat files
    fomPath = cell(celldim);    % Path to FOM *.mat files
    fom = cell(celldim);        % Cell array of stuctures for all FOM data
    disp(' ')
end

%% Import data from FOM *.mat files
disp('PART 2: IMPORT DESIRED CELL ARRAYS')
disp(' ')
disp('Data sets currently in FOM structure ''fom'':')
disp(fomName)
disp('From stored data files:')
disp(fomFile)

prevdir = [pwd '\'];        % Current directory
done = 'n';
while ~strcmp(done,'y')
    % Select desired cell
    disp(['Edit which cell in the {' num2str(celldim) '} array?'])
    a = input('Enter cell indices as a 3-element row vector: ');
    % Display current data set name and file
    disp(' ')
    disp(['Existing data set: ''' fomName{a(1),a(2),a(3)} ''''])
    disp(['Existing FOM data file: ''' fomFile{a(1),a(2),a(3)} ''''])
    % Delete, Import, or Refresh data; or Cancel
    disp(' ')
    disp('    Delete, import, or refresh data in this cell; or cancel?')
    clearfom = input('    (delete/import/refresh/cancel): ','s');
    % Check if trying to refresh non-existant or invalid data
    if strcmp(clearfom,'refresh') && ~exist([fomPath{a(1),a(2),a(3)} ...
            fomFile{a(1),a(2),a(3)}],'file')
        disp('    Cannot refresh data because no valid predefined')
        disp('    file and/or file path exists. Delete, import,')
        clearfom = input('    or cancel? (delete/import/cancel): ','s');
        if ~strcmp(clearfom,'delete') && ~strcmp(clearfom,'import')
            clearfom = 'cancel';
        end
    end
    % Cancel edit
    if ~strcmp(clearfom,'delete') && ~strcmp(clearfom,'import') && ...
            ~strcmp(clearfom,'refresh')
        clearfom = 'cancel';
        disp(['    Cancelled editing data in fom{' num2str(a) '}.'])
    end
    % Clear existing data in cell
    if strcmp(clearfom,'delete') || strcmp(clearfom,'import')
        fomFile{a(1),a(2),a(3)} = ''; fomPath{a(1),a(2),a(3)} = '';
        fomName{a(1),a(2),a(3)} = ''; fom{a(1),a(2),a(3)} = '';
        disp(['    Cleared existing data from fom{' num2str(a) '}.'])
    elseif strcmp(clearfom,'refresh')
        fom{a(1),a(2),a(3)} = '';
        disp(['    Cleared existing data from fom{' num2str(a) '}.'])
    end
    % Define input file directories
    if strcmp(clearfom,'import')
        fomName{a(1),a(2),a(3)} = ...
            input('    Enter name for this data set: ','s');
        disp(['    Select input data file for cell {' num2str(a) '}'])
        [fomFile{a(1),a(2),a(3)},fomPath{a(1),a(2),a(3)}] = ...
            uigetfile([prevdir '*.mat'], 'Select input FOM data file');
    elseif strcmp(clearfom,'refresh')
        disp(['    Refreshing data from ' fomFile{a(1),a(2),a(3)} ])
    end
    % Import or refresh data
    if strcmp(clearfom,'import') || strcmp(clearfom,'refresh')
        % Assume next cell will use same directory as current
        prevdir = fomPath{a(1),a(2),a(3)};
        disp('    Also import FOMs averaged from multiple tracks')
        disp('    (and their standard deviations) if they')
        import_avg = input('    exist (y/n)?: ','s');
        % Load given data file into fom
        disp(' ')
        disp(['    Loading data into fom{' num2str(a) '}...'])
        if strcmp(import_avg,'y') % Include averages and standard devs.
            fom{a(1),a(2),a(3)} = load([fomPath{a(1),a(2),a(3)} ...
                fomFile{a(1),a(2),a(3)}],... % From combineUTLCfom.m
                'fom*','group','nfom','plateref','subset',... % Basic FOMs
                'ampSpot','chi','firstFrame','frameRate','frameRes',...
                'hueRange1','hueRangeLabel1','intLength','lastFrame',...
                'LODtVect','middleFrame','nFrames','noisefloorLOG',...
                'nSpots1','nTracks1','RealPH','RealPN','relConc',...
                'resSpot','rfLimits1','sigSpot','SN','SNfit','SNsVect',...
                'spotFWHM','spotHRF','spotLOD','spotLOD0',...
                'spotLocationX1','spotLocationY1','stdevBlank',...
                'stdevBlankScaled','stdevBlankScaledLOG',...
                'stdevBlankTrk','spotMass1','spotPH','spotPN',...
                'spotSNR','spotVolume1','stdResiduals','stdSpot',...
                'sVect','t','tVect','undilutedConc','zf1','zSpot',...
                ...% Include averages and standard deviations
                'avgSigTrk','avgSigName','avg_*','stdev_*');
            disp(' ')
            disp(['    Done loading data into fom{' num2str(a) '}...'])
            disp ('    (Including track averaged FOMs and stdevs.)')
        else % Don't include averages and stdevs
            fom{a(1),a(2),a(3)} = load([fomPath{a(1),a(2),a(3)} ...
                fomFile{a(1),a(2),a(3)}],... % From combineUTLCfom.m
                'fom*','group','nfom','plateref','subset',... % Basic FOMs
                'ampSpot','chi','firstFrame','frameRate','frameRes',...
                'hueRange1','hueRangeLabel1','intLength','lastFrame',...
                'LODtVect','middleFrame','nFrames','noisefloorLOG',...
                'nSpots1','nTracks1','RealPH','RealPN','relConc',...
                'resSpot','rfLimits1','sigSpot','SN','SNfit','SNsVect',...
                'spotFWHM','spotHRF','spotLOD','spotLOD0',...
                'spotLocationX1','spotLocationY1','stdevBlank',...
                'stdevBlankScaled','stdevBlankScaledLOG',...
                'stdevBlankTrk','spotMass1','spotPH','spotPN',...
                'spotSNR','spotVolume1','stdResiduals','stdSpot',...
                'sVect','t','tVect','undilutedConc','zf1','zSpot');
            disp(['    Done loading data into fom{' num2str(a) '}...'])
        end
    end
    clear a clearfom import_avg;
    disp(' ')
    disp('Data sets currently in FOM structure ''fom'':')
    disp(fomName)
    done = input('Done editting data (y/n)?: ','s');
    disp('Saving recent changes to data set.')
    save([amalgamPath newAmalgam]);
    disp(' ')
end
clear prevdir done oldAmalgam0 amalgamPath0 loadold goodcell;
disp('Data sets in final FOM structure ''fom'':')
disp(fomName)
disp('From stored data files:')
disp(fomFile)
disp('Saving all changes to data set.')
save([amalgamPath newAmalgam]);
disp('*** SUCCESS ALL DESIRED DATA SETS HAVE BEEN LOADED ***')

end