function batchAnalyzeVideoUTLC

% Written by S.R. Jim, (C) October 2013
% Glancing Angle Deposition (GLAD) Laboratory, PI: M.J. Brett
% University of Alberta, Edmonton, Canada
% 
% Purpose:  Batch process a directory of UTLC video recipe files.  These
% recipe files uniquely define the parameters required in the analysis of
% each UTLC development video.  This function processes all recipe files in
% a given directory (*-recipe.mat suffix) using the following functions:
%     - mainUTLC.m:  Process and fit densitogram signals extracted from a
%     directory of images.  (Directory specified in each video's recipe
%     file.)
%     - fomUTLC.m:  Given the fitted densitogram peak locations and
%     widths, noise, and other data generated by the above script,
%     calculate relevant figures-of-merit.
%     - fomUTLCcsv.m:  Calculated FOMs are exported to *.csv by this
%     script.
% 
% Notes:
%   - This script does not calculate limits of detection (LODs).
%   - This script does not calculate separation numbers, real plate numbers
%   or real plate heights.

%% Define file directories
disp(' ')
disp('PART 0: DEFINE FILE DIRECTORIES')
disp(' ')

% Filepath and directory of *-recipe.mat files for processing
disp('Select a recipe file from the directory of UTLC video files ')
disp('(with extension ''-recipe.mat'' if possible.)')
[pickFile,recipePath] = uigetfile(...
    {'*-recipe.mat', 'UTLC Video Analysis Recipes (*-recipe.mat)'; ...
    '*.mat', 'MATLAB Data Files (*.mat)'}, ...
    'Select a recipe file from the directory of UTLC video files.');

% Determine appropriate suffix
if ~isempty(strfind(pickFile,'-recipe.mat'))  % Ends with *-recipe.mat
    pattern = '-recipe.mat';
else
    pattern = '.mat';
end
clear pickFile;

% Files with given suffix in name (*-recipe.mat or *.mat)
recipe = dir([recipePath '*' pattern]);
nRecipe = length(recipe); % Number of recipe files to process
disp(' ')
disp('The recipe file directory is:')
disp(recipePath)
disp(['    It contains ' num2str(nRecipe) ' recipe files.'])
disp(['    The first recipe is: ''' recipe(1).name '''.'])
disp(['    The last recipe is: ''' recipe(nRecipe).name '''.'])

% Select other directories
disp(' ')
disp('Select directory in which to place ''*-main.mat'' files')
disp('output by mainUTLC.m and input into fomUTLC.m:')
opFilePath = uigetdir(recipePath,['Select directory in which to '...
    'place new ''*-main.mat'' files']);
opFilePath = [opFilePath '\'];  % Path for *-main.mat files
disp(opFilePath)

disp(' ')
disp('Select directory in which to place ''*-fom.mat'' files')
disp('output by fomUTLC.m and input into fomUTLCcsv.m:')
opDataPath = uigetdir(opFilePath,['Select directory in which to '...
    'place new ''*-fom.mat'' files']);
opDataPath = [opDataPath '\'];  % Path for *-fom.mat files
disp(opDataPath)

disp(' ')
disp('Select directory in which to place sub directories of ''*.csv''')
disp('files output by fomUTLCcsv.m:')
opCSVPath = uigetdir(opDataPath,['Select directory in which to '...
    'place new subdirectories of ''*.csv'' files']);
opCSVPath = [opCSVPath '\'];  % Path for *.csv files
disp(opCSVPath)

% Generating output file prefixes (without ".mat" or "-recipe.mat")
opPrefix = cell(nRecipe,1);
for j = 1:nRecipe
    opPrefix{j} = recipe(j).name(1:(strfind(recipe(j).name,pattern) - 1));
end
clear j;
disp(' ')
disp('Generated cell array of output file prefixes ''opPrefix'' used in')
disp('filenames for subsequent ''*-main.mat'', ''*-fom.mat'', and ')
disp('''*_(fomname)-(dim1)-(dim2)-(dim3).csv'' files.')

disp(' ')
disp('All required directories/prefixes specified.')

%% Define structure of predefined inputs
disp(' ')
disp('PART 1: DEFINE spec STRUCTURE OF PREDEFINED INPUTS')
disp(' ')

% Inputs for mainUTLC.m function. sVect, tVect, tbVect may be 'all' or a
% vector use to specify which spots, tracks, or blank tracks to consider
spec.sVect = 'all';     % Analyte spots on each track to analyze
spec.tVect = 'all';     % Separation tracks to analyze
spec.tbVect = 'all';    % Blank tracks to analyze
spec.saveData = 'n';    % Save also densitograms, blank track data (y/n)?
% Optional: Save which of densitograms, blank track data?
spec.sampleFreq = NaN;  % Save this data for one out of every ___ frames
spec.saveDensitogram = 'n'; % Raw/fitted densitograms?
spec.saveblankRaw = 'n';    % Blank track data together as a single signal
spec.saveblankRawTrk = 'n'; % Blank track data as separate signals

% Inputs for fomUTLC.m function. 
spec.intLength = 4; % Number of sigmas over which to integrate fitted 
                    % signal; 'FWHM' (with quotes) to use the sqrt(8*ln2)
% Optional: Calculate LOD?
spec.calcLOD = 'n';     % Do not calculated LODs
spec.LODsVect = 'all';  % Calculate LOD for which spots?
spec.LODtVect = 'all';  % Use which tracks to calculate LOD? (Assume 
                        % different mass loadings per track)
spec.lodcriteria = 3;   % How many "sigmas" in LOD calculation?
% Optional: Calculate separation number and real plate height/number? If
% yes, calculate using which spots?  If no, set as "skip'
spec.SNsVect = 'skip';

disp('All predefined inputs specified (in this function''s *.m file')

%% Process data
disp(' ')
disp('PART 3: PROCESS ALL RECIPE FILES')
disp(' ')

process = input('Start processing data or abort (y/n)?: ','s');
if ~strcmp(process,'y')
    error('Script aborted by Silly user... :-(');
end
clear process;

% Time information
disp(['Start processing at: ' datestr(clock, 'yyyy-mm-dd, HH:MM')])

% Process data
for j = 1:nRecipe
    % Display messages
    disp(' ')
    disp('~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*')
    
    % Run mainUTLC.m
    % mainUTLC(spec,(input recipe path\file), (output main path\file))
    disp(' ')
    disp('STARTING PROCESS:  mainUTLC.m  .............................')
    disp(['(RECIPE ' num2str(j) ' OF ' num2str(nRecipe) ')'])
    disp(' ')
    mainUTLC(spec,[recipePath recipe(j).name], ...
        [opFilePath opPrefix{j} '-main.mat']);
    disp(' ')
    disp('COMPLETED PROCESS:  mainUTLC.m  ............................')
    
    % Run fomUTLC.m
    % fomUTLC(spec,(input main path\file), (output fom path\file))
    disp(' ')
    disp('STARTING PROCESS:  fomUTLC.m  ...............................')
    disp(['(RECIPE ' num2str(j) ' OF ' num2str(nRecipe) ')'])
    disp(' ')
    fomUTLC(spec,[opFilePath opPrefix{j} '-main.mat'], ...
        [opDataPath opPrefix{j} '-fom.mat']);
    disp(' ')
    disp('COMPLETED PROCESS:  fomUTLC.m  ..............................')
    
    % Run fomUTLCcsv.m
    % fomUTLCcsv((input main path\file), (output fom path\filePrefix))
    disp(' ')
    disp('STARTING PROCESS:  fomUTLCcsv.m  ...........................')
    disp(['(RECIPE ' num2str(j) ' OF ' num2str(nRecipe) ')'])
    disp(' ')
    % Create a subdirectory if it doesn't exist
    if ~exist([opCSVPath opPrefix{j} '\'],'dir')
        mkdir([opCSVPath opPrefix{j} '\']);
        disp('    * Created new directory for *.csv files: ')
        disp(['    * ' opCSVPath opPrefix{j} '\'])
    end
    fomUTLCcsv([opDataPath opPrefix{j} '-fom.mat'], ...
        [opCSVPath opPrefix{j} '\' opPrefix{j}]);
    disp(' ')
    disp('COMPLETED PROCESS:  fomUTLCcsv.m  ..........................')
    
    % Display recipe file completion time (don't use tic/toc because this
    % is used by mainUTLC.m
    disp(' ')
    disp(['Completed processing recipe file at: ' ...
        datestr(clock, 'yyyy-mm-dd, HH:MM')])
end

% Processing complete
disp(' ')
disp('~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*')
disp(' ')
disp('SUCCESS!  BATCH PROCESSING COMPLETED')
disp(' ')

disp(['Completed processing all recipe files at: ' ...
    datestr(clock, 'yyyy-mm-dd, HH:MM')])

clear j tavg tstart recipe recipePath nRecipe;
    
end