function combinePlateUTLCfom

% Written by S.R. Jim, (C) October 2013
% Glancing Angle Deposition (GLAD) Laboratory, PI: M.J. Brett
% University of Alberta, Edmonton, Canada
% 
% Purpose:  This script is designed to compare the figures of merit
% measured for similarly loaded GLAD UTLC plates.  Videos are analyzed
% using createRecipeUTLC.m --> mainUTLC.m --> fomUTLC.m --> (*OPTIONAL:
% fomUTLCavg.m -->) amalgamatefomUTLC.m.  The last script is used to create
% a 1-D cell array.  Each cell contains all relevant data for one
% run/video.  (*OPTIONAL:  The averaged FOMs produced by fomUTLCavg.m are
% not required by this script.  However, the data file output by that
% script also contains the required data from fomUTLC.m.)
% 
% The script assumes that all development videos are equivalent in loading
% (same analyte, concentration, spot volume, etc.) and processed
% similarly (same frame extraction rate, frame resolution, hue filters,
% signal integration region length, etc.)  Although the number of tracks
% per plate does not need to be the same between plates, it is important to
% note that ALL tracks from EACH plate will be used.
% 
% Notes:
%   - Also calculates relative standard deviations.
%   - Only data for the same number of spots are combined.   (Example: If
%   fom{1} only has data for 3 spots, ensure that only 3 spots from fom{2}
%   are added into the group arrays.
%   - Arrays generated by mainUTLC.m and reduced in fomUTLC.m are copied
%   and corrected if necessary.  Example: Suppose mainUTLC.m generates data
%   for 4 spots and 3 tracks but only 3 spots (sVect = [1 3 4]) and 2
%   tracks (tVect = [1 2]) are considered in fomUTLC.m.  Then generate a
%   new array with this subset and append this to the group arrays.

%% Set file directories
clear all; close all;
disp(' ')
disp('PART 0:  SET INPUT AMALGAMATION AND OUTPUT DATA DIRECTORIES/FILES')
disp(' ')
% Select and load recipe file
disp('Select input ''amalgamatefomUTLC'' file.')
[ipAmalg0,ipAmalgPath0] = uigetfile('*.mat', ...
    'Select input ''amalgamatefomUTLC'' file');
ipAmalgPath0 = [ipAmalgPath0 '\'];
load([ipAmalgPath0 ipAmalg0]);
ipAmalg = ipAmalg0; ipAmalgPath = ipAmalgPath0;
clear ipAmalg0 ipAmalgPath0;
disp(['Loaded input ''amalgamatefomUTLC'' file ''' ipAmalg '''.'])

% Specify output file name
disp(' ')
disp('Define location and name of output file.  This contains data')
disp('from input ''amalgamatefomUTLC'' file and calculations')
disp('from this script.')
opAmalgPath = uigetdir(ipAmalgPath,['Select directory in which to '...
    'place output file.']);
opAmalgPath = [opAmalgPath '\'];
opAmalg = input('Enter output data file name (name.mat): ','s');

% Variable to record the script name and revision
script_combinePlateUTLCfom = 'combinePlateUTLCfom.m rev October 2013';

%% Ensure fom cell array is a 1-D vector 
disp(' ')
disp('PART 1:  ERROR CHECK fom{} CELL ARRAY OF STRUCTURES')
disp(' ')

% Ensure that fom cell array is valid
good_Amalg = exist('fom','var') && isvector(fom);
if ~good_Amalg
    error('No valid ''fom'' cell vector in input amalgamation file');
end
nfom = length(fom);     % Number of fom cells

disp('fom{} cell array error checked.')

%% Error checking
disp(' ')
disp('PART 2:  ENSURE NECESSARY/OPTIONAL FOM VARIABLES EXIST')

% Initialize logical variables to store error check variables
good_AmalgFile = false(nfom,1);
good_AmalgFile_loading = false(nfom,1);
good_AmalgFile_SN = false(nfom,1);

% Initialize variable used to ensure all fom{} cells have data for the
% *same* number of spots (name consistent with those pulled by
% amalgematefomUTLCv*.m script.
nSpots1 = zeros(nfom,1);     % Later set as scalar

% Create vectors storing number of frames and tracks in each cell (later
% set as scalars) (Names consistent with those pulled by
% amalgematefomUTLCv*.m script.
nFrames = zeros(nfom,1);
nTracks1 = zeros(nfom,1);

% Perform error checking
for t = 1:nfom
    disp(' ')
    disp(['Checking fom{' num2str(t) '}: ''' fomName{t} ''''])
    
    % Test for necessary variables
    good_AmalgFile(t) = isfield(fom{t},'ampSpot') && ...
        isfield(fom{t},'hueRange1') && ...
        isfield(fom{t},'hueRangeLabel1') && ...
        isfield(fom{t},'nSpots1') && isfield(fom{t},'nTracks1') && ...
        isfield(fom{t},'resSpot') && isfield(fom{t},'sigSpot') && ...
        isfield(fom{t},'spotFWHM') && isfield(fom{t},'spotHRF') && ...
        isfield(fom{t},'spotPH') && isfield(fom{t},'spotPN') && ...
        isfield(fom{t},'spotSNR') && isfield(fom{t},'stdResiduals') && ...
        isfield(fom{t},'stdSpot') && isfield(fom{t},'sVect') && ...
        isfield(fom{t},'tVect') && isfield(fom{t},'zSpot') && ...
        isfield(fom{t},'zf1');
    if good_AmalgFile(t)
        % Record number of frames, spots, and tracks in cell
        nFrames(t) = fom{t}.nFrames;
        nSpots1(t) = fom{t}.nSpots1;
        nTracks1(t) = fom{t}.nTracks1;
        disp(['    All basic fomUTLC variables exist for ' ...
            num2str(nSpots1(t)) ' spots.'])
    else
        disp('    Some basic variables are missing.  Necessary ')
        disp('    variables: ''ampSpot'', ''hueRange1'', ')
        disp('    ''hueRangeLabel1'', ''nSpots1'', ''nTracks1'', ')
        disp('    ''resSpot'', ''sigSpot'', ''spotFWHM'', ')
        disp('    ''spotHRF'', ''spotPH'', ''spotPN'', ''spotSNR'', ')
        disp('    ''stdResiduals'', ''stdSpot'', ''sVect'', ''tVect'', ')
        disp('    ''zSpot'', and ''zf1''.')
        disp(' ')
        disp('    *** Script will abort after checking other fom{} ***')
        disp(' ')
    end
    
    % Test for LOD variables
    good_AmalgFile_loading(t) = isfield(fom{t},'spotMass1') && ...
        isfield(fom{t},'spotVolume1');
    if good_AmalgFile_loading(t)
        disp('    All spot loading data also exist.')
    else
        disp('    Spot loading data not present.')
    end

    % Test for separation number and real plate number variables
    good_AmalgFile_SN(t) = isfield(fom{t},'SNcalculate') && ...
        strcmp(fom{t}.SNcalculate,'y') && isfield(fom{t},'SN') && ...
        isfield(fom{t},'RealPH') && isfield(fom{t},'RealPN');
    if good_AmalgFile_SN(t)
        disp('    All SN and real plate number/height data also exist.')
    else
        disp('    SN and real plate number/height data not present.')
    end
    
end
clear t;

% Display error checking results (ensure all have given data)
good_AmalgFile = min(good_AmalgFile);
good_AmalgFile_loading = min(good_AmalgFile_loading);
good_AmalgFile_SN = min(good_AmalgFile_SN);
disp(' ')
if good_AmalgFile
    disp('All fom{} cells contain all basic fomUTLC variables.')
else
    disp('Not all fom{} cells contain all basic fomUTLC variables.')
    disp('    ***Script will abort after error checking is complete.')
end
if good_AmalgFile_loading
    disp('All fom{} cells also contain spot loading data.')
else
    disp('Not all fom{} cells contain spot loading data.')
    disp('Skip this analysis.')
end
if good_AmalgFile_SN
    disp('All fom{} cells also contain SN and real plate ')
    disp('number/height data.')
else
    disp('Not all fom{} cells contain SN and real plate ')
    disp('number/height data.  Skip this analysis.')
end

% Abort if necessary basic data does not exist
if ~good_AmalgFile
    disp(' ')
    error('Abort script because minimum basic FOM data is missing!')
end
clear good_AmalgFile;

% Ensure that same number of spots exist in all fom{} cells
disp(' ')
if max(nSpots1) == min(nSpots1)   % All entries are identical
    nSpots1 = nSpots1(1);         % Convert into scalar
    disp(['All fom{} cells contain data for ' ...
        num2str(nSpots1) ' spots.'])
else
    disp(['fom{} cells 1-' num2str(nfom) ' contain data for ____ spots:'])
    disp(nSpots1')
    disp('    Combine data for the first ____ spots?  (At most ')
    nSpotsIP = input(['    ' num2str(min(nSpots1)) ...
        ' spots; control-c to exit.): ']);
    nSpots1 = min(nSpotsIP,min(nSpots1)); % Now a scalar
    disp(['    Combining data for only the first ' ...
        num2str(nSpots1) ' spots.'])
end
clear nSpotsIP;

% Initialize output recipe file (verify that save is permitted)
disp(' ')
disp('Error checking complete.  Test save data to output file.')
save([opAmalgPath opAmalg]);

%% Define extraction
disp(' ')
disp('PART 3: DEFINE SUBSET OF EACH fom{} DATA TO COMBINE')
disp(' ')

% Show number of tracks
nTracks1 = sum(nTracks1);         % Now a scalar
disp(['Data for ' num2str(nTracks1) ' total tracks will be combined.'])
disp(' ')
% Allow user to choose which spot to use during alignment
s0 = 0; % Initialize
while (s0 < 1) || (s0 > nSpots1) || (ceil(s0) ~= floor(s0))
    s0 = input(['Use position of which of the ' num2str(nSpots1) ...
        ' spots during alignment?: ']);
end

% Initialize vector to record cell from which each signal was originally
% extracted (ex. [1 1 1 2 2 3 3 3 3] for 3 tracks from fom{1}, 2 from
% fom{2}, and 4 from fom{3})
plateref = zeros(nTracks1,1);

% Show and plot sample signal (highest hRF zSpot) from each fom{} cell and
% define plateref vector.  Ensure alignment of curves.
figure(1); clf; hold on; sp1 = zeros(nfom,1);
ind1 = 1;   % Initialize index for first iteration.
for t = 1:nfom
    disp(' ')
    disp(['Consider fom{' num2str(t) '}: ''' fomName{t} ''''])
    disp(['    The frame resolution was ' num2str(fom{t}.frameRes) ...
        ' px/mm.'])
    disp(['    Data was extracted from frames at ' ...
        num2str(fom{t}.frameRate) ' fps.'])
    disp(['    Data for ' num2str(fom{t}.nTracks1) ' tracks exists.'])
    disp(['    Signals contain data for ' num2str(fom{t}.nFrames) ...
        ' frames.'])
    
    % Preview some data in figure (zSpot of selected spot)
    sp1(t) = subplot(nfom,1,t);
    plot(permute(fom{t}.zSpot(:,s0,:),[1 3 2]),'.');
    title(['fom ' num2str(t) ' : Plot zSpot (px) for ' ...
        fom{t}.hueRangeLabel1{s0}])
    
    % Define plateref vector
    ind2 = ind1 + fom{t}.nTracks1 - 1;
    plateref(ind1:ind2) = t;
    ind1 = ind2 + 1;    % Start index for next iteration.
end
linkaxes(sp1,'x');  % Link x-axes of all plots
hold off;
clear t ind* sp1;

% Select subset of data
confirm = 'n';
while ~strcmp(confirm,'y')
    disp(' ')
    disp(['Define a ' num2str(nfom) ' x 2 array to select the data '])
    disp('subsets from each fom{} that should be combined together:')
    disp('One row for each fom{} cell.  Each row contains first ')
    disp('and last frame indices (ex. [1 302]).')
    subset = input('Enter array: ');
    
    % Ensure array is valid
    if (size(subset,1) == nfom) && (size(subset,2) == 2)
        subset = round(subset);
        subset(:,1) = max(subset(:,1),ones(nfom,1));    % first index
        subset(:,2) = min(subset(:,2),nFrames);         % second index    
        % Number of frames in subset (NOT in original fom{} cells anymore)
        nFrames = subset(:,2) - subset(:,1) + 1;
        nFrames = min(nFrames);                   % Convert into scalar
        % Recalculate last frame indices
        subset(:,2) = subset(:,1) + nFrames - 1;
        % Display adjusted array and confirm
        disp(' ')
        disp('Subset array adjusted to ensure that all indices are ')
        disp('valid and the same number of frames are selected.')
        disp('Adjusted subset array is:')
        disp(subset)
        
        % Plot preview of subset
        figure(2); clf; hold on; sp2 = zeros(nfom,1);
        for t = 1:nfom
            % Preview some data in figure (zSpot of selected spot)
            sp2(t) = subplot(nfom,1,t);
            plot(permute(fom{t}.zSpot(subset(t,1):subset(t,2),s0,:), ...
                [1 3 2]),'.');
            title(['fom ' num2str(t) ' : Plot zSpot (px) for ' ...
                fom{t}.hueRangeLabel1{s0} ' SUBSET: ' ...
                num2str(subset(t,1)) ':' num2str(subset(t,2))]);
        end
        clear t;
        linkaxes(sp2,'x');  % Link x-axes of all plots
        hold off;
        
        % Confirm arrays?
        confirm = input('Confirm these first/last indices (y/n)?: ','s');
    else
        disp('    Invalid array.  Redefine!')
    end
end
clear confirm sp2;

disp('Required subsets of fom{} data defined.')

%% Initialize variables
disp(' ')
disp('PART 4: INITIALIZE VARIABLES')
disp(' ')

% Zero arrays used during initialization
zeros2D = zeros(nFrames,nTracks1);
zeros3D = zeros(nFrames,nSpots1,nTracks1);

% Array to contain list of 3D figures of merit.  These will be stored in a
% structure of the form group.('fom') and used to produce new structures of
% these names
fomList3D = {'ampSpot'; 'sigSpot'; 'spotFWHM'; 'spotHRF'; 'spotPH'; ...
    'spotPN'; 'spotSNR'; 'stdResiduals'; 'stdSpot'; 'zSpot'};

% Initialize basic variables
for j = 1:length(fomList3D)
    group.(fomList3D{j}) = zeros3D;
    % Ouput data arrays initilized within calculation for loop
end
clear j;
% Initialize fields in group.____ and new structures *.b, *.se, etc.
group.zf1 = zeros2D;
    zf1.b = zeros(nFrames,2);
    zf1.se = zeros(nFrames,2);
    zf1.p = zeros(nFrames,2);
    zf1.dfe = zeros(nFrames,1);
group.resSpot = zeros((nSpots1-1), (nSpots1-1), nFrames, nTracks1);
    % Square base of resSpot array has dimension = nSpots1-1
    resSpot.b = zeros((nSpots1-1), (nSpots1-1), nFrames, 2);
    resSpot.se = zeros((nSpots1-1), (nSpots1-1), nFrames, 2);
    resSpot.p = zeros((nSpots1-1), (nSpots1-1), nFrames, 2);
    resSpot.dfe = zeros((nSpots1-1), (nSpots1-1), nFrames);

% Spot loading data, if it exists
if good_AmalgFile_loading
    group.spotMass1 = zeros(nSpots1,nTracks1);
        spotMass1.b = zeros(nSpots1,2);
        spotMass1.se = zeros(nSpots1,2);
        spotMass1.p = zeros(nSpots1,2);
        spotMass1.dfe = zeros(nSpots1,1);
    group.spotVolume1 = zeros(1,nTracks1);  % Initialization of 
                                            % spotVolume1.___ not needed
end

% Separation number and real plate number, if exists
if good_AmalgFile_SN 
    group.SN = zeros2D;
        SN.b = zeros(nFrames,2);
        SN.se = zeros(nFrames,2);
        SN.p = zeros(nFrames,2);
        SN.dfe = zeros(nFrames,1);
    group.RealPH = zeros2D;
        RealPH.b = zeros(nFrames,2);
        RealPH.se = zeros(nFrames,2);
        RealPH.p = zeros(nFrames,2);
        RealPH.dfe = zeros(nFrames,1);
    group.RealPN = zeros2D;
        RealPN.b = zeros(nFrames,2);
        RealPN.se = zeros(nFrames,2);
        RealPN.p = zeros(nFrames,2);
        RealPN.dfe = zeros(nFrames,1);
end

clear zeros2D zeros3D zeros4D;

disp('Required variables initialized.')

%% Combine data
disp(' ')
disp('PART 5: COMBINE SELECTED SUBSETS OF DATA FROM EACH fom{} CELL')
disp(' ')

for i = 1:nfom
    
    % Extract basic data with 3-D array shape
    for j = 1:length(fomList3D)
        % Ensure that the appropriate spot and track subset (selected in
        % fomUTLCv*.m) is considered.  Example case: 4 spots fit in
        % mainUTLCv*.m but only 3 are analyzed in fomUTLCv*.m.  The array 
        % 'stdResiduals' will contain data for 4 spots instead of 3 so
        % generate new 'stdResiduals1' and combine this with other data
        if size(fom{i}.(fomList3D{j}),2) ~= length(fom{i}.sVect) || ...
                size(fom{i}.(fomList3D{j}),3) ~= length(fom{i}.tVect)
            disp(' ')
            disp(['fom{' num2str(i) '}: ''' fomList3D{j} ...
                ''' has data for ' ...
                num2str(size(fom{i}.(fomList3D{j}),2)) ' spots and ' ...
                num2str(size(fom{i}.(fomList3D{j}),3)) ' tracks.'])
            disp('--> Creating and appending a new subset array: ')
            disp(['''' fomList3D{j} '1'' for spots, tracks:'])
            disp(fom{i}.sVect)  % Spot numbers
            disp(fom{i}.tVect)  % Track numbers
            % New array using sVect and tVect to select data.  Data for all
            % frames are still contained.
            fom{i}.([fomList3D{j} '1']) = ...
                fom{i}.(fomList3D{j})(:,fom{i}.sVect,fom{i}.tVect);
            % Combine subset of this data
            group.(fomList3D{j})(:,:,(plateref == i)) = ... 
                fom{i}.([fomList3D{j} '1'])...
                (subset(i,1):subset(i,2),1:nSpots1,:);
            disp('New array created and appended as required.')
        else
            % Put values into appropriate layers of array
            group.(fomList3D{j})(:,:,(plateref == i)) = ... 
                fom{i}.(fomList3D{j})...
                (subset(i,1):subset(i,2),1:nSpots1,:);
        end
    end
    
    % Extract other basic data
    group.zf1(:,(plateref == i)) = fom{i}.zf1(subset(i,1):subset(i,2),:);
    group.resSpot(:,:,:,(plateref == i)) = ...  % Only for nSpots1 spots
        fom{i}.resSpot(1:(nSpots1-1),1:(nSpots1-1), ...
        subset(i,1):subset(i,2),:);
    
    % Extract other data if pressent
    if good_AmalgFile_loading     % Spot loading data, if it exists
        group.spotMass1(:,(plateref == i)) = fom{i}.spotMass1(1:nSpots1,:);
        group.spotVolume1((plateref == i)) = fom{i}.spotVolume1;
    end

    if good_AmalgFile_SN % SN and real PN, if exist
        group.SN(:,(plateref == i)) = ...
            fom{i}.SN(subset(i,1):subset(i,2),:);
        group.RealPH(:,(plateref == i)) = ...
            fom{i}.RealPH(subset(i,1):subset(i,2),:);
        group.RealPN(:,(plateref == i)) = ...
            fom{i}.RealPN(subset(i,1):subset(i,2),:);
    end

end
clear i j;

disp('Data arrays combined as required.')

% Save data
disp(['Saving data to ' opAmalg])
save([opAmalgPath opAmalg]);

%% Perform linear regression to assess reproducibility between cells
disp(' ')
disp('PART 6: PERFORM LINEAR REGRESSION AND AVERAGING ACROSS fom{} CELLS')
disp(' ')

% Process basic data with 3-D array shape
disp('Processing basic figure-of-merit data.')
for j = 1:length(fomList3D)
    % Assign data to temporary array
    ipdata = group.(fomList3D{j});

    % Calculate averages, standard deviations (over all tracks); and
    % relative standard deviation (% rsd)
    avg = nanmean(ipdata,3);
    stdev = nanstd(ipdata,0,3);
    rsd = abs(100 * stdev ./ avg);
    
    % Initialize required data arrays (temporary)
    b = zeros(nFrames,nSpots1,2); % Linear fit const,slope
    se = zeros(nFrames,nSpots1,2); % Standard error in const,slope
    % Probability of obtaining a test statistic at least as extreme as the
    % one that was actually observed, assuming that the null hypothesis
    p = zeros(nFrames,nSpots1,2);
    dfe = zeros(nFrames,nSpots1);   % Degrees of freedom for error
	
    % Linear regression
    for f = 1:nFrames
        for s = 1:nSpots1
            
            % Ensure that there is enough data for regression.  Ensure that
            % at least 3 real value data points exist:
            %   (x1,y1), (x2,y2),..., (xN,yN)
            % With at least one unique x-value.
            test1 = sum(~isnan(ipdata(f,s,:))); % Number of valid data (>2)
            test2 = plateref(~isnan(ipdata(f,s,:))); % Column vector
            % Remove redundant elements (ie. data from same UTLC plate)
            test2 = unique(test2,'rows');
            test2 = length(test2);    % How many different UTLC plates?(>1)
            
            % Ordinary least squares regression across all tracks on each
            % plate (plate noted by plateref vector) if enough data points
            % from enough unique UTLC plates
            if (test1 > 2) && (test2 > 1)
                [b_fit,stats] = robustfit(...
                    plateref,permute(ipdata(f,s,:),[3 1 2]),'ols');
                % Fitted const,slope; and standard error and p-value for
                % these parameters
                b(f,s,1) = b_fit(1); b(f,s,2) = b_fit(2);
                se(f,s,1) = stats.se(1); se(f,s,2) = stats.se(2);
                p(f,s,1) = stats.p(1); p(f,s,2) = stats.p(2);
                % Degress of freedom
                dfe(f,s) = stats.dfe;
            else % Set all values to NaN
                b(f,s,1) = NaN; b(f,s,2) = NaN;
                se(f,s,1) = NaN; se(f,s,2) = NaN;
                p(f,s,1) = NaN; p(f,s,2) = NaN;
            end
            clear test*;
        end
    end
    
    % Assign to appropriate structure (use string as variable)
    eval([fomList3D{j} '.avg = avg;']);
    eval([fomList3D{j} '.stdev = stdev;']);
    eval([fomList3D{j} '.rsd = rsd;']);
    eval([fomList3D{j} '.b = b;']);
    eval([fomList3D{j} '.se = se;']);
    eval([fomList3D{j} '.p = p;']);
    eval([fomList3D{j} '.dfe = dfe;']);
    clear ipdata avg stdev rsd b se p dfe b_fit stats;
end
clear j f s;

% Other basic data: migration front and resolution
zf1.avg = nanmean(group.zf1,2);
zf1.stdev = nanstd(group.zf1,0,2);
resSpot.avg = nanmean(group.resSpot,4);
resSpot.stdev = nanstd(group.resSpot,0,4);

for f = 1:nFrames
    % Migration front position
    [b_fit,stats] = robustfit(plateref,group.zf1(f,:)','ols');
        zf1.b(f,:) = b_fit';
        zf1.se(f,:) = stats.se';
        zf1.p(f,:) = stats.p';
        zf1.dfe(f) = stats.dfe;
        clear b_fit stats;
    
    % Resolution: Reshape slice of 4D array into into 2D matrix of
    % dimension (nSpots1-1)^2 x nTracks1 array
    sig = reshape(group.resSpot(:,:,f,:), (nSpots1-1)^2,nTracks1,1,1);
    % Temporary 2D arrays to hold b values before reshaping
        res_b = zeros((nSpots1-1)^2,1,1,2);
        res_se = zeros((nSpots1-1)^2,1,1,2);
        res_p = zeros((nSpots1-1)^2,1,1,2);
        res_dfe = zeros((nSpots1-1)^2,1);
    % Linear regression
    for k = 1:(nSpots1-1)^2
        % Ensure that there is enough data for regression.  Ensure that
        % at least 3 real value data points exist:
        %   (x1,y1), (x2,y2),..., (xN,yN)
        % With at least one unique x-value.
        test1 = sum(~isnan(sig(k,:))); % Number of valid data (>2)
        test2 = plateref(~isnan(sig(k,:))); % Column vector
        % Remove redundant elements (ie. data from same UTLC plate)
        test2 = unique(test2,'rows');
        test2 = length(test2);    % How many different UTLC plates?(>1)
        
        % Ordinary least squares regression across all tracks on each
        % plate (plate noted by plateref vector) if enough data points
        % from enough unique UTLC plates
        if (test1 > 2) && (test2 > 1)
            [b_fit,stats] = robustfit(plateref,sig(k,:)','ols');
            res_b(k,1,1,1) = b_fit(1); res_b(k,1,1,2) = b_fit(2);
            res_se(k,1,1,1) = stats.se(1);
            res_se(k,1,1,2) = stats.se(2);
            res_p(k,1,1,1) = stats.p(1); res_p(k,1,1,2) = stats.p(2);
            res_dfe(k) = stats.dfe;
        else % Insufficent data points (ex. res of spot vs. itself)
            res_b(k,1,1,1) = NaN; res_b(k,1,1,2) = NaN;
            res_se(k,1,1,1) = NaN;
            res_se(k,1,1,2) = NaN;
            res_p(k,1,1,1) = NaN; res_p(k,1,1,2) = NaN;
            res_dfe(k) = NaN;
        end
        clear test*;
    end
    clear k sig;
    % Reshaping and inserting into relevant arrays
    resSpot.b(:,:,f,:) = reshape(res_b, (nSpots1-1),(nSpots1-1),1,2);
    resSpot.se(:,:,f,:) = reshape(res_se, (nSpots1-1),(nSpots1-1),1,2);
    resSpot.p(:,:,f,:) = reshape(res_p, (nSpots1-1),(nSpots1-1),1,2);
    resSpot.dfe(:,:,f) = reshape(res_dfe, (nSpots1-1),(nSpots1-1));
    clear b_fit stats res_b res_se res_p res_dfe;
end
clear f;
disp('Completed processing basic figure-of-merit data.  Saving...')

% Save data
save([opAmalgPath opAmalg]);

% % Spot loading data, if it exists
if good_AmalgFile_loading
    disp(' ')
    disp('Processing spot loading data.')
    % Spot volume
    spotVolume1.avg = nanmean(group.spotVolume1);
    spotVolume1.stdev = nanstd(group.spotVolume1,0);
    [b_fit,stats] = robustfit(plateref,group.spotVolume1','ols');
        spotVolume1.b = b_fit';
        spotVolume1.se = stats.se';
        spotVolume1.p = stats.p';
        spotVolume1.dfe = stats.dfe;
        clear b_fit stats;
    % Spot Mass
    spotMass1.avg = nanmean(group.spotMass1,2);
    spotMass1.stdev = nanstd(group.spotMass1,0,2);
    for s = 1:nSpots1
        [b_fit,stats] = robustfit(plateref,group.spotMass1(s,:)','ols');
            spotMass1.b(s,:) = b_fit';
            spotMass1.se(s,:) = stats.se';
            spotMass1.p(s,:) = stats.p';
            spotMass1.dfe(s) = stats.dfe;
            clear b_fit stats;
    end
    clear s;
    disp('Completed processing spot loading data.  Saving...')
    % Save data
    save([opAmalgPath opAmalg]);
end

% SN and real PN, if exist
if good_AmalgFile_SN
    disp(' ')
    disp('Processing SN, Real PH, Real PN data.')
    for f = 1:nFrames
        % Separation number
        [b_fit,stats] = robustfit(plateref,group.SN(f,:)','ols');
            SN.b(f,:) = b_fit';
            SN.se(f,:) = stats.se';
            SN.p(f,:) = stats.p';
            SN.dfe(f) = stats.dfe;
            clear b_fit stats;
        % Real plate height
        [b_fit,stats] = robustfit(plateref,group.RealPH(f,:)','ols');
            RealPH.b(f,:) = b_fit';
            RealPH.se(f,:) = stats.se';
            RealPH.p(f,:) = stats.p';
            RealPH.dfe(f) = stats.dfe;
            clear b_fit stats;
        % Separation number
        [b_fit,stats] = robustfit(plateref,group.RealPN(f,:)','ols');
            RealPN.b(f,:) = b_fit';
            RealPN.se(f,:) = stats.se';
            RealPN.p(f,:) = stats.p';
            RealPN.dfe(f) = stats.dfe;
            clear b_fit stats;
    end
    clear f;
    disp('Completed processing SN, Real PH, Real PN data.')
    % Save data
    save([opAmalgPath opAmalg]);
end

% Tidy-up
clear good_Amalg* fomList3D;

% Save data
disp(['Saving data to ' opAmalg])
save([opAmalgPath opAmalg]);

disp('***SUCCESS, SCRIPT COMPLETE***')

end