function createRecipeUTLC

% Written by S.R. Jim, (C) October 2013
% Based on createRecipeUTLCv2.m written by A.J. Oko, September 2011
% Glancing Angle Deposition (GLAD) Laboratory, PI: M.J. Brett
% University of Alberta, Edmonton, Canada
% 
% Purpose:  This script is used to define the parameters required for GLAD
% UTLC video analysis.  It enables input of an existing recipe file
% (oldRecipe) and output of a recipe file with a new filename (newRecipe).
% This recipe file is then input into a mainUTLCv* script for subsequent
% video analysis.
% 
% Subfunctions:  frontFit.m
% 
% Notes:
%   - Note 1:  PART 8 is used to define regions in frames from which blank
%   signals may be calculated.  Future work with signals may enable
%   estimation of system noise and limits of detection (LOD).  This feature
%   is not fully functional in the current time-resolved UTLC package.

clear all; close all;
%% Set old (if desired) and new recipe files; directory of input images
disp(' ')
disp('PART 0: FILE DIRECTORIES')

% Select old recipe file
oldRecipe0 = input('Load old recipe file (y/n): ','s');
if strcmp(oldRecipe0,'y')
    [oldRecipe0,recipePath0] = uigetfile('*.mat', ...
        'Select old recipe file');
    recipePath0 = [recipePath0 '\'];
    load([recipePath0 oldRecipe0]);
    oldRecipe = oldRecipe0; recipePath = recipePath0;
    disp(['Loaded recipe file ''' oldRecipe '''.'])
else
    oldRecipe = ''; recipePath = '';
    disp('Creating entirely new recipe file.')
end
clear oldRecipe0 recipePath0;

% Specify output recipefile name
disp('Define location and name of new recipe file.')
recipePath = uigetdir(recipePath,['Select directory in which to '...
    'place new recipe file.']);
recipePath = [recipePath '\'];
newRecipe = input('Enter output recipe file name (name.mat): ','s');

% Variable to record the script name and revision
script_Recipe = 'createRecipeUTLC.m rev October 2013';

% Initialize output recipe file (verify that save is permitted)
save([recipePath newRecipe]);

% Set directory of UTLC video frames to analyze
disp(' ')
disp('Choose directory of extracted video frame *.jpg/*.tif files.')
disp('Navigate to directory and select a frame of the correct ')
disp('file format: 8-bits/channel RGB *.jpg or *.tif.')
if ~exist('filepath','var')
    filepath = '';      % Initialize or assume same as for recipe file
end
if ~exist('filename','var')
    filename.name = '';	% Initialize if it doesn't already exist
end
[img_ext,filepath] = uigetfile(...
    {'*.jpg;*.tif', '*.jpg or *.tif images'; ...
    '*.jpg', '*.jpg images'; ...
    '*.tif', '*.tif images'}, ...
    'Select a frame (*.jpg or *.tif) extracted from video.', ...
    [filepath filename(1).name]);
[~,~,img_ext] = fileparts(img_ext);     % Find file extension
filename = dir([filepath '*' img_ext]); % Filenames only of given format

% Initialize relevant filename, image arrays, etc.
nFrames = length(filename);
disp(' ')
disp(['Selected directory contains ' num2str(nFrames) ' frames.'])
disp(['The first frame is: ''' filename(1).name ''''])
disp(['The last frame is: ''' filename(nFrames).name ''''])

% Extract specific frames for use throughout script
firstFrame = imread([filepath filename(1).name]);
lastFrame = imread([filepath filename(nFrames).name]);
middleFrame = imread([filepath filename(round(nFrames/2)).name]);

% Produced enhanced versions of the firstFrame, lastFrame, and middleFrame
% (for later use if required)
firstFrame_enh = firstFrame;
lastFrame_enh = lastFrame;
middleFrame_enh = middleFrame;
for j = 1:3
    firstFrame_enh(:,:,j) = histeq(firstFrame_enh(:,:,j),256);
    lastFrame_enh(:,:,j) = histeq(lastFrame_enh(:,:,j),256);
    middleFrame_enh(:,:,j) = histeq(middleFrame_enh(:,:,j),256);
end
clear j;
    
%% Input information about video and calculate resolution
disp(' ')
disp('PART 1: VIDEO FRAME SET INFORMATION')

confirm = 'n';
while ~strcmp(confirm,'y')
    clf; imshow(middleFrame); axis on; hold on;
    disp(' ')
    if ~exist('frameRate','var')
        % Input frame rate
        frameRate = input('Enter frame extraction rate (fps): ');
    end
    if ~exist('frameRes','var')
        % Calculate frame image resolution
        disp('Specify 4 corners of rectangular reference region used to')
        disp('calculate frame image resolution [px/mm]. CW from top left.')
        refrect = zeros(4,2);
        [refrect(:,1),refrect(:,2)] = ginput(4);    % 4 corners of rect.
        disp('Input real dimensions [mm] of rectangle as row vector')
        refrectsize = input('[width height]: ');
        refrect2 = zeros(4,2);  % refrect with rows rearranged
        refrect2(1:3,:) = refrect(2:4,:); refrect2(4,:) = refrect(1,:); 
        refrect2 = refrect - refrect2;
        refrectlengths = [norm(refrect2(1,:)), norm(refrect2(2,:)), ...
            norm(refrect2(3,:)), norm(refrect2(4,:))];
        % Calculate resolution by dividing circumferences: px/mm.
        frameRes = sum(refrectlengths(1:4)) / ( 2*sum(refrectsize) );
        refrectlengths(5) = norm(refrect(1,:) - refrect(3,:));
        refrectlengths(6) = norm(refrect(2,:) - refrect(4,:));
        clear refrect2;
    end
    
    % Show corners of reference rectangle
    for i = 1:4
        plot(refrect(i,1),refrect(i,2),'or');
    end
    hold off;
    
    % Display current values
    disp(['Current frame extraction rate: ' num2str(frameRate) ' fps'])
    disp(['Reference rectangle of real width x height = ' ...
        num2str(refrectsize(1)) ' x ' num2str(refrectsize(2)) ' [mm]'])
    disp('has measured pixel dimensions of top, right, bottom, left')
    disp('edges and TL-BR, TR-BL diagonals of:')
    disp(refrectlengths)
    disp(['Current frame image resolution: ' num2str(frameRes) ' [px/mm]'])
    confirm = input('Confirm these values (y/n): ','s');
    if ~strcmp(confirm,'y')   % Clear and redefine
        clear_frameRate = input('    Clear ''frameRate'' (y/n)?: ','s');
        if strcmp(clear_frameRate,'y')
            clear frameRate;
        end
        disp('    Clear ''frameRes'' and redefine graphically (g),')
        clear_frameRes = ...
            input('    numerically (n), or cancel (c)?: ','s');
        if strcmp(clear_frameRes,'g')
            clear frameRes refrectsize refrectlengths;
        elseif strcmp(clear_frameRes,'n')
            frameRes = input('        Enter frame resolution (px/mm): ');
            refrect = zeros(4,2); refrect(:) = NaN;
            refrectsize = [NaN NaN];
            refrectlengths = [NaN NaN NaN NaN NaN NaN];
        end
    end
end
clear i clear_frameRate clear_frameRes;
save([recipePath newRecipe]);   % Save changes

%% Get initial spot location for each track from first frame
disp(' ')
disp('PART 2: SPECIFY NUMBER AND INITIAL LOCATIONS OF SPOTS')

confirm = 'n';
while ~strcmp(confirm,'y')
    disp(' ')
    % Preview track locations if available
    clf; imshow(firstFrame_enh); axis on; hold on;
    if exist('defineTrack','var')
        for i = 1:size(defineTrack,1)
            % TOP left corner coordinates and dimensions (TOP because of
            % inverted image y-axis. Note: At this point, nTracks doesn't
            % need to equal size(defineTrack,1) )
            rectangle('Position', defineTrack(i,:));
        end
    end
    % Specify number of tracks
    if ~exist('nTracks','var') && exist('spotLocationX','var') && ...
            exist('spotLocationY','var') && ...
            (length(spotLocationX) == length(spotLocationY))
        nTracks = length(spotLocationY);
    elseif ~exist('nTracks','var') || ~exist('spotLocationX','var') || ...
            ~exist('spotLocationY','var')
        nTracks = input('Enter number of tracks that can be analyzed: ');
    end
    % Input spot locations if required
    if ~exist('spotLocationX','var') || ~exist('spotLocationY','var') ...
            || (length(spotLocationX) ~= length(spotLocationY))
        disp('Locate each initial spot position (left to right)')
        [spotLocationX,spotLocationY] = ginput(nTracks);
    end
    % Correct imported location vectors if required
    spotLocationTEST1 = [spotLocationX,spotLocationY];
    spotLocationTEST2 = sortrows(spotLocationTEST1,1);
    if max(max( spotLocationTEST1 ~= spotLocationTEST2 ))
        spotLocationX = spotLocationTEST2(:,1);
        spotLocationY = spotLocationTEST2(:,2);
        disp('''spotLocationX'' and ''spotLocationY'' vectors updated so ')
        disp('that all spot locations are sorted left to right.')
    end
    clear spotLocationTEST1 spotLocationTEST2;
    % Preview spot positions and track locations
    disp('Initial spot x,y positions (left to right) defined as [px]:')
    disp(' ')
    disp([spotLocationX spotLocationY])
    for i = 1:nTracks   % Add locations of each spot to figure
        plot(spotLocationX(i),spotLocationY(i),'or');
    end
    hold off;
    confirm = input('Confirm these values (y/n): ','s');
    % Redefine spot locations if necessary
    if ~strcmp(confirm,'y')
        redefineSpotLoc1 = ...
            input('    Redefine single/all spot locations (s/a)?: ','s');
    end
    % Clear all spot location definitions or edit single spot
    if exist('redefineSpotLoc1','var') && strcmp(redefineSpotLoc1,'a')
        clear spotLocationX spotLocationY;
        disp('    All existing spot location definitions deleted.')
    end
    % Edit single spots graphically or numerically
    while exist('redefineSpotLoc1','var') && ~strcmp(redefineSpotLoc1,'a')
        disp(' ')
        spotNum = input(['    Edit which spot (1 - ' ...
            num2str(nTracks) ')?: ']);
        redefineSpotLoc2 = ...
            input('    Redefine graphically or numerically (g/n)?: ','s');
        if strcmp(redefineSpotLoc2,'g')
            disp(['    Locate initial spot ' num2str(spotNum)])
            [spotLocationX0,spotLocationY0] = ginput(1);
            spotLocationX(spotNum) = spotLocationX0;
            spotLocationY(spotNum) = spotLocationY0;
            clear spotLocationX0 spotLocationY0;
        else
            clf; imshow(firstFrame); axis on; hold on;
            disp(['    Enter coordinates of spot ' num2str(spotNum)])
            editSpot = input ('    as a row vector ([x y] in px): ');
            spotLocationX(spotNum) = editSpot(1);
            spotLocationY(spotNum) = editSpot(2);
        end
        % Display editted coordiantes for given spot
        disp(['    Spot ' num2str(spotNum) ' has coordinates:'])
        disp(['    (x,y) = (' num2str(spotLocationX(spotNum)) ',' ...
            num2str(spotLocationY(spotNum)) ') px'])
        % Confirm and stop editing
        editSpot = ...
            input('    Continue editing spot locations (y/n)?: ','s');
        if strcmp(editSpot,'n')  % Exit while loop
            clear redefineSpotLoc*;
        end
    end
    clear redefineSpotLoc*;
end
clear confirm i spotNum redefineSpotLoc* editSpot;
save([recipePath newRecipe]);   % Save changes

%% Define track locations
disp(' ')
disp('PART 3: SPECIFY LOCATION OF TRACKS')

confirm = 'n';
while ~strcmp(confirm,'y')
    % Select (un)enhanced images for this section
    disp(' ')
    disp('Use enhanced versions of ''middleFrame'' and')
    enh = input('''lastFrame'' in this section (y/n)?: ','s');
    if strcmp(enh,'n')
        middleFrameDefTrk = middleFrame;
        lastFrameDefTrk = lastFrame;
    else
        middleFrameDefTrk = middleFrame_enh;
        lastFrameDefTrk = lastFrame_enh;
    end
    clf; imshow(lastFrameDefTrk); axis on;
    disp(' ')
    % Define all tracks if required
    if ~exist('defineTrack','var') || (size(defineTrack,1) ~= nTracks)
        disp('No valid ''defineTrack'' array exists.')
        disp(['Define locations of ' num2str(nTracks) ...
            ' tracks graphically or '])
        make_defineTrack = ...
            input('numerically (g/n)?: ','s');
        disp(' ')
        defineTrack = zeros(nTracks,4);
        if strcmp(make_defineTrack,'g')
            disp('Graphically: Crop track locations. (Height of crop ')
            disp('will be forced to the full frame height.)')
            figure(5);
            for trackNum = 1:nTracks
                disp(['    Crop location of track ' num2str(trackNum) ...
                    ' of ' num2str(nTracks)])
                clf;
                [~,defineTrack(trackNum,:)] = imcrop(lastFrameDefTrk);
            end
        else
            figure(5);  % Open a new window so that figures 1-4 remain
            clf; imshow(lastFrameDefTrk); axis on; hold on;
            disp('Numerically: Enter left/right corner x-positions of ')
            disp(['each track together as a ' num2str(nTracks) ...
                ' x 2 matrix. (One row '])
            disp('for each track, columns for left/right x-positions.)')
            editTrk = input ('Enter in px: ');
            editTrk = [min(editTrk,[],2) max(editTrk,[],2)]; % Proper order
            defineTrack(:,1) = editTrk(:,1);    % left corner
            defineTrack(:,3) = editTrk(:,2) - editTrk(:,1) + 1; % width
        end
        % Ensure defineTrack selects full height of frame
        defineTrack = round(defineTrack);
        defineTrack(:,2) = 0; defineTrack(:,4) = size(lastFrame,1);
    end
    clear make_defineTrack editTrk;
    
    % Correct imported defineTrack array if required
    defineTrackTEST = round(defineTrack);
    defineTrackTEST(:,2) = 0; defineTrackTEST(:,4) = size(lastFrame,1);
    defineTrackTEST = sortrows(defineTrackTEST,1);
    if max(max( defineTrack ~= defineTrackTEST ))
        defineTrack = defineTrackTEST;
        disp(' ')
        disp('''defineTrack'' array updated so that all entries are')
        disp('integers, all tracks span full height of frame, and ')
        disp('all tracks are sorted left to right.')
    end
    clear defineTrackTEST;

    % Preview track locations
    disp(' ')
    disp('Current ''defineTrack'' (rows for [x y width height] in px):')
    disp(' ')
    disp(defineTrack);
    
    disp('The left, right edge coordinates for the tracks are (px):')
    disp(' ')
    disp([defineTrack(:,1) (defineTrack(:,1) + defineTrack(:,3) - 1)])
    
    % Check to see if any tracks overlap.  Calculate gap between tracks 
    % and see if any are < 0.
    defineTrackRIGHT = (defineTrack(1:(nTracks-1),1) + ...
        defineTrack(1:(nTracks-1),3) - 1);
    defineTrackLEFT = defineTrack(2:nTracks,1);
    defineTrackGAP = defineTrackLEFT - defineTrackRIGHT;
    if max(defineTrackGAP < 0)
        disp('Warning: Some of the tracks overlap!')
    end
    clear defineTrackRIGHT defineTrackLEFT defineTrackGAP;
    
    % Preview on middle frame (unenhanced)
    figure(1); clf; imshow(middleFrame); axis on; hold on;
    for i = 1:nTracks       % Indicate location of each track
        % TOP left corner coordinates and dimensions (TOP because of
        % inverted image y-axis
        rectangle('Position', defineTrack(i,:));
        plot(spotLocationX(i),spotLocationY(i),'or');
    end
    title(['Separation tracks on (unenhanced) ''middleFrame'' ' ...
        '(Frame # ' num2str(round(nFrames/2)) '): ' ...
        filename(round(nFrames/2)).name]);
    hold off;
    
    % Preview on middle frame (enhanced)
    figure(2); clf; imshow(middleFrame_enh); axis on; hold on;
    for i = 1:nTracks       % Indicate location of each track
        % TOP left corner coordinates and dimensions (TOP because of
        % inverted image y-axis
        rectangle('Position', defineTrack(i,:));
        plot(spotLocationX(i),spotLocationY(i),'or');
    end
    title(['Separation tracks on (enhanced) ''middleFrame'' ' ...
        '(Frame # ' num2str(round(nFrames/2)) '): ' ...
        filename(round(nFrames/2)).name]);
    hold off;
    
    % Preview on last frame (unenhanced)
    figure(3); clf; imshow(lastFrame); axis on; hold on;
    for i = 1:nTracks       % Indicate location of each track
        % TOP left corner coordinates and dimensions (TOP because of
        % inverted image y-axis
        rectangle('Position', defineTrack(i,:));
        plot(spotLocationX(i),spotLocationY(i),'or');
    end
    title(['Separation tracks on (unenhanced) ''lastFrame'' (Frame # ' ...
        num2str(round(nFrames)) '): ' filename(nFrames).name]);
    hold off;
        
    % Preview on last frame (enhanced)
    figure(4); clf; imshow(lastFrame_enh); axis on; hold on;
    for i = 1:nTracks       % Indicate location of each track
        % TOP left corner coordinates and dimensions (TOP because of
        % inverted image y-axis
        rectangle('Position', defineTrack(i,:));
        plot(spotLocationX(i),spotLocationY(i),'or');
    end
    title(['Separation tracks on (enhanced) ''lastFrame'' (Frame # '  ...
        num2str(round(nFrames)) '): ' filename(nFrames).name]);
    hold off;
    
    % Confirm or edit track locations
    confirm = input('Confirm these track locations (y/n): ','s');
    if ~strcmp(confirm,'y')
        redefineTrack1 = ...
            input('    Redefine single/all tracks (s/a)?: ','s');
    end
    % Clear all track location definitions or edit single tracks
    if exist('redefineTrack1','var') && strcmp(redefineTrack1,'a')
        clear defineTrack;
        disp('    All existing separation track definitions deleted.')
    end
    % Edit single tracks graphically or numerically
    while exist('redefineTrack1','var') && ~strcmp(redefineTrack1,'a')
        disp(' ')
        trackNum = input(['    Edit which track (1 - ' ...
            num2str(nTracks) ')?: ']);
        redefineTrack2 = ...
            input('    Redefine graphically or numerically (g/n)?: ','s');
        if strcmp(redefineTrack2,'g')
            disp(['    Re-crop track ' num2str(trackNum)])
            figure(5);  % Open a new window so that figures 1-4 remain
            [~,defineTrack(trackNum,:)] = imcrop(lastFrameDefTrk);
        else
            figure(5);  % Open a new window so that figures 1-4 remain
            clf; imshow(lastFrameDefTrk); axis on; hold on;
            disp('    Enter left/right corner x-positions of ')
            editTrk = input (['    track ' num2str(trackNum) ...
                ' as a row vector ([left right] in px): ']);
            editTrk = [min(editTrk) max(editTrk)];  % Ensure propper order
            defineTrack(trackNum,1) = editTrk(1);
            defineTrack(trackNum,3) = editTrk(2) - editTrk(1) + 1;
        end
        % Ensure defineTrack selects full height of frame
        defineTrack = round(defineTrack);
        defineTrack(:,2) = 0; defineTrack(:,4) = size(lastFrame,1);
        
        % Confirm and stop editing
        editTrk = ...
            input('    Continue editing ''defineTrack'' (y/n)?: ','s');
        if strcmp(editTrk,'n')  % Exit while loop
            clear redefineTrack*;
            close 5;    % Close figure 5
        end
    end
    clear redefineTrack*;
end
close all;  % Close all figures (ie. figures 1-5)
clear confirm* i trackNum redefineTrack* editTrk;
clear enh middleFrameDefTrk lastFrameDefTrk;
save([recipePath newRecipe]);   % Save changes

%% Input Labels and Hue Ranges for each analyte
disp(' ')
disp('PART 4: ANALYTE SPOT LABELS AND HUE RANGES')

clf; imshow(middleFrame); axis on;
confirm = 'n';
while ~strcmp(confirm,'y')
    disp(' ')
    % Define hueRange array
    if ~exist('hueRange','var') || (min(hueRange(:)) < 0) || ...
            (max(hueRange(:)) > 360)
        nSpots = input('Enter number of analytes to analyze: ');
        hueRange = zeros(nSpots,2); % Intitialize hueRange array
        disp('Enter analyte hue ranges in decreasing hRF as row vectors')
        for i = 1:nSpots
            hueRange(i,:) = input(['    Enter hue range for ' ...
                'analyte ' num2str(i) ': ']);
        end
    end
    hueRange(hueRange == 360) = 0;  % Check for 360 == 0 discontinuity
    % Ensure that nSpots is defined.
    if ~exist('nSpots','var') || (size(hueRange,1) ~= nSpots)
        nSpots = size(hueRange,1);
        disp('(Re)define ''nSpots'' so that it is consistent with')
        disp(['input ''hueRange''.  nSpots = ' num2str(nSpots) '.'])
    end
    % Define hueRangeLabels
    if ~exist('hueRangeLabel','var') || (size(hueRangeLabel,1) ~= nSpots)
        % Display and label selected hueRange
        disp('Label analyte hue ranges')
        hueRangeLabel = cell(nSpots,1);
        for i = 1:nSpots
            templabel = input(['    Name analyte ' num2str(i) ...
                ': H = [' num2str(hueRange(i,1)) ':' ...
                num2str(hueRange(i,2)) ']: '],'s');
            hueRangeLabel{i} = [templabel ': H = [' ...
                num2str(hueRange(i,1)) ':' num2str(hueRange(i,2)) ']'];
        end
    end
    % Display current hueRange and hueRangeLabel and confirm
    disp('Current analyte hue ranges in descending hRF are:')
    disp(hueRange)
    disp('Current analyte hue ranges labels in descending hRF are:')
    disp(hueRangeLabel)
    confirm = input('Confirm these values (y/n): ','s');
    if ~strcmp(confirm,'y')   % Clear hueRange and/or hueRangeLabel
        redohueRange = input(['    Redefine ''hueRange'' and ' ...
            '''hueRangeLabel'' (y/n)?: '],'s');
        redohueRangeLabel = ...
            input('    Redefine only ''hueRangeLabel'' (y/n)?: ','s');
        if ~strcmp(redohueRange,'n')
            clear hueRange hueRangeLabel;
        end
        if ~strcmp(redohueRangeLabel,'n')
            clear hueRangeLabel;
        end
        if strcmp(redohueRange,'n') && strcmp(redohueRangeLabel,'n')
            disp('    No values changed.  Current values confirmed.')
            confirm = 'y';
        end
    end
    clear templabel redohueRange redohueRangeLabel;
end
save([recipePath newRecipe]);   % Save changes

%% Information about spot loading
disp(' ')
disp('PART 5: SPOT LOADING INFORMATION')

% Undiluted dye concentration
disp(' ')
disp('PART 5A: Undiluted dye concentration')
disp([num2str(nSpots) ' analyte hue filters are defined.'])
if exist('undilutedConc','var')  % Check to see if appropriate data exists
    disp([num2str(length(undilutedConc)) ' undiluted concentrations ' ...
        'have been defined.'])
end
goPart = input('Proceed with this part (y/n)?: ','s');
confirm = 'n';
while ~strcmp(confirm,'y') && strcmp(goPart,'y')
    disp(' ')
    if ~exist('undilutedConc','var')
        disp('Specify the concentraton of each analyte for which a hue')
        disp('filter was defined. Define as a row vector of form')
        undilutedConc = input('[conc1 conc2 conc3...] in units [g/L]: ');
        undilutedConc = undilutedConc'; % Convert to column vector
    else
        disp('Current undiluted analyte conc. [g/L] is (descending hRF):')
        disp(undilutedConc)
    end
    if length(undilutedConc) == nSpots
        confirm = input('Confirm these values (y/n): ','s');
    end
    if ~strcmp(confirm,'y')   % Clear hueRange and redefine
        clear undilutedConc;
    end
end
clear goPart;
save([recipePath newRecipe]);   % Save changes

% Applied spot mass loading
disp(' ')
disp('PART 5B: Applied spot mass loading')
disp([num2str(nTracks) ' separation tracks/ initial spots were defined.'])
if exist('spotVolume','var')    % Check to see if appropriate data exists
    disp([num2str(length(spotVolume)) ' applied spot volumes '  ...
        'have been defined.'])
end
disp(' ')
goPart = input('Proceed with this part (y/n)?: ','s');
confirm = 'n';
while ~strcmp(confirm,'y') && strcmp(goPart,'y')
    disp(' ')
    if ( ~exist('relConc','var') || ~exist('spotVolume','var') || ...
            ~exist('spotMass','var') )
        relConc = input(['Specify relative sample concentration ' ...
            '([0,1]; ex. 0.2 for 20%): ']);
        % Applied spot volumes (row vector: 1 x nTracks)
        disp('Specify applied spot volumes [nL] for each track')
        spotVolume = input('Enter row vector: ');
    else
        disp(['The relative sample concentration is: ' num2str(relConc)])
        disp('The applied spot volumes are (left to right):')
        disp(spotVolume)
    end
    if exist('undilutedConc','var')
        spotMass = relConc .* (undilutedConc * spotVolume);
        disp('Calculated mass [ng] of each analyte (rows) in spot')
        disp('applied to each track (columns):')
        disp(spotMass)
    else
        disp('Mass of each analyte in spots applied to each track')
        disp('cannot be calculated because ''undilutedConc'' does not')
        disp('exist.  Forced to skip mass calculation.')
        clear spotMass;
    end
    confirm = input('Confirm these values (y/n): ','s');
    if ~strcmp(confirm,'y')   % Clear hueRange and redefine
        clear relConc spotVolume spotMass;
    end
end
clear goPart;
save([recipePath newRecipe]);   % Save changes

%% Fit mobile phase migration front
disp(' ')
disp('PART 6: MOBILE PHASE FRONT FIT CALCULATION')

confirm = 'n'; close all;
while ~strcmp(confirm,'y')
    measurechi = '';    % Initialize
    if ~exist('chi','var') || ~exist('zStart','var') || ...
            ~exist('tStart','var')
        % Specify a global chi value or individual values for each track
        disp(' ')
        disp('No valid ''chi'', ''zStart'', and/or ''tStart'' present.')
        disp('Fit a separate migration front for each track (s),')
        disp('fit a single global migration front for all tracks (g),')
        measurechi = input(['or define manually as a 1 x nTracks row ' ...
            'vectors (d): '],'s');
        % Initialize fitted parameters
        chi = zeros(1,nTracks);
        zStart = zeros(1,nTracks);
        tStart = zeros(1,nTracks);
        nMeasure = 10;  % Number of measurements used during fitting
        % Initialize arrays to store measurements
        zf_measure = zeros(nMeasure,nTracks);
        zf_measure_time = zeros(nMeasure,nTracks);
    end
    
    % Guess at zStart used when fitting with frontFit.m
    if strcmp(measurechi,'d')
        zStart_Guess = NaN;     % No guess required
    elseif (sum(chi(:)) == 0) && (sum(zStart(:)) == 0) && ...
            (sum(tStart(:)) == 0)   % If just initialized above
        confirm_zStart_Guess = 'n';
        while ~strcmp(confirm_zStart_Guess,'y')
            % Show image of plate from which guess at zStart can be made
            clf; imshow(middleFrame); axis on; hold on;
            % Input guess at zStart
            disp(' ')
            disp('Define a guess for ''zStart'': The point at which the')
            disp('mobile phase enters the layer (ex. edge of glass frit).')
            disp('Important:  Fitting is very sensitive to this guess.')
            disp('    Define manually (m) or with graphical')
            define_zStart_Guess = input('    interface (g): ','s');
            % Set zStart_Guess
            if strcmp(define_zStart_Guess,'g')
                disp('    Use cursors to guess at ''zStart'' position')
                disp('    (ignore x-cursor).')
                [~,zStart_Guess] = ginput(1);
            else
                disp('    Manually enter a pixel position to guess ')
                zStart_Guess = input('    at ''zStart'' (px): ');
            end
            if isscalar(zStart_Guess)
                zStart_Guess = round(zStart_Guess);
                disp(['    Pixel position of ' ...
                    num2str(zStart_Guess) ' px defined.'])
                disp('    Preview defined position/bound as green line.')
                plot([1;size(middleFrame,2)],...  % Add green line
                [zStart_Guess;zStart_Guess],'-g');
                hold off;
                confirm_zStart_Guess = ...
                    input('    Confirm this guess (y/n): ','s');
            else
                disp('    Invalid guess.  Guess at ''zStart'' must be')
                disp('    a scalar.  Guess again!')
            end
        end
        clear confirm_zStart_Guess;
        close all;
    end

    % Measure separate chi values
    if strcmp(measurechi,'s')
        for i = 1:nTracks
            disp(' ')
            disp(['Measure ''chi''/''zStart''/''tStart'' value on ' ...
                'track ' num2str(i) ' of ' num2str(nTracks)])
            % Define ROIs using defineTrack values
            chitrkloc = defineTrack(i,:);
            % A region 3 times wider centred on chitrkloc
            chitrkloc2 = chitrkloc;
            chitrkloc2(1) = chitrkloc(1)-chitrkloc(3)-1;
            chitrkloc2(3) = 3*chitrkloc(3);
            % Perform measurements
            confirm2 = 'n';
            while ~strcmp(confirm2,'y')
                disp(' ')
                % Fit values. Use spotLocationY(i) as the guess at the
                % zStart and 1 as guess at tStart
                [chi(i),zStart(i),tStart(i),...
                    zf_measure(:,i),zf_measure_time(:,i)] = frontFit(...
                    filepath,filename,zStart_Guess,frameRate,...
                    chitrkloc,chitrkloc2,nMeasure,'y');
                % Display measured values
                disp(['    Fitted ''chi''/''zStart''/''tStart'' ' ...
                    'values returned:'])
                disp(['        chi = ' num2str(chi(i)) ' [px^2/frame]'])
                disp(['        zStart = ' num2str(zStart(i)) ' [px]'])
                disp(['        tStart = ' num2str(tStart(i)) ' [frames]'])
                % Error-check fitted values and itterate if necessary
                % (allow negative zStart(i) values if immersion line is off
                % of frame
                goodchifit = isreal(chi(i)) && (chi(i) > 0) && ...
                    isreal(zStart(i)) && isreal(tStart(i)) && ...
                    (tStart(i) >= 0);
                % Confirm if valid
                if goodchifit
                    confirm2 = ...
                        input('    Confirm fitted values (y/n)?: ','s');
                else
                    disp('    Bad fitted values!  Accept (a), invalidate')
                    acceptbadchi = input(['    as ''NaN'' (nan), or ' ...
                        'remeasure (r)?: '],'s');
                    if strcmp(acceptbadchi,'a')
                        confirm2 = 'y';
                    elseif strcmp(acceptbadchi,'nan')
                        chi(i) = NaN; zStart(i) = NaN; tStart(i) = NaN;
                        confirm2 = 'y';
                    end
                end
            end
            save([recipePath newRecipe]);   % Save changes
        end
        clear i chitrkloc goodchifit acceptbadchi confirm2;
    end
    
    % Measure a single chi and make global
    if strcmp(measurechi,'g')
        disp(' ')
        disp('Measure a set of ''chi''/''zStart''/''tStart'' values')
        disp('and apply globally to all tracks.')
        % Perform measurements
        confirm2 = 'n';
        while ~strcmp(confirm2,'y')
            disp(' ')
            % Fit values. Keeping arguments 5/6 empty forces frontFit.m to
            % manually define ROI; empty argument 8 enables option to
            % have/skip image enhancement
            [chi(1),zStart(1),tStart(1),...
                zf_measure(:,1),zf_measure_time(:,1)] = frontFit(...
                filepath,filename,zStart_Guess,frameRate,'','',...
                nMeasure,'');
            % Display measured values
            disp(['    Fitted ''chi''/''zStart''/''tStart'' ' ...
                'values returned:'])
            disp(['        chi = ' num2str(chi(1)) ' [px^2/frame]'])
            disp(['        zStart = ' num2str(zStart(1)) ' [px]'])
            disp(['        tStart = ' num2str(tStart(1)) ' [frames]'])
            % Error-check fitted values and itterate if necessary
            goodchifit = isreal(chi(1)) && (chi(1) > 0) && ...
                isreal(zStart(1)) && isreal(tStart(1)) && (tStart(1) >= 0);
            % Confirm if valid
            if goodchifit
                confirm2 = ...
                    input('    Confirm fitted values (y/n)?: ','s');
            else
                disp('    Bad fitted values!  Accept (a), invalidate')
                acceptbadchi = input(['    as ''NaN'' (nan), or ' ...
                    'remeasure (r)?: '],'s');
                if strcmp(acceptbadchi,'a')
                    confirm2 = 'y';
                elseif strcmp(acceptbadchi,'nan')
                    chi(1) = NaN; zStart(1) = NaN; tStart(1) = NaN;
                    confirm2 = 'y';
                end
            end
        end
        % Apply fitted values to all tracks
        chi(:) = chi(1); zStart(:) = zStart(1); tStart(:) = tStart(1);
        zf_measure = repmat(zf_measure(:,1),1,nTracks);
        zf_measure_time = repmat(zf_measure_time(:,1),1,nTracks);
        clear goodchifit acceptbadchi confirm2;
        save([recipePath newRecipe]);   % Save changes
    end
    
    % Manually enter values
    if strcmp(measurechi,'d')
        disp(' ')
        disp(['Define ''chi'', ''zStart'', and ''tStart'' as 1 x ' ...
            num2str(nTracks) ' row vectors'])
        chi = input('Enter ''chi'' row vector: ');
        zStart = input('Enter ''zStart'' row vector: ');
        tStart = input('Enter ''tStart'' row vector: ');
        % No measurements taken
        zf_measure(:) = NaN; zf_measure_time(:) = NaN; zStart_Guess = NaN;
        save([recipePath newRecipe]);   % Save changes
    end
    
    % Ensure values are valid, display, plot, and confirm
    disp(' ')
    if (length(chi) == nTracks) && (length(zStart) == nTracks) && ...
            (length(tStart) == nTracks)
        
        % Display current values
        disp('Current ''chi'' (in [px^2/frames]), ''zStart'' (in [px]), ')
        disp('and ''tStart'' (in [frames]) values.  (Row vectors ')
        disp('transposed into columns so that each row in below array ')
        disp('represents one track):')
        disp(' ')
        disp([chi' zStart' tStart'])
        
        % Calculate absolute front position (wrt top of frame)
        % nFrames x nTracks array of front positions
        % zf0 = sqrt( chi * (time + tStart) ) + zStart;
        zf0 = sqrt( repmat(chi,nFrames,1) .* ...
            ( repmat(((1:nFrames)' - 1),1,nTracks) + ...
            repmat(tStart,nFrames,1) ) );
        zf0 = zf0 + repmat(zStart,nFrames,1);
        
        % Ensure zf0 is real
        if ~isreal(zf0)
            disp('Excluding values of ''zf0'' that are not real.')
            disp('(ie. Setting imaginary/complex values to NaN.)')
            zf0(zf0 ~= real(zf0)) = NaN; 
        end
        
        % Verify graphically
        close all;
        disp(' ')
        disp('Assess all migration front fitted curves graphically.')
        figure(5); clf;
        strlegend = cell(nTracks,1);    % Temporary legend
        for i = 1:nTracks
            strlegend{i} = ['Track ' num2str(i)];
        end
        hold on;
        plot(zf_measure_time,zf_measure,'x');   % Measurements
        legend(strlegend);
        plot(repmat((1:nFrames)',1,nTracks),zf0,'-');   % Fitted equations
        title('(Fitted) migration front fits versus measurements');
        xlabel('Frames'); ylabel('Front position wrt top of frame  [px]')
        hold off;
        
        % Display overlaid migraton front positions on middle and last
        % frames
        disp(' ')
        disp('Preview MIDDLE and LAST frames with overlaid front ')
        disp('definitions. Bottom edge of rectangle represents front.')
        frontpos = defineTrack; % x,y,width,(height) of tracks
        
        % Overlaid on middle frame (unenhanced)
        figure(1);
        clf; imshow(middleFrame); axis on; hold on; % Show middle frame
        frontpos(:,4) = zf0(round(nFrames/2),:)';   % Height is front pos.
        if max(frontpos(:,4) <= 0)  % Check if invalid zf0 calculated.
            frontpos((frontpos(:,4) <= 0),4) = 1;   % Dummy value!
            disp('    Calculated zf0 invalid. Displaying "dummy" fronts.')
        end
        for i = 1:nTracks
            rectangle('Position', frontpos(i,:),'EdgeColor','r');
        end
        hold off;
        title(['Migration fronts on (unenhanced) ''middleFrame'' ' ...
            '(Frame # ' num2str(round(nFrames/2)) '): ' ...
            filename(round(nFrames/2)).name]);
        
        % Overlaid on middle frame (enhanced)
        figure(2);
        clf; imshow(middleFrame_enh); axis on; hold on; % Show middle frame
        frontpos(:,4) = zf0(round(nFrames/2),:)';   % Height is front pos.
        if max(frontpos(:,4) <= 0)  % Check if invalid zf0 calculated.
            frontpos((frontpos(:,4) <= 0),4) = 1;   % Dummy value!
            disp('    Calculated zf0 invalid. Displaying "dummy" fronts.')
        end
        for i = 1:nTracks
            rectangle('Position', frontpos(i,:),'EdgeColor','r');
        end
        hold off;
        title(['Migration fronts on (enhanced) ''middleFrame'' ' ...
            '(Frame # ' num2str(round(nFrames/2)) '): ' ...
            filename(round(nFrames/2)).name]);
        
        % Overlaid on last frame (unenhanced)
        figure(3);
        clf; imshow(lastFrame); axis on; hold on;   % Show last frame
        frontpos(:,4) = zf0(nFrames,:)';            % Height is front pos.
        if max(frontpos(:,4) <= 0)  % Check if invalid zf0 calculated.
            frontpos((frontpos(:,4) <= 0),4) = 1;   % Dummy value!
            disp('    Calculated zf0 invalid. Displaying "dummy" fronts.')
        end
        for i = 1:nTracks
            rectangle('Position', frontpos(i,:),'EdgeColor','r');
        end
        hold off;
        title(['Migration fronts on (unenhanced) ''lastFrame'' ' ...
            '(Frame # ' num2str(nFrames) '): ' filename(nFrames).name]);
        
        % Overlaid on last frame (enhanced)
        figure(4);
        clf; imshow(lastFrame_enh); axis on; hold on;   % Show last frame
        frontpos(:,4) = zf0(nFrames,:)';            % Height is front pos.
        if max(frontpos(:,4) <= 0)  % Check if invalid zf0 calculated.
            frontpos((frontpos(:,4) <= 0),4) = 1;   % Dummy value!
            disp('    Calculated zf0 invalid. Displaying "dummy" fronts.')
        end
        for i = 1:nTracks
            rectangle('Position', frontpos(i,:),'EdgeColor','r');
        end
        hold off;
        title(['Migration fronts on (enhanced) ''lastFrame'' ' ...
            '(Frame # ' num2str(nFrames) '): ' filename(nFrames).name]);
        
        clear i frontpos strlegend;
        
        % Confirm fits?
        confirm = input('Confirm these migration fits (y/n): ','s');
    else
        disp('Current chi fit data is insufficent/invalid for ')
        disp([num2str(nTracks) ' tracks.  Redefine.'])
        disp(' ')
    end
    
    % Edit or clear chi values?
    if ~strcmp(confirm,'y')
        disp('    Redefine migration fronts for single/all')
        redefineChi1 = ...
            input('    separation tracks or cancel (s/a/c)?: ','s');
        close all;
    end
    
    % Clear all track location definitions or edit single tracks
    if exist('redefineChi1','var') && strcmp(redefineChi1,'a')
        clear chi zStart tStart;
        disp('    All existing migration front definitions deleted.')
    end
    
    % Edit single tracks graphically or numerically
    while exist('redefineChi1','var') && strcmp(redefineChi1,'s')
        disp(' ')
        i = input(['    Edit which track (1 - ' ...
            num2str(nTracks) ')?: ']);
        redefineChi2 = ...
            input(['    Remeasure, enter manually, or cancel ' ...
            '(r/m/c)?: '],'s');
        if strcmp(redefineChi2,'r')
            disp(' ')
            disp(['    Remeasure values on track ' num2str(i) ...
                ' of ' num2str(nTracks)])
            % Perform measurements
            confirm2 = 'n';
            while ~strcmp(confirm2,'y')
                disp(' ')
                % Edit zStart_Guess
                disp(['    Current ''zStart_Guess'' = ' ...
                    num2str(zStart_Guess) ' [px].'])
                redefine_zStart_Guess = ...
                    input('    Use this value (y/n)?: ','s');
                if strcmp(redefine_zStart_Guess,'n')
                    confirm3 = 'n';
                    while ~strcmp(confirm3,'y')
                        zStart_Guess = ...
                            input('        New ''zStart_Guess'' [px]: ');
                        confirm3 = input(['        Confirm this ' ...
                            'value (y/n)?: '],'s');
                    end
                end
                clear redefine_zStart_Guess;
                % Fit values
                chitrkloc = defineTrack(i,:);   % Track location used
                nMeasure = size(zf_measure,1);  % Make nMeasure consistent
                % A region 3 times wider centred on chitrkloc
                chitrkloc2 = chitrkloc;
                chitrkloc2(1) = chitrkloc(1)-chitrkloc(3)-1;
                chitrkloc2(3) = 3*chitrkloc(3);
                % Image enhancement is optional
                [chi(i),zStart(i),tStart(i),...
                    zf_measure(:,i),zf_measure_time(:,i)] = frontFit(...
                    filepath,filename,zStart_Guess,frameRate,...
                    chitrkloc,chitrkloc2,nMeasure,'');
                % Display measured values
                disp(['    Fitted ''chi''/''zStart''/''tStart'' ' ...
                    'values returned:'])
                disp(['        chi = ' num2str(chi(i)) ' [px^2/frame]'])
                disp(['        zStart = ' num2str(zStart(i)) ' [px]'])
                disp(['        tStart = ' num2str(tStart(i)) ' [frames]'])
                % Error-check fitted values and itterate if necessary
                % (allow negative zStart(i) values if immersion line is off
                % of frame
                goodchifit = isreal(chi(i)) && (chi(i) > 0) && ...
                    isreal(zStart(i)) && isreal(tStart(i)) && ...
                    (tStart(i) >= 0);
                % Confirm if valid
                if goodchifit
                    confirm2 = ...
                        input('    Confirm fitted values (y/n)?: ','s');
                else
                    disp('    Bad fitted values!  Accept (a), invalidate')
                    acceptbadchi = input(['    as ''NaN'' (nan), or ' ...
                        'remeasure (r)?: '],'s');
                    if strcmp(acceptbadchi,'a')
                        confirm2 = 'y';
                    elseif strcmp(acceptbadchi,'nan')
                        chi(i) = NaN; zStart(i) = NaN; tStart(i) = NaN;
                        confirm2 = 'y';
                    end
                end
            end
            clear chitrkloc goodchifit acceptbadchi confirm2;
        elseif strcmp(redefineChi2,'m')
            disp(' ')
            disp(['    Enter values for track ' num2str(i) ...
                ' of ' num2str(nTracks)])
            chiEnter = ...
                input('    as a [chi zStart tstart] row vector: ');
            chi(i) = chiEnter(1);
            zStart(i) = chiEnter(2);
            tStart(i) = chiEnter(3);
            % No measurements taken
            zf_measure(:,i) = NaN;
            zf_measure_time(:,i)= NaN;
            clear chiEnter;
        end
        save([recipePath newRecipe]);   % Save changes
        % Confirm and stop editing
        disp(' ')
        editTrk = ...
            input('    Continue editing migraton fits (y/n)?: ','s');
        if strcmp(editTrk,'n')  % Exit while loop
            clear redefineChi*;
        end
        clear editTrk;
    end
    clear redefineChi*;
end
clear chitrkloc* goodchifit acceptbadchi redefineChi*;
clear i editTrk measurechi nMeasure chiEnter confirm*;
clear define_zStart_Guess

% Calculate migraton front distance wrt applied spot location
zf = zf0 - repmat(spotLocationY',nFrames,1);
disp('Calculated migration front positions for all tracks in all frames.')

save([recipePath newRecipe]);   % Save changes

%% Initial guesses and fitting limits for peaks (formerly getHRf.m)
disp(' ')
disp('PART 7: INITIAL GUESSES/WINDOWS OF ANALYTE RF VALUES')

confirm = 'n'; close all;
while ~strcmp(confirm,'y')
    disp(' ')
    measureRF = '';    % Initialize
    if ~exist('rfGuess','var')
        % Specify a global chi value or individual values for each track
        disp('No valid ''rfGuess'' values present.')
        disp('Measure separate ''rfGuess'' values for each track (s),')
        disp('Measure global ''rfGuess'' values for all tracks (g),')
        measureRF = input(['or define manually as a 1 x nSpots ' ...
            'row vector (d): '],'s');
        % Initialize values
        rfGuess = zeros(nSpots,nTracks);    % Guesses on RF values
        disp(' ')
    end
    if strcmp(measureRF,'s')   % Measure separate RF values
        disp('Click on the (approximate) positions of each spot on each')
        disp('track.  Identify spots on tracks from left to right in')
        disp('descending RF value.  If unable to identify, click well')
        disp('above initial applied spot location to skip.')
        for i = 1:nTracks
            % Show middle frame with overlaid track and applied spot
            clf; imshow(middleFrame_enh); axis on; hold on;
            rectangle('Position', defineTrack(i,:));
            plot(spotLocationX(i),spotLocationY(i),'or'); hold off;
            disp(['Measure ''rfGuess'' values on track ' ...
                num2str(i) ' of ' num2str(nTracks)])
            [~,rfGuessTEMP] = ginput(nSpots);
            % Negate spots with locations that cannot be identified
            % (selection is < 80% of the spotLocationY value.
            rfGuessTEMP(rfGuessTEMP < 0.8*spotLocationY(i)) = NaN;
            rfGuess(:,i) = (rfGuessTEMP - spotLocationY(i)) / ...
                zf(round(nFrames/2),i); % Estimate RFs for middle frame
        end
        % Assume that an analyte spot that could not be identified has a
        % similar RF value to other spots of the same analyte
        for j = 1:nSpots
            rfGuessTEMP = rfGuess(j,:);
            if (max(isnan(rfGuessTEMP)) == 1) && ...
                    (min(isnan(rfGuessTEMP)) == 0)
                rfGuessTEMP(isnan(rfGuessTEMP)) = ...
                    mean(rfGuessTEMP(~isnan(rfGuessTEMP)));
                disp(['Adjusted "unknown" values for spot ' num2str(j) ...
                    ' of ' num2str(nSpots)])
            elseif (max(isnan(rfGuessTEMP)) == 1) && ...
                    (min(isnan(rfGuessTEMP)) == 1)
                disp(['No valid ''rfGuess'' value for spot' num2str(j)])
                rfGuessTEMP(1,:) = input(['on any tracks!  Manually ' ...
                    'define for all tracks [0,1]: ']);
            end
            rfGuess(j,:) = rfGuessTEMP;
        end
    end    
    if strcmp(measureRF,'g')   % Measure RF values and make global
        clf; imshow(middleFrame_enh); axis on; hold on;
        disp('On which track should RF guess be based?  Pick a track')
        picktrk = input(['number from 1-' num2str(nTracks) ': ']);
        % Preview track and applied spot location
        rectangle('Position', defineTrack(picktrk,:));
        plot(spotLocationX(picktrk),spotLocationY(picktrk),'or'); hold off;
        % Select spot locations
        disp('Click on the (approximate) positions of each spot on ')
        disp('this track.  Identify spots in descenting RF value.')
        [~,rfGuessTEMP] = ginput(nSpots);
        rfGuessTEMP = (rfGuessTEMP - spotLocationY(picktrk)) / ...
            zf(round(nFrames/2),picktrk);  % Estimate RF for middle frame
        rfGuess = repmat(rfGuessTEMP,1,nTracks);
    end 
    if strcmp(measureRF,'d') % Define RF values manually
        disp(['Specify ''rfGuess'' values for ' num2str(nSpots) ...
            ' analyte in descending'])
        rfGuessTEMP = input(['RF value as a row vector with elements ' ...
            '[0,1]: '])';
        rfGuess = repmat(rfGuessTEMP,1,nTracks);
    end
    
    % Display current values
    disp('Current ''rfGuess'' values (one analyte per row, ')
    disp('one track per column) are:')
    disp(rfGuess)
    
    % Use rfGuess to set limits of possible spot RF values (rfLimits)
    % Repeat criteria used by A.J. Oko in getHRf.m to set limits
    rfLimits = zeros([size(rfGuess) 2]);
    rfLimitsLOW = rfGuess; rfLimitsUP = rfGuess;
    rfLimitsLOW(rfGuess < 0.05) = rfGuess(rfGuess < 0.05) - 0.03;
    rfLimitsLOW(rfGuess >= 0.05) = rfGuess(rfGuess >= 0.05) / 1.3;
    rfLimitsUP(rfGuess < 0.05) = rfGuess(rfGuess < 0.05) + 0.03;
    rfLimitsUP(rfGuess >= 0.05) = rfGuess(rfGuess >= 0.05) * 1.3;
    rfLimits(:,:,1) = rfLimitsLOW; rfLimits(:,:,2) = rfLimitsUP;
    disp('''rfLimits'' calculated from above based on A.J. Oko criteria')

    % Ensure values are valid and confirm
    disp(' ')
    good_rfGuess = min(~isnan(rfGuess(:))) && ...
        (min(rfGuess(:)) >= -0.1) && (max(rfGuess(:)) <= 1.1) && ...
        (size(rfGuess,1) == nSpots) && (size(rfGuess,2) == nTracks);
    good_rfLimits = (size(rfLimits,1) == nSpots) && ...
        (size(rfLimits,2) == nTracks) && (size(rfLimits,3) == 2) && ...
        min(~isnan(rfLimits(:)));
    if good_rfGuess && good_rfLimits
        confirm = input('Confirm these values (y/n): ','s');
    else
        disp('''rfGuess'' and ''rfLimits'' values are invalid. Redefine.')
    end
    clear measureRF picktrk rfGuessTEMP rfLimitsLOW rfLimitsUP;
    clear good_rfGuess good_rfLimits;
    if ~strcmp(confirm,'y')   % Clear and redefine
        clear rfGuess rfLimits;
    end
end
save([recipePath newRecipe]);   % Save changes

%% Define blank regions on frames
disp(' ')
disp('PART 8: DEFINE BLANK REGIONS FOR SYSTEM NOISE CALCULATION')

confirm = 'n'; close all;
while ~strcmp(confirm,'y')
    disp(' ')
    
    % Blank tracks between separation tracks
    disp(['Defining "blank" regions between ' num2str(nTracks) ...
        ' separation'])
    disp('tracks to use in system noise calculation (''defineBlank'')')
    
    % Set the top left corner position and width of blank track
    disp('Calculating position and width of blank regions.')
    if nTracks > 1
        defineBlank0 = zeros((nTracks-1),4); % 2D array to be tiled into 3D
        defineBlank0(:,1) = defineTrack(1:(nTracks-1),1) + ...
            defineTrack(1:(nTracks-1),3) + 1;
        defineBlank0(:,2) = ceil( max(spotLocationY(1:(nTracks-1)), ...
            spotLocationY(2:nTracks)) );
        defineBlank0(:,3) = defineTrack(2:nTracks,1) - ...
            defineBlank0(:,1) - 1;
    else
        % Create a dummy blank track as the left column of pixels
        defineBlank0 = [1 0 1 size(lastFrame,1)];
    end
    defineBlank = repmat(defineBlank0,[1 1 nFrames]);
    
    % Set bottom edge of crops manually or float with migration front
    % (within manual/automatic bounds)
    disp(' ')
    disp('Manually/automatically set height of blank regions')
    confirm2 = 'n';
    while ~strcmp(confirm2,'y')
        clf; imshow(middleFrame); axis on; hold on; % Show middle frame
        disp(' ')
        % Check existing variables if present
        good_LowEdge = exist('defineBlank_LowEdge','var') && ... 
            isnumeric(defineBlank_LowEdge) &&  ...
            (defineBlank_LowEdge > max(defineBlank0(:,2))) && ...
            (defineBlank_LowEdge <= size(middleFrame,1));
        good_LowEdgeType = exist('defineBlank_LowEdgeType','var') && ...
            ( strcmp(defineBlank_LowEdgeType,'fix') || ...
            strcmp(defineBlank_LowEdgeType,'front') );
        % Define new values if not present or invalid
        if ~good_LowEdge || ~good_LowEdgeType
            % How should bottom edge of blank regions be defined
            disp('    How should the lower edge of the blank regions')
            disp('    be defined?  Fix to a given position ("fix") or')
            disp('    move with migraton front ("front") up to a set')
            defineBlank_LowEdgeType = ...
                input('    bound (type without quotes): ','s');
            disp('    Define this manually (m) or with graphical')
            defineBlank_LowEdge = input('    interface (g): ','s');
            % What value should be the edge/bound on the blank regions
            if strcmp(defineBlank_LowEdge,'g')
                disp('    Use cursors to define blank region lower edge')
                disp('    position or bound (ignore x-cursor).')
                [~,defineBlank_LowEdge] = ginput(1);
                defineBlank_LowEdge = floor(defineBlank_LowEdge);
                disp(['    Pixel position of ' ...
                    num2str(defineBlank_LowEdge) ' defined.'])
            else
                disp('    Manually enter a pixel position/bound between')
                disp('    starting position and bottom of frame.  ie.')
                defineBlank_LowEdge = input(['    between ' ...
                    num2str(max(defineBlank0(:,2))) ' and ' ...
                    num2str(size(middleFrame,1)) ' px.: ']);
            end
            % Error correct if necessary
            if defineBlank_LowEdge <= max(defineBlank0(:,2))
                defineBlank_LowEdge = max(defineBlank0(:,2)) + 1;
            elseif defineBlank_LowEdge > size(middleFrame,1)
                defineBlank_LowEdge = size(middleFrame,1);
            end
            disp(' ')
        end
        % Display current blank edge calculation method and position
        if strcmp(defineBlank_LowEdgeType,'fix')
            disp('    Bottom edge of blank regions will be FIXED to')
            disp(['    a position ' num2str(defineBlank_LowEdge) ...
                ' px from top of frame.'])
        else
            disp('    Bottom edge of blank regions will MOVE WITH ')
            disp('    MIGRATION FRONT to a maxiumum position of ')
            disp(['    ' num2str(defineBlank_LowEdge) ...
                ' px from top of frame.'])
        end
        % Check current variables
        good_LowEdge = exist('defineBlank_LowEdge','var') && ... 
            isnumeric(defineBlank_LowEdge) &&  ...
            (defineBlank_LowEdge > max(defineBlank0(:,2))) && ...
            (defineBlank_LowEdge <= size(middleFrame,1));
        good_LowEdgeType = exist('defineBlank_LowEdgeType','var') && ...
            ( strcmp(defineBlank_LowEdgeType,'fix') || ...
            strcmp(defineBlank_LowEdgeType,'front') );
        disp(' ')
        if good_LowEdge && good_LowEdgeType
            disp('    Preview defined position/bound as green line.')
            plot([1;size(middleFrame,2)],...  % Add green line
                [defineBlank_LowEdge;defineBlank_LowEdge],'-g');
            hold off;
            confirm2 = input('    Confirm these values (y/n): ','s');
        else
            disp('    Redefine: ''defineBlank_LowEdge'' and/or')
            disp('    ''defineBlank_LowEdgeType'' variables are invalid.')
        end
        if ~strcmp(confirm2,'y')
            clear defineBlank_LowEdge defineBlank_LowEdgeType;
        end
        clear good_LowEdge good_LowEdgeType;
    end
    clear confirm2;
    
    % Calculate defineBlankHeight required to satisfy the above
    if nTracks > 1
        defineBlankHeight = defineBlank_LowEdge .* ...
            ones(size(zf0,1),(nTracks-1));  % Fixed position wrt frame top
        if ~strcmp(defineBlank_LowEdgeType,'fix')   % Moving lower edge
            % Set lower edge of blank using front position wrt top of frame
            % Consider front of track to left then to right of blank region
            defineBlankHeight = ...
                min( defineBlankHeight, zf0(:,1:(nTracks-1)) );
            defineBlankHeight = min( defineBlankHeight, zf0(:,2:nTracks) );
        end
    end
    defineBlankHeight = defineBlankHeight - ... % wrt top of blank region
        repmat(defineBlank0(:,2)',nFrames,1);   
    defineBlank(:,4,:) = floor( permute(defineBlankHeight,[2 3 1]) );
    
    % Delete blank tracks with 0 or negative calculated track widths
    defineBlankTEST = defineBlank;
    defineBlankTEST((defineBlankTEST(:,3,1) <= 0),:,:) = [];
    if size(defineBlank,1) ~= size(defineBlankTEST,1)
        disp(['    Deleting ' num2str(size(defineBlank,1) - ...
            size(defineBlankTEST,1)) ' invalid blank tracks since these'])
        disp('    have negative or 0 track width.')
        defineBlank = defineBlankTEST;
    end
    clear defineBlankTEST;
    nBlankTracks = size(defineBlank,1); % Number of blank tracks
    disp(' ')
    disp(['''defineBlank'' array defined. Rows for each of ' ...
        num2str(nBlankTracks) ' blank regions, '])
    disp('columns for (x,y,width,height), layers for each frame.')
    
    % Display an overview of blank regions
    clf; imshow(middleFrame); axis on; hold on; % Show middle frame
    disp(' ')
    disp('Preview MIDDLE frame with overlaid blank region definitions:')
    if strcmp(defineBlank_LowEdgeType,'fix')
        disp('    Green : Bottom edge of all blank regions in all frames.')
    else
        disp('    Green : Bottom edge of blank regions move with the')
        disp('        migration fronts to a maximum position represented')
        disp('        by this line.')
    end
    plot([1;size(middleFrame,2)],...  % Add green line
        [defineBlank_LowEdge;defineBlank_LowEdge],'-g');
    disp('    Red   : Blank regions for MIDDLE frame.')
    disp('    Blue  : Blank regions for LAST frame.')
    for i = 1:nBlankTracks
        rectangle('Position', defineBlank(i,:,round(nFrames/2)),...
            'EdgeColor','r');   % Middle frame
        rectangle('Position', defineBlank(i,:,nFrames),...
            'EdgeColor','b');   % Last grame
    end
    hold off;
    
    % Confirm calculated values and continue
    disp(' ')
    confirm = input('Confirm these values (y/n): ','s');
    if ~strcmp(confirm,'y')
        clear defineBlank nBlankTracks;
    end
    clear defineBlank0 defineBlankHeight;
end

%% Conclusion and time stamping
clear confirm* i j trackNum tempFrame dummyImage;
disp(' ')
runDate_Recipe = datestr(clock, 'yyyy-mm-dd, HH:MM');
disp(['Time stamp: ' runDate_Recipe])
disp(['Saving recipe file as: ''' newRecipe ''''])
save([recipePath newRecipe]);   % Save changes
disp('***SUCCESS SCRIPT FINALLY DONE!!!***')

end