function filteredChromaPlotVideo

% Written by S.R. Jim, (C) October 2013
% Glancing Angle Deposition (GLAD) Laboratory, PI: M.J. Brett
% University of Alberta, Edmonton, Canada
% 
% Purpose:  Consider the colour filtered densitograms for a given track and
% their evolution over the course of a chromatographic development.  Use
% this movie to assess the appropriateness of the selected colour filters
% and the gaussian fits.  This script does not calculate any signals and
% only plots hue-filtered densitogram signals already produced by
% mainUTLC.m.  It allows multiple separation track videos to be generated.
% 
% Inputs:  Data file generated by mainUTLC.m, original directory of frames
% used to generate this data file.
% 
% Outputs: Videos of chromatograms plotted along side their cropped track
% image.

%% Set file directories
clear all; close all;
disp(' ')
disp('PART 0:  SET INPUT DATA AND OUTPUT VIDEO DIRECTORIES/FILES')
disp(' ')

% Select recipe file
disp('Select input data file (output by mainUTLC.m)')
[ipData0,ipDataPath0] = uigetfile('*.mat', ...
    'Select input data file (output by mainUTLC.m)');
ipDataPath0 = [ipDataPath0 '\'];

% Load desired variables from recipe file
load([ipDataPath0 ipData0], ...
    'defineTrack1','Densitogram','filepath','filename','fitDensitogram',...
    'frameRate','hueRangeLabel1','img_ext','middleFrame','nFrames',...
    'nSamples','nSpots1', 'nTracks1','samples','sampleFreq',...
    'spotLocationX1','spotLocationY1','zf1');
ipData = ipData0; ipDataPath = ipDataPath0;
clear ipData0 ipDataPath0;
disp(['Loaded variables from input data file ''' ipData '''.'])

% Specify output video file name
disp(' ')
disp('Define location and name of output *.avi video file(s).')
opFilePath = uigetdir(ipDataPath,['Select directory in which to '...
    'place output *.avi file(s).']);
opFilePath = [opFilePath '\'];
disp('Enter video file name prefix.  This will be labelled as:')
disp('    ''PREFIX-T#.avi'', where ''T#'' denotes track number')
disp('    (as listed in ''defineTrack1'')')
opFilePREFIX = input('Enter file name PREFIX: ','s');

% Set directory of UTLC video frames to analyze:
disp(' ')
disp(['CONFIRM directory of frame *' img_ext ' extracted from video'])
% Assume location to be same as defined when creating recipe
filepath = uigetdir(filepath,...
    ['CONFIRM directory of frame *' img_ext ' extracted from video']);
filepath = [filepath '\'];

% Display information about image directory
filename = dir([filepath '*' img_ext]);
nFrames = length(filename);
disp(' ')
disp(['Selected directory contains ' num2str(nFrames) ' frames.'])
disp(['The first frame is: ''' filename(1).name ''''])
disp(['The last frame is: ''' filename(end).name ''''])

% Display information about information in mainUTLCv* input file
disp(' ')
disp(['Input file has sampled data for ' num2str(nSamples) ' frames.'])
disp(['The first frame is: ''' filename(1).name ''''])
disp(['The last frame is: ''' filename(samples(end)).name ''''])
disp(['Data exists for 1 frame out of every ' num2str(sampleFreq) ...
    ' frames.'])

% Select frame midway through video for error check and preview
fmHeight = size(middleFrame,1); fmWidth = size(middleFrame,2);

%% Error check
disp(' ')
disp('PART 1:  ERROR CHECK AND DATA SUBSET SELECTION')
disp(' ')

% Verify that necessary variables exist
existInputVars = exist('defineTrack1','var') && ...
    exist('spotLocationX1','var') && exist('spotLocationY1','var') && ...
    exist('hueRangeLabel1','var') && exist('zf1','var') && ...
    exist('Densitogram','var') && exist('fitDensitogram','var') && ...
    exist('frameRate','var') && exist('nSamples','var') && ...
    exist('samples','var') && exist('sampleFreq','var') && ...
    exist('nSpots1','var') && exist('nTracks1','var');
if ~existInputVars
    disp('Warning: some variables missing from ''InputFile'' *.mat file')
    go = input('Continue (y/n)? ','s');
    if strcmp(go,'n')
        error('Script stopped by Silly user.');
    end
end
clear go;

% Select subset of variables that corresponds to sampled Densitogram and
% fitDensitogram data
zf1 = zf1(samples,:);
filename = filename(samples,:);
time = ([1:nFrames]' - 1) ./ frameRate;
time = time(samples);

% Check validity of defineTrack1 values
good_defineTrack1 = exist('defineTrack1','var') && ...
    (size(defineTrack1,1) == nTracks1) && ...   % Data for all tracks
    (size(defineTrack1,2) == 4) && ...
    (min(defineTrack1(:)) >= 0) && ...
    (max(defineTrack1(:,1)) <= fmWidth) && ...  % Ensure x,width<fmWidth
    (max(defineTrack1(:,3)) <= fmWidth) && ...
    (max(defineTrack1(:,2)) <= fmHeight) && ... % Ensure y,Height<fmHeight
    (max(defineTrack1(:,4)) <= fmHeight);
if ~good_defineTrack1
    error('''defineTrack1'' array is invalid, Silly!');
end

% Check validity of Densitogram input data arrays
good_Densitogram = exist('Densitogram','var') && ...
    (size(Densitogram,1) == fmHeight) && ... % Length of vector
    (size(Densitogram,2) == nSpots1) && ... % Number of spots
    (size(Densitogram,3) == nSamples) && ...    % Number of sampled frames
    (size(Densitogram,4) == nTracks1) && ...    % Number of tracks
    (min(Densitogram(:)) >= 0);
if ~good_Densitogram
    error('''Densitogram'' data array is invalid, Silly!')
end
clear good_Densitogram;

% Check validity of Densitogram input data arrays
good_fitDensitogram = exist('fitDensitogram','var') && ...
    (size(fitDensitogram,1) == fmHeight) && ... % Length of vector
    (size(fitDensitogram,2) == nSpots1) && ...  % Number of spots
    (size(fitDensitogram,3) == nSamples) && ... % Number of sampled frames
    (size(fitDensitogram,4) == nTracks1) && ... % Number of tracks
    (min(fitDensitogram(:)) >= 0);
if ~good_fitDensitogram
    error('''fitDensitogram'' data array is invalid, Silly!')
end
clear good_fitDensitogram;
disp('Error check and data selection complete.')

%% Select desired track(s) to analyze
disp(' ')
disp('PART 2:  SELECT DESIRED TRACK(S) TO ANALYZE')
disp(' ')

close all; figure(1); imshow(middleFrame);   % Preview frame
disp('Preview loaded ''defineTrack1'' track and spot position data.')
disp('Note that these track numbers may be a subset of those in the ')
disp('''RecipeFile'' originally input *into* ''mainUTLCv*.m''')
hold on;
for i = 1:nTracks1      % Indicate location of each track
    % TOP left corner coordinates and dimensions (TOP because of
    % inverted image y-axis
    rectangle('Position', defineTrack1(i,:));
    plot(spotLocationX1(i),spotLocationY1(i),'or');
end
clear i;
hold off

% Input selected track numbers
confirm = 'n';
while ~strcmp(confirm,'y')
    disp(' ')
    disp(['Data for ' num2str(nTracks1) ' tracks is available. For which'])
    disp('of these tracks a video should be generated?  Input a')
    selectTrkNum = input('row vector to select from available tracks: ');
    nSelectTrkNum = length(selectTrkNum);
    trkLoc = defineTrack1(selectTrkNum,:);
    initialSpotLocY = spotLocationY1(selectTrkNum);
    initialSpotLocX = spotLocationX1(selectTrkNum);
    % Preview selected track location(s) and starting position(s)
    disp('Track position(s) defined.')
    clf; imshow(middleFrame);
    opFile = cell(nSelectTrkNum,1);
    hold on;
    for i = 1:nSelectTrkNum      % Indicate location of each track
        % TOP left corner coordinates and dimensions (TOP because of
        % inverted image y-axis
        rectangle('Position', trkLoc(i,:));
        plot(initialSpotLocX(i),initialSpotLocY(i),'or');
        % Create filename string
        opFile{i} = [opFilePREFIX '-T' num2str(selectTrkNum(i)) '.avi'];
    end
    clear i;
    hold off
    confirm = input('Confirm track selection (y/n)?: ','s');
end
clear confirm;

%% Select desired hue filters
disp(' ')
disp('PART 3:  SELECT DESIRED HUE FILTERS/LABELS')

confirm = 'n';
while ~strcmp(confirm,'y')
    clear filterLabel;
    disp(' ')
    disp('Loaded ''hueRangeLabel1'' filter definitions:')
    disp(hueRangeLabel1)
    disp('Enter row vector to select desired hue filter label. Note that ')
    disp('these may be a subset of those in the ''RecipeFile''')
    selectFilterNum = input('originally input *into* ''mainUTLC.m'': ');
    filterLabel = hueRangeLabel1(selectFilterNum);  % Cell array of labels
    Nfilters = length(filterLabel);
    disp('Specified filters:')
    disp(filterLabel)
    confirm = input('Confirm filters/labels (y/n)?: ','s');
end
clear confirm;

%% Extract relevant Densitogram and fitDensitogram data
disp(' ')
disp('PART 4:  EXTRACT RELEVANT AVAILABLE DENSITOGRAM DATA')
disp(' ')

Densitogram = Densitogram(:,selectFilterNum,:,selectTrkNum);
fitDensitogram = fitDensitogram(:,selectFilterNum,:,selectTrkNum);
disp('Subset of densitogram data selected.')

%% Setup video
disp(' ')
disp('PART 5:  SPECIFY VIDEO PARAMETERS')
disp(' ')

disp(['Video frames were extracted at rate ' num2str(frameRate) ' (fps).'])
disp('This data was sampled so that one data point was extracted')
disp(['from every ' num2str(sampleFreq) ' frames.'])
disp('Define output frame rate and playback speed')

confirm = 'n';
while ~strcmp(confirm,'y')
    disp(' ')
    vidFrameRate = input('Enter desired output video frame rate (fps): ');
    playspeed = sampleFreq * vidFrameRate / frameRate;
    disp(['The playback speed will be ' num2str(playspeed) ' x.'])
    confirm = input('Confirm this frame rate and playback (y/n)?: ','s');
end
clear confirm;

disp(' ')
disp('Set additional video definitions:')
confirm = 'n';
while ~strcmp(confirm,'y')
    vidFrameSize = input(['Size of each video frame (ex. ' ...
        '[w h]=[800 600] typical): ']);
    dataColour = [];
    while length(dataColour) ~= Nfilters
        disp(['Select ' num2str(Nfilters) ' colours of plotted signals.'])
        dataColour = ...
            input('    Specify using string like "rgbcmyk": ','s');
    end
    plotsignalFit = input('Plot fitted signals (y/n): ','s');
    plotUnfilteredSignal = input(['Plot offset unfiltered summed ' ...
        'saturaton signal (y/n): '],'s');
    plotInitialSpotLocY = input('Plot applied spot position (y/n): ','s');
    plotZfront = input('Plot migration front position (y/n): ','s');
    disp(['Maximum value of the densitogram ''Densitogram'' is: ' ...
        num2str(max(Densitogram(:)))])
    disp(['    110% of this value is: ' ...
        num2str(1.1*max(Densitogram(:)))])
    disp(['Maximum value of the densitogram ''fitDensitogram'' is: ' ...
        num2str(max(fitDensitogram(:)))])
    disp(['    110% of this value is: ' ...
        num2str(1.1*max(fitDensitogram(:)))])
    disp('Constrain y-axis limits across all frames? Type ''n'' (with ')
    disp('quotes) to allow automatic limits or enter row vector to fix ')
    yaxislim = input('y-axis limits ([ymim ymax]): ');
    confirm = input('Confirm all of these values (y/n)?: ','s');
end
clear confirm;

%% Process tracks from frames and generate output video frames
disp(' ')
disp('PART 6:  PROCESS/PLOT DATA AND RENDER VIDEO')
disp(' ')

disp('Reminder: Verify that previous versions of video(s):')
disp(opFile)
disp('have been deleted from output directory!')
disp(' ')

process = input('Start processing data or abort (y/n)?: ','s');
if strcmp(process,'n')
    error('Script aborted by Silly user... :-(');
end

% Process chromatograms and generate video
for t = 1:nSelectTrkNum
    close all;
    disp(' ')
    disp(['Begin processing data for video ' num2str(t) ' of ' ...
        num2str(nSelectTrkNum) '.'])
    disp(['Current video name: ' opFile{t}])
    % Video settings
    clear aviobj;
    aviobj = avifile([opFilePath opFile{t}], 'compression', 'None', ...
        'fps', vidFrameRate);
    fig = figure('visible','off');
    set(fig,'Position',[50 50 vidFrameSize]);
    set(fig,'Color',[1 1 1]);
    
    tic; tstart = tic;             % Start timer
    if t == 1
        tstart0 = tstart;
    end
    for m = 1:nSamples
        disp(' ')
        disp(['Considering frame ' num2str(m) ' of ' ...
            num2str(nSamples) '...'])

        % Read given frame, crop track, convert to HSV
        img = imread([filepath filename(m).name]);
        trk0 = imcrop(img,trkLoc(t,:));
        
        % Display rotated track image and title
        clf; sp = zeros(1,2);   % Initialize subplot handles
        sp(1) = subplot(2,1,1);
        imshow(imrotate(trk0,90));
        set(sp(1),'Position',[0.1 0.7 0.85 0.35]);
        title(['Track ' num2str(selectTrkNum(t)) ' cropped from: ' ...
            filename(m).name ' (rotated 90 deg CCW); t = ' ...
            num2str(time(m)) ' s']);
        xlim([0 fmHeight]);

        % Prepare plot for colour-filtered densitogram data
        sp(2) = subplot(2,1,2); hold on;
        set(sp(2),'Position',[0.1 0.15 0.85 0.6]);
        xlabel('Position [px]'); ylabel('Filtered saturation sum [a.u.]');
        title(['Track loc. [x y w h] = [' num2str(trkLoc(t,1)) ' ' ...
            num2str(trkLoc(t,2)) ' ' num2str(trkLoc(t,3)) ' ' ...
            num2str(trkLoc(t,4)) ']; Extraction / video rate = ' ...
            num2str(frameRate) ' / ' num2str(vidFrameRate) ...
            ' fps; Playback speed = ' num2str(playspeed) ' x.']);
        if ~strcmp(yaxislim,'n')      % Set y-axis limits if they exist
            ylim(yaxislim);
        end

        % Add imported signal but not fitted signal for each filter
        for h = 1:Nfilters             % For all hue filters
            plot(1:fmHeight,Densitogram(:,h,m,t),['.' dataColour(h)]);
        end

        % Add calculated unfiltered summed saturation Densitogram as grey
        % line if desired. Create unfiltered Densitogram with added offset
        % (10% of peak)
        if strcmp(plotUnfilteredSignal,'y')
            trk = rgb2hsv(trk0);
            trk(:,:,1) = 360 .* trk(:,:,1);
            fmHeight = size(trk,1);
            % Unfiltered Densitogram with 10% offset
            UnfilteredSignal = ...
                sum(trk(:,:,2),2) + max(sum(trk(:,:,2),2))/10;
            plot(1:fmHeight,UnfilteredSignal,'Color',[0.5 0.5 0.5]);
            filterLabel{Nfilters+1} = 'UNFILTERED, offset';
            clear UnfilteredSignal;
        end

        % Add legend for above signals
        legend(filterLabel, 'Location', 'NorthEast');

        % Add imported fitted Densitogram for each filter if desired
        if strcmp(plotsignalFit,'y')
            for h = 1:Nfilters             % For all hue filters
                plot(1:fmHeight,fitDensitogram(:,h,m,t),...
                    ['-' dataColour(h)]);
            end
        end

        % Add initial spot location if desired
        if strcmp(plotInitialSpotLocY,'y')
            % Vertical line at initilSpotLocY
            ylims = get(sp(2),'ylim');
            plot([initialSpotLocY(t);initialSpotLocY(t)],ylims,'--k');
        end

        % Add migration front position if desired
        if strcmp(plotZfront,'y')
            % Add initialSpotLocY back into zf1
            Zfront = zf1(m,t) + initialSpotLocY(t);
            ylims = get(sp(2),'ylim');
            plot([Zfront;Zfront],ylims,'--k');   % Vertical line at Zfront
        end
        xlim([0 fmHeight]);
        hold off;

        % Export frame as video frame
        aviobj = addframe(aviobj,fig);
        disp(['    Added data for frame ' num2str(m) ...
            ' of ' num2str(nSamples)])

        % Clean-up
        clear img trk0 trk;

        % Estimate time remaining
        toc;    % Display total elapsed time
        testimate = toc(tstart) * (nSamples-m) / m;
        disp(['Estimated time remaining for this video:   ' ...
            num2str(fix(testimate)) ' seconds.'])
    end
    close(fig);
    aviobj = close(aviobj); % Finished this video! On to the next!
    disp(['Completed processing data for video ' num2str(t) ' of ' ...
        num2str(nSelectTrkNum) '.'])
    disp(['Video name: ' opFile{t}])
end

disp(['Total processing time was ' num2str(fix(toc(tstart0))) ' seconds.'])
disp(' ')
disp('***filteredChromaPlotVideo.m IS COMPLETE***')

end