function [D,avg_px,stdev_px,avg_D,stdev_D] = ...
    filteredDensitogram(img,hueRange)

% Written by S.R. Jim, (C) October 2013
% Glancing Angle Deposition (GLAD) Laboratory, PI: M.J. Brett
% University of Alberta, Edmonton, Canada
% 
% Purpose:  This script is designed to analyze an input UTLC track image
% (img, developed top to bottom) or a blank track region given a set of
% colour filters (hueRange).  It then outputs a 2D array (D) whose
% columns contain colour-filtered densitograms.  It also outputs the mean
% and standard deviation row vectors calculated in two different ways:
% 
%   - avg_px, stdev_px : Mean and standard deviation in the saturation 
%   values of all PIXELS in img with hues inside of the filter.  The
%   saturation values of pixels with hue outside of the filter are excluded
%   from this analysis.
% 
%   - avg_D, stdev_D : Mean and standard deviation in the calculated
%   DENSITOGRAM signal.  Recall that the signal 'D' is determined both by
%   the number of pixels with hue inside of the filter as well as the
%   saturation values of those pixels.  Pixels with hue outside of the
%   filter have their saturation values set to 0 before signal summing.
% 
% Notes:
%   - Unfiltered densitograms are possible when hueRange(i,1) == 
%   hueRange(i,2).  (Ex. [0 0]; special case: [0 360] --> [0 0])

%% Error check
good_img = (size(img,3) == 3) && isa(img(:),'uint8');
if ~good_img
    error('Input RGB image array ''img'' is invalid, Silly!')
end
good_hueRange = (size(hueRange,2) == 2) && (max(hueRange(:)) <= 360) && ...
    (min(hueRange(:)) >= 0);
if ~good_hueRange
    error('Input ''hueRange'' array is invalid, Silly!')
end
hueRange(hueRange == 360) = 0;  % Check for 360 == 0 discontinuity

%% Analyze track image
% RGB -> HSV conversion
img = rgb2hsv(img);
img(:,:,1) = 360 .* img(:,:,1);     % Scale hue to range [0,360)

% Initialize output array and vectors
nFilters = size(hueRange,1);
D = zeros(size(img,1),nFilters);    
avg_px = zeros(1,nFilters); stdev_px = zeros(1,nFilters);
avg_D = zeros(1,nFilters); stdev_D = zeros(1,nFilters);

% Define and apply hue filters
for i = 1:size(hueRange,1)
    if hueRange(i,1) < hueRange(i,2)        % Ex. [30 200]
        filteredSat = img(:,:,2) .* ( (img(:,:,1) >= hueRange(i,1)) & ...
            (img(:,:,1) <= hueRange(i,2)) );
    elseif hueRange(i,1) > hueRange(i,2)    % Ex. [200 30]
        filteredSat = img(:,:,2) .* ( (img(:,:,1) >= hueRange(i,1)) | ...
            (img(:,:,1) <= hueRange(i,2)) );
    elseif hueRange(i,1) == hueRange(i,2)   % Ex. [0 0] or [360 360]
        filteredSat = img(:,:,2);           % Unfiltered case
    end
    D(:,i) = sum(filteredSat,2);    % hue-filtered densitogram
    
    % Calculate mean and standard deviation over all indivual pixels with
    % permitted hues.  Exclude filtered-out (saturation = 0) values by
    % setting to NaN
    filteredSat(filteredSat == 0) = NaN;
    % Calculate mean and standard deviation while excluding NaN values
    % Flag = 0 causes normalization by (N-1)
    avg_px(i) = nanmean(filteredSat(:));
    stdev_px(i) = nanstd(filteredSat(:),0);
    
    % Calculate mean and standard deviation in calculated densitogram
    % signal.  This incorporates all values (no NaNs introduced).
    avg_D(i) = mean(D(:,i));
    stdev_D(i) = std(D(:,i),0);
    
    clear filteredSat;
end
clearvars -except D avg* stdev*;

end