function fomUTLC(spec,inFile,outFile)

% Written by S.R. Jim, (C) October 2013
% Glancing Angle Deposition (GLAD) Laboratory, PI: M.J. Brett
% University of Alberta, Edmonton, Canada
% 
% Purpose:  Given information produced by the recipe file (output by
% createRecipeUTLC.m) and calculated/fitted data file (output by
% mainUTLC.m), this script calculates several figures of merit (FOM) that
% are used to evaluate performance from a UTLC development video.  This
% script can be called by batchAnalyzeVideoUTLC.m.
% 
% Input:  Specify the *.mat file that contains information produced by the
% aformentioned scripts.
% 
% Optional Inputs (those from batchAnalyzeVideoUTLC.m):
%   - spec:  A structure containing predefined values for inputs required
%   by this script.
%   - inFile:  Full filepath and name of the input recipe file.
%   - outFile:  Full filepath and name of the output main file.
% 
% Outputs:  Relevant figures of merit saved in a new *.mat file.
% 
% Notes:
%   - Based on figuresOfMerit.m written by A.J. Oko, September 2011
%   - This script attempts to calculate the signal-to-noise ratio and
%   limits of detection (LODs).  Noise is calculated from signals generated
%   on the blank tracks.  These values should not be used without further
%   verification.  Improved understanding of noise behaviours must be
%   performed.
%   - This script attempts to calculate the separation number (SN), real
%   plate number (RealPN), and real plate height (RealPH). These values are
%   valid only if typical spot broadening occurs:  Higher RF spots broaden
%   more than lower RF spots.  Spot focussing and other effects can cause
%   nonsensical values.

%% Set file directories
close all;
disp(' ')
disp('PART 0:  SET INPUT DATA AND OUTPUT FOM DATA FILE DIRECTORIES/FILES')
disp(' ')

% Select and load main UTLC file (may be in function argument)
if exist('inFile','var') && logical(exist(inFile,'file'))
    [ipDataPath0,ipData0,ext] = fileparts(inFile);
    ipData0 = [ipData0 ext];
    clear inFile ext;
else
    disp('Select input data file (output by mainUTLC.m)')
    [ipData0,ipDataPath0] = uigetfile('*.mat', ...
        'Select input data file (output by mainUTLC.m)');
end
ipDataPath0 = [ipDataPath0 '\'];
load([ipDataPath0 ipData0]);
ipData = ipData0; ipDataPath = ipDataPath0;
clear ipData0 ipDataPath0;
disp(['Loaded input data file ''' ipData '''.'])

% Specify output file name if not defined as a string in the function
% argument.  (Note the file itself doesn't need to exist yet.)
disp(' ')
if exist('outFile','var')
    [opDataPath,opData,ext] = fileparts(outFile);
    opDataPath = [opDataPath '\'];
    opData = [opData ext];
    clear outFile ext;
    disp(['Specified output data file ''' opData '''.'])
else
    disp('Define location and name of output figures-of-merit data file.')
    disp('Contains data from input file and that from this script.')
    opDataPath = uigetdir(ipDataPath,['Select directory in which to '...
        'place output FOM data file.']);
    opDataPath = [opDataPath '\'];
    opData = input('Enter output FOM data file name (name.mat): ','s');
end

% Variable to record the script name and revision
script_fomUTLC = 'fomUTLC.m rev October 2013';

% Delete Densitogram and fitDensitogram signals if present
if exist('Densitogram','var')
    clear Densitogram;
    disp('''Densitogram'' data array cleared from memory.')
end
if exist('fitDensitogram','var')
    clear fitDensitogram;
    disp('''fitDensitogram'' data array cleared from memory.')
end

% Unpack predefined values for inputs stored in spec
if exist('spec','var') && ~isempty(spec)
    % Number of sigmas over which to integrate fitted signal; 'FWHM' (with
    % quotes) to use the sqrt(8*ln2)
    intLength = spec.intLength;
    calcLOD = spec.calcLOD;
    LODsVect = spec.LODsVect;   % Calculate LOD for which spots?
    LODtVect = spec.LODtVect;   % Using which tracks?
    lodcriteria = spec.lodcriteria; % How many "sigmas" in LOD calculation?
    SNsVect = spec.SNsVect;     % Calculate SN, Real PN, Real PH
end

% Initialize output recipe file (verify that save is permitted)
disp(['Saving data to ' opData ])
save([opDataPath opData]);

%% Basic calculations
disp(' ')
disp('PART 1:  BASIC CALCULATIONS: t, hRF, FWHM, PN, PH')
disp(' ')

% Time (column) vector
t = ([1:nFrames]' - 1) ./ frameRate;

% hRF calculation
zf1_3D = repmat(permute(zf1,[1 3 2]),[1 nSpots1 1]);
spotHRF = 100 * zSpot ./ zf1_3D;    % 100 * retention factor (RF)

% Peak width (FWHM)
spotFWHM = stdSpot*sqrt(8*log(2));

% Theoretical plate number and plate height
spotPN = (zSpot./stdSpot).^2;
spotPH = (stdSpot.^2) ./ zSpot;   % = migration distance / plate number

clear zf1_3D;
disp('Calculated hRF, FWHM, PN, PH')
disp(['Saving data to ' opData ])
save([opDataPath opData]);  % Save changes

%% Resolution
disp(' ')
disp('PART 2:  RESOLUTION BETWEEN ANALYTE SPOTS')
disp(' ')

% Arrange arrays to calculate resSpot(analyte1,analyte2,frame,track)
% Example: resSpot(2,3,100,5) is the resolution between analytes 2 and 4
% (=3+1) in frame 100 on track 5.
% Introduce a 4th dimension (analyte2) to arrays with original dimensions
% (frame x analyte x track).

% Square base of "array1"s has ROWS that correspond to analyte1 number
% (analytes 1:(nSpots1-1)) and identical COLUMNS.
zSpot1 = permute(zSpot,[2 4 1 3]);  % dim1 corresponds to analyte1 number
zSpot1 = repmat(zSpot1(1:(nSpots1-1),1,:,:),[1 (nSpots1-1) 1 1]);
stdSpot1 = permute(stdSpot,[2 4 1 3]);	% similar to above
stdSpot1 = repmat(stdSpot1(1:(nSpots1-1),1,:,:),[1 (nSpots1-1) 1 1]);

% Square base of "array2"s has COLUMNS that correspond to analyte2 number
% (analytes 2:nSpots1) and identical ROWS.
zSpot2 = permute(zSpot,[4 2 1 3]);  % dim2 corresponds to analyte2 number
zSpot2 = repmat(zSpot2(1,2:nSpots1,:,:),[(nSpots1-1) 1 1 1]);
stdSpot2 = permute(stdSpot,[4 2 1 3]);  % similar to above
stdSpot2 = repmat(stdSpot2(1,2:nSpots1,:,:),[(nSpots1-1) 1 1 1]);

% Calculate resolution
resScale = 0.5; % Scaling factor depends on res criteria and spot width
resSpot = resScale * (zSpot1 - zSpot2) ./ (stdSpot1 + stdSpot2);

% Set all nonsensical and redundant resolution values to NaN 
%   Ex: resSpot(2,1,f,trk) = NaN since this considers spot 2 vs spot 2
%   Ex: resSpot(3,1,f,trk) = NaN since this considers spot 3 vs spot 2 and
%	resSpot(2,2,frame,track) already considers spot 2 vs spot 3.
% Use a lower triangular matrix of NaN values
resRedundant = repmat(tril(NaN*ones(nSpots1-1),-1),[1 1 nFrames nTracks1]);
resSpot = resSpot + resRedundant;
clear zSpot1 zSpot2 stdSpot1 stdSpot2 resScale resRedundant;
disp(['Calculated resolution between ' num2str(nSpots1) ' spots.'])
disp(['Saving data to ' opData])
save([opDataPath opData]);  % Save changes

%% Signal, noise, and SNR
disp(' ')
disp('PART 3:  SIGNAL, NOISE, AND SIGNAL-TO-NOISE')
disp(' ')

% Calculate the signal as the area under the fitted gaussian curves (input
% the length of the integration region in the development direction in
% number of stdevs).
if exist('intLength','var') && (length(intLength) == 1)
    if isa(intLength,'char') && strcmp(intLength,'FWHM')
        intLength = sqrt(8*log(2));
    end
    disp(['Specified integration region width is ' num2str(intLength) ...
        ' sigmas in length.'])
else
    disp('Define a symmetric region over which each fitted gaussian peak')
    disp('should be integrated.  Express the length of this region in')
    disp('number of standard deviations ("sigmas"; ex. Type "4" for a')
    disp('region that is +/- 2 sigma from the gaussian centre).  Or type')
    disp('''FWHM'' (with quotes) to use sqrt(8*ln2).')
    confirm = 'n';
    while ~strcmp(confirm,'y')
        disp(' ')
        intLength = input('    Enter integration width: ');
        if isa(intLength,'char') && strcmp(intLength,'FWHM')
            intLength = sqrt(8*log(2));
        end
        if isnumeric(intLength) && isscalar(intLength)
            intLength = abs(intLength);
            disp(['    Specified integration region width is ' ...
                num2str(intLength) ' sigmas in length.'])
            confirm = input('    Confirm this value (y/n): ','s');
        end
        if ~strcmp(confirm,'y') % Clear intLength and redefine
            clear intLength;
        end
    end
    clear confirm;
end

% Consider integrated area to be the "signal" associated with each spot
% sigSpot(frame,spot,track)
sigSpot = sqrt(2*pi) * erf(intLength/(2*sqrt(2))) .* ampSpot .* stdSpot;
disp(' ')
disp('Calculated signal as integral of fitted peak area for each spot')
disp('on each track in each frame.')

% Signal-to-noise ratio (SNR) calculation.
% Scale the noise by the absolute width of the track (px)
% (defineTrack(track,3,frame)+1) and of the integration zone length (px)
% (intLength*stdSpot). Single pixel noise is stdevBlank(frame,spot)
stdevBlank3D = repmat(stdevBlank,[1 1 nTracks1]);
% trkWidth scaling factor (+1 since a specified width of N pixels gives a
% crop N+1 pixels wide)
trkWidth3D = repmat( permute(defineTrack1(:,3),[2 3 1]), ...
    [nFrames nSpots1 1]) + 1;
% Array of specified integration region lengths (in px)
intLengthpx3D = intLength * stdSpot;

% Scale noise by area of spot for given mass --> 
% an nFrames x nSpots1 x nTracks1 array
stdevBlankScaled = stdevBlank3D .* sqrt(trkWidth3D.*intLengthpx3D);
disp('Calculated noise scaled for 2D area of each spot on each track')
disp('in each frame.')

% Calculate SNR-- an nFrames x nSpots1 x nTracks1 array
spotSNR = sigSpot ./ stdevBlankScaled;
disp('Calculated SNR for each spot on each track in each frame.')

%% LOD
disp(' ')
disp('PART 4:  LIMIT OF DETECTION (LOD)')
disp(' ')

% Calculate LOD or skip analysis?
if ~exist('spotMass1','var')
    disp('''spotMass1'' missing. LOD calculations cannot be performed ')
    disp('even if desired. Skipping this analysis.')
    calcLOD = 'n';
elseif exist('calcLOD','var') && ~strcmp(calcLOD,'y')
    disp('Skipping LOD calculations.')
elseif exist('calcLOD','var') && strcmp(calcLOD,'y')
    disp('LOD calculations will be performed.')
else
    calcLOD = input('Calculate LOD (y/n)?: ','s');
end

% Check LODsVect and LODtVect required in optional LOD calculations
good_LODsVect = exist('LODsVect','var') && ( strcmp(LODsVect,'all') || ...
    (isvector(LODsVect) && min(floor(LODsVect) == ceil(LODsVect)) && ...
    (min(LODsVect) >= 1) && (max(LODsVect) <= nSpots1) && ...
    (length(LODsVect) == length(unique(LODsVect)))) );
good_LODtVect = exist('LODtVect','var') && ( strcmp(LODtVect,'all') || ...
    (isvector(LODtVect) && min(floor(LODtVect) == ceil(LODtVect)) && ...
    (min(LODtVect) >= 1) && (max(LODtVect) <= nTracks1) && ...
    (length(LODtVect) == length(unique(LODtVect)))) );

% Define LODsVect, LODtVect, lodcriteria if no valid values exist
if strcmp(calcLOD,'y') && (~good_LODsVect || ~good_LODtVect || ...
        ~exist('lodcriteria','var'))
    disp(' ')
    disp(['Spot signal, scaled noise, and area calculated for ' ...
        num2str(nSpots1) ' spots.'])
    disp('These spots have defined hue ranges of: ')
    disp(hueRangeLabel1)
    disp('For which spots should the LOD be calculated?  It is best to ')
    disp('select only those that have reliable gaussian fits.')
    disp('This data corresponds to the subset of tracks ORIGINALLY')
    disp('defined in recipe file that was selected in mainUTLCv*.m:')
    disp(tVect) % Subset of tracks selected in main
    disp('These tracks were renumbered by mainUTLCv*.m as NEW tracks:')
    disp(1:nTracks1)
    disp('Calculated mass [ng] of each analyte (rows) in spot applied ')
    disp('to each of these NEW tracks (columns):')
    disp(spotMass1)
    disp('From which NEW tracks should the LOD be calculated?')
    
    % Select subset of data for LOD calculations and LOD criteria
    confirm = 'n';
    while ~strcmp(confirm,'y')
        disp(' ')
        disp('Select subset of spots and NEW tracks for LOD calculation.')
        LODsVect = ...
            input('    Enter a row vector to select available spots: ');
        LODtVect = ...
            input('    Enter a row vector to select from NEW tracks: ');
        disp('    On what criteria (# of sigmas) should LOD be based?')
        lodcriteria = input('    (ex. enter "3" for 3 * sigma criteria: ');
        if isnumeric(LODsVect) && isnumeric(LODtVect)
            LODspotMass1 = spotMass1(LODsVect,LODtVect); % Subset of masses
            disp('    Defined subset of ''spotMass1'' data:')
            disp(LODspotMass1)
            confirm = input('    Confirm this selection (y/n): ','s');
        else
            disp('    Invalid selections.')
        end
    end
elseif strcmp(calcLOD,'y') && good_LODsVect && good_LODtVect && ...
        exist('lodcriteria','var')
    % Correct for when LODsVect and/or LODtVect = 'all'
    if strcmp(LODsVect,'all')  % LODsVect = 'all' case
        LODsVect = 1:nSpots1;
    end
    if strcmp(LODtVect,'all')  % LODtVect = 'all' case
        LODtVect = 1:nTracks1;
    end
    LODspotMass1 = spotMass1(LODsVect,LODtVect); % Subset of masses
    disp('LOD calculations will be performed for (new) spots:')
    disp(LODsVect)
    disp('with data from (new) tracks:')
    disp(LODtVect)
    disp('that have spot masses ''LODspotMass1'':')
    disp(LODspotMass1)
    disp(['according to the ' num2str(lodcriteria) ' sigma criteria.'])
end
clear good_LODsVect good_LODtVect confirm;

if strcmp(calcLOD,'y')  % Proceed with LOD calculation
    % Number of spots; subset of signals, noise, and SNR
    LODnSpots1 = size(LODspotMass1,1);  % Number of spots to consider
    LODsigSpot = sigSpot(:,LODsVect,LODtVect);  
    LODstdevBlankScaled = stdevBlankScaled(:,LODsVect,LODtVect);
    LODspotSNR = spotSNR(:,LODsVect,LODtVect);
    
    % LOD method 1: Use SNR
    disp(' ')
    disp('Method 1: Calculate ''spotLOD0'' based on SNR measurements')
    spotLOD0 = zeros(nFrames,LODnSpots1);      % Calculated spot LODs
    snrSpotFIT = zeros(nFrames,LODnSpots1,2);  % Linear regress. parameters
    for f = 1:nFrames
        for s = 1:LODnSpots1
            % Extract relevant data and make into column vectors
            spotMass1_1 = LODspotMass1(s,:)';
            spotSNR_1 = permute(LODspotSNR(f,s,:),[3 1 2]);
            % Ensure that there are at least two real number ordered pairs
            % (spotMass1_1(i),spotSNR_1(i)).  Create a test vector that
            % uses the fact that Real_number .* NaN = NaN
            LOD0dataTest = spotMass1_1 .* spotSNR_1;
            if length(LOD0dataTest(~isnan(LOD0dataTest))) >= 2
                % Remove ordered pairs with NaNs as one or both ordinates
                spotMass1_1 = spotMass1_1(~isnan(LOD0dataTest));
                spotSNR_1 = spotSNR_1(~isnan(LOD0dataTest));
                % Linear regress. (ind. = 1), permute to insert into array
                snrSpotFIT(f,s,:) = permute( polyfit(spotMass1_1, ...
                    spotSNR_1, 1), [1 3 2]);            
                spotLOD0(f,s) = (lodcriteria - snrSpotFIT(f,s,2)) / ...
                    snrSpotFIT(f,s,1);
            else % Cannot perform fit or calculate LOD
                snrSpotFIT(f,s,:) = NaN;
                spotLOD0(f,s) = NaN;
            end
            clear spotMass1_1 spotSNR_1 LOD0dataTest;
        end
    end
    clear f s;
    disp('    Calculated ''spotLOD0'' for each spot in each frame.')

    % LOD method2: Use log10s of signals
    disp(' ')
    disp('Method 2: Calculate ''spotLOD'' based on absolute measurements')
    disp('Use measuremement of signal and (unscaled/scaled) signal noise')
    spotMass1LOG = log10(LODspotMass1);    % independent variable
    sigSpotLOG = log10(LODsigSpot);        % dependent variable

    % Calculate noise floor for each frame/spot as mean of noise across all
    % tracks (after scaling). + log10(lodcriteria) from setting LOD
    % threshold to (lodcriteria)*stdeveBlankScaled; log10(x*y) = log10(x) +
    % log10(y) Cannot use mean(...) because this incorporates NaNs
    stdevBlankScaledLOG = log10(LODstdevBlankScaled); % noise
    noisefloorLOG1 = stdevBlankScaledLOG;
    noisefloorLOG1(isnan(noisefloorLOG1)) = 0;  % Duplicate with NaN-->0
    % Count number of valid values to incorporate into average
    noisefloorLOG2 = ~isnan(stdevBlankScaledLOG);
    % Calculate average noise and scale by lodcriteria. (Note that if none
    % of the tracks in the frame have valid peak fits for a given analyte,
    % the calculated noisefloorLOG value is 0/0 == NaN
    noisefloorLOG = sum(noisefloorLOG1,3) ./ sum(noisefloorLOG2,3) + ...
        log10(lodcriteria);
    % now a nFrames x LODnSpots1 x 1 array

    spotLOD = zeros(nFrames,LODnSpots1);           % calculated spot LODs
    sigSpotLOGFIT = zeros(nFrames,LODnSpots1,2);   % linear regress. param.
    for f = 1:nFrames
        for s = 1:LODnSpots1
            % Extract relevant data and make into column vectors
            spotMass1LOG1 = spotMass1LOG(s,:)';
            sigSpotLOG1 = permute(sigSpotLOG(f,s,:),[3 1 2]);
            % Ensure that there are at least two real number ordered pairs
            % (spotMass1LOG1(i),sigSpotLOG1(i)).  Create a test vector that
            % uses the fact that Real_number .* NaN = NaN
            LODdataTest = spotMass1LOG1 .* sigSpotLOG1;
            if length(LODdataTest(~isnan(LODdataTest))) >= 2
                % Remove ordered pairs with NaNs as one or both ordinates
                spotMass1LOG1 = spotMass1LOG1(~isnan(LODdataTest));
                sigSpotLOG1 = sigSpotLOG1(~isnan(LODdataTest));
                % Linear regress. (ind. = 1), permute to insert into array
                sigSpotLOGFIT(f,s,:) = permute( polyfit(spotMass1LOG1, ...
                    sigSpotLOG1, 1), [1 3 2]);            
                spotLOD(f,s) = (noisefloorLOG(f,s) - ...
                    sigSpotLOGFIT(f,s,2)) / sigSpotLOGFIT(f,s,1);
                spotLOD(f,s) = 10 ^ spotLOD(f,s);
            else % Cannot perform fit or calculate LOD
                sigSpotLOGFIT(f,s,:) = NaN;
                spotLOD(f,s) = NaN;
            end
                clear spotMass1LOG1 sigSpotLOG1 LODdataTest;
        end
    end
    clear f s;
    disp('    Calculated LOD for each spot in each frame.')
end
clear stdevBlank3D trkWidth3D intLengthpx3D noisefloorLOG1 noisefloorLOG2;
disp(['Saving data to ' opData ])
save([opDataPath opData]);  % Save changes

%% Separation number
disp(' ')
disp('PART 5:  SEPARATION NUMBER AND "REAL PLATE NUMBER"')
disp(' ')

% Correct for SNsVect = 'all'
if exist('SNsVect','var') && strcmp(SNsVect,'all')
    SNsVect = 1:nSpots1;
end

% Calculate separation numbers?
if exist('SNsVect','var') && isnumeric(SNsVect) && (length(SNsVect) >= 2)
    disp('Separation numbers and real plate numbers/heights will ')
    disp('be calculated on each track using (new) spots:')
    disp(SNsVect)
    SNcalculate = 'y';
elseif (nSpots1 < 2) || (exist('SNsVect','var') && strcmp(SNsVect,'skip'))
    disp('Separation number and real plate number/height calculations')
    disp('either cannot be performed (too few spots) or will be skipped.')
    SNsVect = 'skip';
    SNcalculate = 'SN not calculated';
else
    disp(['Spot position and width data is available for ' ...
        num2str(nSpots1) ' spots.'])
    disp('These spots have defined hue ranges of: ')
    disp(hueRangeLabel1)
    disp('On which spots (>=2) should separation number calculation be')
    disp('be based?  Select only those with reliable gaussian fits.')
    confirm = 'n';
    SNcalculate = '';       % Used to determine if SN should be calculated
    while ~strcmp(confirm,'y')
        disp('    Enter row vector to pick spots or type ''skip'' (with ')
        SNsVect = input('    quotes) to skip calculations: ');
        if isnumeric(SNsVect) && (length(SNsVect) >= 2)
            SNcalculate = 'y';
            confirm = input('    Confirm this selection (y/n): ','s');
        elseif strcmp(SNsVect,'skip')
            SNcalculate = 'SN not calculated';
            clear SNsVect;
            confirm = input('    Confirm this selection (y/n): ','s');
        else
            disp('    Invalid input or insufficient number of spots.')
        end
        disp(' ')
    end
    clear confirm;    
end

% Calculate separation number using subset of data (if desired above)
if strcmp(SNcalculate,'y')
    % Select subset of available data
    SNspotRF = spotHRF(:,SNsVect,:) / 100;  % Scale back to range [0 1]
    SNspotFWHM = spotFWHM(:,SNsVect,:);
    % Perform fit of spot width vs. hRF data
    SNfit = zeros(nFrames,nTracks1,2);      % Initialize
    for f = 1:nFrames
        for trk = 1:nTracks1
            SNspotRF1 = SNspotRF(f,:,trk)';   % Extract data to be fit
            SNspotFWHM1 = SNspotFWHM(f,:,trk)';
            % Ensure that there are at least two real number ordered pairs
            % (SNspotRF1(i),SNspotFWHM1(i)).  Create a test vector that
            % uses the fact that Real_number .* NaN = NaN
            SNdataTest = SNspotRF1 .* SNspotFWHM1;
            if length(SNdataTest(~isnan(SNdataTest))) >= 2
                % Remove pairs that have NaNs as one or both ordinates
                SNspotRF1 = SNspotRF1(~isnan(SNdataTest));
                SNspotFWHM1 = SNspotFWHM1(~isnan(SNdataTest));
                % Linear fit (ind = 1); permute to insert into array
                SNfit(f,trk,:) = permute( polyfit(SNspotRF1,SNspotFWHM1,...
                    1), [1 3 2]);
            else % Cannot perform fit
                SNfit(f,trk,:) = NaN;
            end
            clear SNspotRF1 SNspotFWHM1 SNdataTest;
        end
    end    
    % Calculate Separation Number (SN = zF / (b1 + b0) - 1) and 
    % N_real = 5.54 * (zF / (b1 - b0))^2; H_real = zF / N_real; where: 
    % b0 : FWHM extrap to RF = 0 --> y-intercept: b0 = SNfit(f,s,2)
    % b1 : FWHM extrap to RF = 1 --> b1 = SNfit(f,trk,1)*1 + SNfit(f,trk,2)
    % b1 + b0 = SNfit(f,trk,1) + 2 * SNfit(f,s,2)
    % b1 - b0 = SNfit(f,trk,1)
    SN = zf1 ./ (SNfit(:,:,1) + 2 * SNfit(:,:,2)) - 1;
    RealPN = sqrt(8*log(2)) * ( zf1 ./ SNfit(:,:,1) ) .^ 2;
    RealPH = zf1 ./ RealPN;
    disp('Calculated separation number ''SN'', real plate number')
    disp('''RealPN'', and real plate height ''RealPH'' using method in')
    disp('Thin-layer chromatography: a modern practical approach by ')
    disp('P.E. Wall, 2005; Royal Society of Chemistry')
else
    disp('Calculation skipped. ''SN'',''RealPN'', ''RealPH'' not created.')
end
clear f trk;

%% Save changes
clear inFile outFile spec;
disp(' ')
runDate_fomUTLC = datestr(clock, 'yyyy-mm-dd, HH:MM');
disp(['Time stamp: ' runDate_fomUTLC])
disp(['Saving data to ' opData])
save([opDataPath opData]);  % Save changes
disp('***SUCCESS SCRIPT FINALLY DONE!!!***')

end