function fomUTLCavg

% Written by S.R. Jim, (C) October 2013
% Glancing Angle Deposition (GLAD) Laboratory, PI: M.J. Brett
% University of Alberta, Edmonton, Canada
% 
% Purpose:  Average the figures-of-merit from "similarly" loaded tracks.
% This script processes a file output by fomUTLC.m.  Basic figures-of-
% merit (zSpot, stdSpot, spotFWHM, spotHRF, spotPN, spotPH, etc.) and
% optional figures-of-merit (spotLOD, etc.) are considered.  Means and
% standard deviatons are calculated at each instant in time.
% 
% The output file contains all fomUTLC.m variables (unaltered) and all
% variables generated in this script.

%% Set file directories
clear all; close all;
disp(' ')
disp('PART 0:  SET INPUT RECIPE AND OUTPUT DATA DIRECTORIES/FILES')
disp(' ')
% Select and load recipe file
disp('Select input fomUTLC file.')
[ipFOM0,ipFOMPath0] = uigetfile('*.mat', ...
    'Select input fomUTLC file');
ipFOMPath0 = [ipFOMPath0 '\'];
load([ipFOMPath0 ipFOM0]);
ipFOM = ipFOM0; ipFOMPath = ipFOMPath0;
clear ipFOM0 ipFOMPath0;
disp(['Loaded input fomUTLC file ''' ipFOM '''.'])

% Specify output file name
disp(' ')
disp('Define location and name of output file.  This contains data')
disp('from input fomUTLC file and calculations from this script.')
opFOMPath = uigetdir(ipFOMPath,['Select directory in which to '...
    'place output file.']);
opFOMPath = [opFOMPath '\'];
opFOM = input('Enter output data file name (name.mat): ','s');

% Variable to record the script name and revision
script_fomUTLC_avg = 'fomUTLCavg.m rev October 2013';

%% Error checking
disp(' ')
disp('PART 1:  ENSURE NECESSARY/OPTIONAL FOM VARIABLES EXIST')
disp(' ')

% Test for necessary variables
good_fomFile = exist('ampSpot','var') && exist('hueRange1','var') && ...
    exist('hueRangeLabel1','var') && exist('nTracks1','var') && ...
    exist('nSpots1','var') && exist('resSpot','var') && ...
    exist('sigSpot','var') && exist('spotFWHM','var') && ...
    exist('spotHRF','var') && exist('spotPH','var') && ...
    exist('spotPN','var') && exist('spotSNR','var') && ...
    exist('stdSpot','var') && exist('sVect','var') && ...
    exist('tVect','var') && exist('zSpot','var') && exist('zf1','var');
if good_fomFile
    disp('All basic fomUTLC variables exist.')
else
    disp('Some basic variables are missing.  Necessary variables:')
    disp('''ampSpot'', ''hueRange1'', ''hueRangeLabel1'', ''nSpots1'', ')
    disp('''nTracks1'', ''resSpot'', ''sigSpot'', ''spotFWHM'', ')
    disp('''spotHRF'', ''spotPH'', ''spotPN'', ''spotSNR'', ''stdSpot'', ')
    disp('''tVect'', ''zSpot'', and ''zf1''.')
	error('Abort script.')
end
clear good_fomFile;

% Test for LOD variables
good_fomFile_loading = exist('spotMass1','var') && ...
    exist('spotVolume1','var');
if good_fomFile_loading
    disp('All spot loading data also exist.')
else
    disp('Spot loading data not present.')
end
    
% Test for separation number and real plate number variables
good_fomFile_SN = exist('SNcalculate','var') && ...
    strcmp(SNcalculate,'y') && exist('SN','var') && ...
    exist('RealPH','var') && exist('RealPN','var');
if good_fomFile_SN
    disp('All SN and real plate number/height data also exist.')
else
    disp('SN and real plate number/height data not present.')
end

% Initialize output recipe file (verify that save is permitted)
save([opFOMPath opFOM]);

%% Specify averaging and initialize variables
disp(' ')
disp('PART 2:  SPECIFY NUMBER OF AVERAGED FOM SIGNALS FOMs')
disp(' ')

% Display information about available tracks
fprintf('FOM data for %d spots on %d tracks available',nSpots1,nTracks1)
disp(' ')
disp('The original spot numbers (as specified in recipe) are:')
disp(sVect)
disp('These have labels:')
disp(hueRangeLabel1)
disp('The original tracks (as specified in recipe) are:')
disp(tVect)
if exist('spotVolume1','var')   % Spot volume data
    disp('Their applied spot volumes are (nL):')
    disp(spotVolume1)
end
if exist('spotMass1','var')
    disp('Their applied spot masses are (ng):')
    disp(spotMass1)
end

% Set number of averaged signals to create
confirm = 'n';
while ~strcmp(confirm,'y')
    avgSigNum = input('Generate how many averaged tracks?: ');
    confirm = input('    Confirm this value (y/n)?: ','s');
end
clear confirm;

% Intialize variables
disp(' ')
disp('Initializing variables for averages/standard deviations.')

zeros2D = zeros(size(ampSpot,1),avgSigNum);
zeros3D = zeros(size(ampSpot,1),size(ampSpot,2),avgSigNum);
zeros4D = zeros(size(resSpot,1),size(resSpot,2),size(resSpot,3),...
    avgSigNum);

avg_ampSpot = zeros3D; stdev_ampSpot = zeros3D;
avg_resSpot = zeros4D; stdev_resSpot = zeros4D;
avg_sigSpot = zeros3D; stdev_sigSpot = zeros3D;
avg_spotFWHM = zeros3D; stdev_spotFWHM = zeros3D;
avg_spotHRF = zeros3D; stdev_spotHRF = zeros3D;
avg_spotPH = zeros3D; stdev_spotPH = zeros3D;
avg_spotPN = zeros3D; stdev_spotPN = zeros3D;
avg_spotSNR = zeros3D; stdev_spotSNR = zeros3D;
avg_stdSpot = zeros3D; stdev_stdSpot = zeros3D;
avg_zSpot = zeros3D; stdev_zSpot = zeros3D;
avg_zf1 = zeros2D; stdev_zf1 = zeros2D;

if good_fomFile_loading     % Spot loading data, if it exists
    avg_spotMass1 = zeros(nSpots1,avgSigNum);
    stdev_spotMass1 = zeros(nSpots1,avgSigNum);
    avg_spotVolume1 = zeros(1,avgSigNum);
    stdev_spotVolume1 = zeros(1,avgSigNum);
end

if good_fomFile_SN	% Separation number and real plate number, if exist
    avg_SN = zeros2D; stdev_SN = zeros2D;
    avg_RealPH = zeros2D; stdev_RealPH = zeros2D;
    avg_RealPN = zeros2D; stdev_RealPN = zeros2D;
end

disp('Variables for averaged values initialized.')

% Intialize other variables used to define averaging
avgSigTrk = cell(1,avgSigNum);  % To contain vectors used to select tracks
avgSigName = cell(1,avgSigNum); % To contain names for new signal names

clear zeros*

%% Specify FOMs of which tracks to average/smooth
disp(' ')
disp('PART 3:  SELECT TRACKS TO BE AVERAGED FOR EACH SIGNAL')

for j = 1:avgSigNum
    % Display available tracks (again)
    disp(' ')
    disp('The original tracks (as specified in recipe) are:')
    disp(tVect)
    if exist('spotVolume1','var')   % Spot volume data
        disp('Their applied spot volumes are (nL):')
        disp(spotVolume1)
    end
    if exist('spotMass1','var')     % Spot mass data
        disp('Their applied spot masses are (ng):')
        disp(spotMass1)
    end
    
    % Specify which tracks should be averaged together
    disp(' ')
    disp('Which tracks should be averaged together for')
    fprintf('averaged signal %d of %d?',j,avgSigNum)
    confirm = 'n';
    while ~strcmp(confirm,'y')
        disp(' ')
        disp('    Specify subset of above tracks to be averaged together')
        trkselect = ...
            input('    as a row vector of (new) track numbers: ');
        trkselect = sort(trkselect);    % Ensure tracks in ascending order
        good_trkselect = isvector(trkselect) && ...
            (min(trkselect) >= 1) && (max(trkselect) <= nTracks1);
        if good_trkselect
            disp(['    Tracks to be averaged into signal ' num2str(j)])
            disp(tVect(trkselect))
            if exist('spotVolume1','var')   % Spot volume data
                disp('    With applied spot volumes (nL):')
                disp(spotVolume1(trkselect))
            end
            if exist('spotMass1','var')     % Spot mass data
                disp('    With applied spot masses (ng):')
                disp(spotMass1(:,trkselect))
            end
            avgSigTrk{j} = trkselect; clear trkselect;
            
            % Name this signal
            avgSigName{j} = input('    Name for this signal?: ','s');
            
            % Confirm
            disp(' ')
            confirm = input('    Confirm these selections (y/n)?: ','s');
        else
            disp('    Invalid track selection.  Try again!')
        end
    end
end
clear j confirm;

%% Calculate signals of averages and standard deviations
disp(' ')
disp('PART 4:  PROCESS SIGNALS (GENERATE AVERAGES + STANDARD DEVIATIONS)')

for j = 1:avgSigNum
    disp(['Now processing FOMs for signal ' num2str(j) ' of ' ...
        num2str(avgSigNum) '.'])
    
    sig3D = ampSpot(:,:,avgSigTrk{j});
        avg_ampSpot(:,:,j) = nanmean(sig3D,3);
        stdev_ampSpot(:,:,j) = nanstd(sig3D,0,3);
    sig3D = sigSpot(:,:,avgSigTrk{j});
        avg_sigSpot(:,:,j) = nanmean(sig3D,3);
        stdev_sigSpot(:,:,j) = nanstd(sig3D,0,3);
    sig3D = spotFWHM(:,:,avgSigTrk{j});
        avg_spotFWHM(:,:,j) = nanmean(sig3D,3);
        stdev_spotFWHM(:,:,j) = nanstd(sig3D,0,3);
    sig3D = spotHRF(:,:,avgSigTrk{j});
        avg_spotHRF(:,:,j) = nanmean(sig3D,3);
        stdev_spotHRF(:,:,j) = nanstd(sig3D,0,3);
    sig3D = spotPH(:,:,avgSigTrk{j});
        avg_spotPH(:,:,j) = nanmean(sig3D,3);
        stdev_spotPH(:,:,j) = nanstd(sig3D,0,3);
    sig3D = spotPN(:,:,avgSigTrk{j});
        avg_spotPN(:,:,j) = nanmean(sig3D,3);
        stdev_spotPN(:,:,j) = nanstd(sig3D,0,3);
    sig3D = spotSNR(:,:,avgSigTrk{j});
        avg_spotSNR(:,:,j) = nanmean(sig3D,3);
        stdev_spotSNR(:,:,j) = nanstd(sig3D,0,3);
    sig3D = stdSpot(:,:,avgSigTrk{j});
        avg_stdSpot(:,:,j) = nanmean(sig3D,3);
        stdev_stdSpot(:,:,j) = nanstd(sig3D,0,3);
    sig3D = zSpot(:,:,avgSigTrk{j});
        avg_zSpot(:,:,j) = nanmean(sig3D,3);
        stdev_zSpot(:,:,j) = nanstd(sig3D,0,3);
    
    sig2D = zf1(:,avgSigTrk{j});
        avg_zf1(:,j) = nanmean(sig2D,2);
        stdev_zf1(:,j) = nanstd(sig2D,0,2);

    sig4D = resSpot(:,:,:,avgSigTrk{j});
        avg_resSpot(:,:,:,j) = nanmean(sig4D,4);
        stdev_resSpot(:,:,:,j) = nanstd(sig4D,0,4);

    if good_fomFile_loading     % Spot loading data, if it exists
        sig = spotMass1(:,avgSigTrk{j});
            avg_spotMass1(:,j) = nanmean(sig,2);
            stdev_spotMass1(:,j) = nanstd(sig,0,2);
        sig = spotVolume1(avgSigTrk{j});
            avg_spotVolume1(j) = nanmean(sig);
            stdev_spotVolume1(j) = nanstd(sig,0);
    end
        
    if good_fomFile_SN	% SN and real plate number, if it exists
        sig2D = SN(:,avgSigTrk{j});
            avg_SN(:,j) = nanmean(sig2D,2);
            stdev_SN(:,j) = nanstd(sig2D,0,2);
        sig2D = RealPH(:,avgSigTrk{j});
            avg_RealPH(:,j) = nanmean(sig2D,2);
            stdev_RealPH(:,j) = nanstd(sig2D,0,2);
        sig2D = RealPN(:,avgSigTrk{j});
            avg_RealPN(:,j) = nanmean(sig2D,2);
            stdev_RealPN(:,j) = nanstd(sig2D,0,2);
    end

end
clear sig sig2D sig3D sig4D j;

disp(' ')
disp(['Finished processing ' num2str(avgSigNum) ' signals.'])
disp(['Saving data to ' opFOM ])
save([opFOMPath opFOM]);  % Save changes
disp('Script complete!')

end