function fomUTLCcsv(inFile,outFilePrefix)

% Written by S.R. Jim, (C) October 2013
% Glancing Angle Deposition (GLAD) Laboratory, PI: M.J. Brett
% University of Alberta, Edmonton, Canada
% 
% Purpose:  Export already calculated UTLC figures-of-merit as *.csv files.
% These FOMs were calculated from UTLC development videos using MATLAB
% scripts createRecipeUTLC.m, mainUTLC.m, and fomUTLC.m.  This script can
% be called by batchAnalyzeVideoUTLC.m.
% 
% Input:  A file generated by fomUTLC.m that contains 2D, 3D, or 4D arrays
% of calculated figures of merit.
% 
% Optional Inputs (those from batchAnalyzeVideoUTLC.m):
%   - Full file path (including file name) to an exisiting *.mat file
%   containing the FOMs to be exported to *.csv.
%   - Full file path and file name prefix for exported *.csv files
% 
% Outputs:
%   - This script exports one *.csv file for each FOM contained in the
%   input file (if it exists).  The file name is of the form:
%               *_(fomname)-(dim1)-(dim2)-(dim3).csv
%   with:
%       - (fomname):  Name of the FOM variable (ex. spotHRF)
%       - (dim1), (dim2), (dim3):  Dimensions of (3D) output array before
%       flattening into a *.csv (ex. 300, 3, 6)
%   example: test_spotHRF-300-3-6.csv
%   - It also exports a *.txt file containing information about the
%   all performed analysis

%% Set file directories
close all;
disp(' ')
disp('PART 0:  SET INPUT FOM DATA AND OUTPUT *.CSV FILE DIRECTORIES/FILES')
disp(' ')

% Select and load main UTLC file (may be in function argument)
if exist('inFile','var') && logical(exist(inFile,'file'))
    [ipPath0,ip0,ext] = fileparts(inFile);
    ip0 = [ip0 ext];
    clear inFile ext;
    ipPath0 = [ipPath0 '\'];
else
    disp('Select input FOM data file (output by fomUTLCv*.m)')
    [ip0,ipPath0] = uigetfile('*.mat', ...
        'Select input data file (output by fomUTLCv*.m)');
end

% Load variables to be exported to *.csv into a structure.  Missing arrays
% are listed in the command window but do not interupt script.
fom = load([ipPath0 ip0], ...
    'spotVolume1','spotMass1', ...  % Spot loading informaton
    'RealPH','RealPN','SN','zf1', ... % 2D arrays then 3D arrays (below)
    'ampSpot','sigSpot','spotFWHM','spotHRF','spotLOD0', 'spotLOD', ...
    'spotPH','spotPN','spotSNR','stdevBlankScaled','stdResiduals', ...
    'stdSpot','zSpot', ...
    'resSpot'); % 4D array

% Load the rest of the FOM data for other analysis
load([ipPath0 ip0]);
disp(['Loaded input data file ''' ip0 '''.'])

% Specify output file name prefix if not defined as a string in the
% function argument.
disp(' ')
if exist('outFilePrefix','var')
    [outPath,outPrefix,~] = fileparts(outFilePrefix);
else
    disp('Define location and prefix of output FOM *.csv files.')
    outPath = uigetdir(ipPath0,['Select directory in which to '...
        'place output FOM *.csv files.']);
    outPrefix = input('Enter output data *.csv file name prefix: ','s');
    outFilePrefix = [outPath '\' outPrefix];
    disp(' ')
end
disp('Output FOM *.csv files will have prefix:')
disp(outPrefix)
disp('and will be saved in directory:')
disp([outPath '\'])

%% Export *.csv files
disp(' ')
disp('PART 1:  WRITE INFO *.txt AND EXPORT FOMS TO *.csv FILES')
disp(' ')

% Turn on diary file
diary([outFilePrefix '-info.txt']);

% Information about the video frames
disp('UTLC Video Analysis Information')
disp(['Data set name: ' outPrefix])
disp(' ')
disp('BASIC INFORMATION ........................................')
disp(['Number of frames: ' num2str(nFrames)])
disp(['    Extracted at frame rate: ' num2str(frameRate) ' fps'])
disp(['    Pixel dimensions of frame (H x W): ' ...
    num2str(size(firstFrame,1)) ' x ' num2str(size(firstFrame,2)) ' px'])
disp(['    Assumed frame resolution: ' num2str(frameRes) ' px/mm'])
disp(['    First frame: ' filename(1).name])
disp(['    Last frame: ' filename(nFrames).name])
disp(' ')
disp(['Number of separation tracks: ' num2str(nTracks1)])
disp('Number of blank tracks (between separation tracks) ')
disp(['    used in noise calculation: ' ...
    num2str(nBlankTracks1)])
disp([num2str(nSpots1) ' analyte colour filters were analyzed:'])
disp(hueRangeLabel1)
disp('Migration distances (''zSpot'') and front positions (''zf1'')')
disp('    calculated with respect to the initial spot position.')
disp('Signal magnitude (used in SNR calculations) was the area ')
disp('    underneath the fitted colour-filtered chromatogram peak ')
disp(['    integrated over a ' num2str(intLength) ' peak standard '...
    'deviation wide region'])
disp('    centred about the peak position.')
disp(' ')
% Spot loading information
if isfield(fom,'spotVolume1') || isfield(fom,'spotMass1')
    disp('SPOT LOADING INFORMATION .................................')
    if isfield(fom,'spotVolume1')
        disp(['Spot volumes applied to ' num2str(nTracks1) ...
            ' tracks are (nL):'])
        disp(spotVolume1)
    end
    if exist('undilutedConc','var') && exist('relConc','var')
        disp(['Applied sample contained analytes with undiluted ' ...
            'concentrations (ng/nL):'])
        disp(undilutedConc)
        disp(['This was then diluted to ' num2str(100*relConc) ' %.'])
    end
    if isfield(fom,'spotMass1')
        disp(['The applied masses for all analytes (rows) and tracks ' ...
            '(columns) are (ng):'])
        disp(spotMass1)
    end
end
% LOD information
if isfield(fom,'spotLOD0') && isfield(fom,'spotLOD')
    disp('LIMITS OF DETECTION ......................................')
    disp(['"' num2str(lodcriteria) ' sigma" criteria was used to ' ...
        'calculate LODs.'])
    disp('''spotLOD0'' was calculated by extrapolating SNR vs. mass data')
    disp(['    to the "SNR = ' num2str(lodcriteria) '" level.'])
    disp('''spotLOD'' was calculated by extrapolating signal vs. mass ')
    disp(['    data in logarithmic space to the ' ...
        '"' num2str(lodcriteria) ' x noise" level.'])
    disp('Calculated LODs quoted in ng.')
end

% File exporting
disp(' ')
disp('FIGURE-OF-MERIT DATA EXPORTED TO *.csv FILES .............')
disp('(Dimensions on original 2D and 3D arrays listed in file names.)')
disp(' ')
% List of 2D and 3D FOMs to consider
fomList2D = {'RealPH'; 'RealPN'; 'SN'; 'zf1'};
fomList2D_mm = {'RealPH'; 'zf1'};
fomList3D = {'ampSpot'; 'sigSpot'; 'spotFWHM'; 'spotHRF'; 'spotLOD0'; ...
    'spotLOD'; 'spotPH'; 'spotPN'; 'spotSNR'; 'stdevBlankScaled'; ...
    'stdResiduals'; 'stdSpot'; 'zSpot'};
fomList3D_mm = {'spotFWHM'; 'stdSpot'; 'zSpot'};

% Export spot loading information
if isfield(fom,'spotVolume1')
    csvwrite([outFilePrefix '_spotVolume1-1-' ...
        num2str(nTracks1) '.csv'],fom.spotVolume1);
    disp('''spotVolume1'' exported')
end
if isfield(fom,'spotMass1')
    csvwrite([outFilePrefix '_spotMass1-' ...
        num2str(nSpots1) '-' num2str(nTracks1) '.csv'], ...
        fom.spotMass1);
    disp('''spotMass1'' exported')
end

% Export fomList2D
disp(' ')
disp(['2D arrays of dimension ' num2str(nFrames) ' frames x ' ...
    num2str(nTracks1) ' tracks exported:'])
for j = 1:length(fomList2D)
    if isfield(fom,fomList2D{j})
        csvwrite([outFilePrefix '_' fomList2D{j} '-' ...
            num2str(nFrames) '-' num2str(nTracks1) '.csv'], ...
            fom.(fomList2D{j}));
        disp(['''' fomList2D{j} ''''])
    end
end
clear j;

% Export fomList3D
disp(' ')
disp(['3D arrays of dimension ' num2str(nFrames) ' frames x ' ...
    num2str(nSpots1) ' spots x ' num2str(nTracks1) ' tracks exported:'])
for j = 1:length(fomList3D)
    if isfield(fom,fomList3D{j})
        csvwrite([outFilePrefix '_' fomList3D{j} '-' ...
            num2str(nFrames) '-' num2str(nSpots1) '-' ...
            num2str(nTracks1) '.csv'], fom.(fomList3D{j}));
        disp(['''' fomList3D{j} ''''])
    end
end
clear j;

% Migration distances and plate height in microns
disp(' ')
disp('Export some figures-of-merit in real units (mm, um).')
for j = 1:length(fomList2D_mm)   % FOMs in mm
    if isfield(fom,fomList2D_mm{j})
        csvwrite([outFilePrefix '_' fomList2D_mm{j} '_mm-' ...
            num2str(nFrames) '-' num2str(nTracks1) '.csv'], ...
            (fom.(fomList2D_mm{j}) ./ frameRes) );  % Convert to mm
        disp(['''' fomList2D_mm{j} '_mm'''])
    end
end
clear j;
for j = 1:length(fomList3D_mm)   % FOMs in mm
    if isfield(fom,fomList3D_mm{j})
        csvwrite([outFilePrefix '_' fomList3D_mm{j} '_mm-' ...
            num2str(nFrames) '-' num2str(nSpots1) '-' ...
            num2str(nTracks1) '.csv'], ...
            (fom.(fomList3D_mm{j}) ./ frameRes) );  % Convert to mm
        disp(['''' fomList3D_mm{j} '_mm'''])
    end
end
clear j;
% Plate height in um
if isfield(fom,'spotPH')
    csvwrite([outFilePrefix '_spotPH_um-' num2str(nFrames) '-' ...
        num2str(nSpots1) '-' num2str(nTracks1) '.csv'], ...
        (fom.spotPH * 1000 ./ frameRes) );   % Convert to um
    disp('''spotPH_um''')
end

% Resolution
if isfield(fom,'resSpot')
    % Convert 4D array into 3D array
    % Original coordinates: spot1,(spot2-1), frames, tracks
    % New coordinates: pair of spots1&2, frames, tracks where pairing
    % gives columns for:
    %   1vs2, 1vs3, ..., 2vs2, 2vs3, ..., 3vs2, 3vs3, ..., (N-1)vsN
    % (Note some nonsensical and redundant pairings still exist but have 
    % all values = NaN)
    resSpotCSV = permute(fom.resSpot,[2 1 3 4]);
    resSpotCSV = reshape(resSpotCSV, ...
        [1 (nSpots1-1)^2 nFrames nTracks1]);
    resSpotCSV = permute(resSpotCSV,[3 2 4 1]);
    
    % Remove nonsensical and redunant pairings (all values = NaN)
    % Define a vector with 1's where columns should be removed
    % Lower triangular matrix similar to NaN-ed areas at base of original
    % 4D resSpot array
    redundant = tril(ones(nSpots1-1,nSpots1-1),-1)';
    redundant = reshape(redundant,[1 (nSpots1-1)^2]);
    redundant = logical(redundant);
    resSpotCSV(:,redundant,:) = []; % Delete these columns from 3D array
    % Write to *.csv file
    csvwrite([outFilePrefix '_resSpot-' ...
        num2str(nFrames) '-' num2str(size(resSpotCSV,2)) '-' ...
        num2str(nTracks1) '.csv'], resSpotCSV);
    % Label columns in diary file
    % Spot #1: ex: [1 1 1 2 2 2 3 3 3]
    spotLabel1 = repmat([1:(nSpots1-1)],[(nSpots1-1) 1]);
    spotLabel1 = reshape(spotLabel1,[1 (nSpots1-1)^2]);
    % Spot #2: ex: [2 3 4 2 3 4 2 3 4]
    spotLabel2 = repmat([2:nSpots1],[1 (nSpots1-1)]);
    spotLabel = [spotLabel1; spotLabel2];
    spotLabel(:,redundant) = [];
    % Display exported information
    disp(' ')
    disp('''resSpot'' exported.  This 2D array has dimensions:')
    disp(['    ' num2str(nFrames) ' frames x ' ...
        num2str(size(spotLabel,2)) ' spot pairings x ' ...
        num2str(nTracks1) ' tracks.'])
    disp('Columns correspond to pairings between the spot listed in the ')
    disp('    top row with that in the bottom row:')
    disp(spotLabel)
    disp('    ex: Column 1 of exported ''resSpot'' array denote ')
    disp('    between Spot 1 and Spot 2.')
    clear spotLabel*
end

% Processing history:
disp(' ')
disp('DATA PROCESSING HISTORY ..................................')
disp(['Recipe file name: ' newRecipe])
disp(['    Generated using: ' script_Recipe])
disp(['    On: ' runDate_Recipe])
disp(['Main UTLC file name: ' newRecipe])
disp(['    Generated using: ' script_mainUTLC])
disp(['    On: ' runDate_MainUTLC])
disp(['FOM UTLC file name: ' newRecipe])
disp(['    Generated using: ' script_fomUTLC])
disp(['    On: ' runDate_fomUTLC])
disp('FOM files output to *.csv using: fomUTLCcsv.m rev October 2013')
disp(' ')
disp(['END OF INFO FILE, ' datestr(clock, 'yyyy-mm-dd, HH:MM')])

% End of script
diary off;
clear all;
end