function  [chi,zStart,tStart,zf_measured,time] = ...
    frontFit(filepath,filename,zStart_guess,tStart_guess,...
    chitrkloc1,chitrkloc2,nSteps,enh)

% Written by S.R. Jim, (C) October 2013
% Glancing Angle Deposition (GLAD) Laboratory, PI: M.J. Brett
% University of Alberta, Edmonton, Canada
% 
% Purpose:  This script is designed to calculate the migration constant chi
% (in units px^2/frame).  It considers a region (defined by chitrkloc1)
% cropped from select frames in a given directory (filepath).  The user
% identifies the migration front by eye.  These measurements are used to
% fit the required chi, zStart, and tStart scalar values.  (Another larger
% region, chitrkloc2, is shown above the R,G,B,L curves generated using the
% region defined by chitrkloc1.)
% 
% Notes:
%   - Based on getChi2.m written by A.J. Oko, October 2011
%   - This revision uses an equation form that includes parameters (chi, 
%   zStart, tStart) to satisfy the equation zf(t):
%       zf = sqrt(chi .* (t + tStart)) + zStart
%   This equation acknowledges that:
%       - the immersion line from which the solvent flows (~ zStart) 
%       depends on the position of the UTLC plate.
%       - the solvent started flowing before the first frame.  It started 
%       ~ tStart frames ago.

%% Error check
close all;
nFrames = length(filename);
if nFrames < 5
    disp('Insufficent total number of frames.  There must be at least 5')
    disp('to give the 3 steps (samples) required for fitting.')
    disp('Setting fitted values to NaN. ''zf_measured'', ''time'' not ')
    disp('measured.')
    chi = NaN; zStart = NaN; tStart = NaN;
    return
end
if ~exist('nSteps','var') || ( nSteps > sqrt(2*nFrames) ) || ...
        (nSteps < 3)
    disp('No valid ''nSteps'' value entered.')
    disp(['Frame directory contains ' num2str(nFrames) ' frames.'])
    disp(['Valid ''nSteps'' must be within [3,' ...
        num2str(floor(sqrt(2*nFrames))) '] (number of fitting'])
    disp('parameters and sqrt(2*nFrames).')
    nSteps = min(floor(sqrt(2*nFrames), 10)); % Default value for nSteps
    disp(['Assume a default value of nSteps = ' num2str(nSteps)])
end

%% Select frames
nSteps = round(nSteps);     % Ensure nSteps in an integer
% Define fselect vector to select frames.  Ensure that the first frame is
% always selected.
test = round(nFrames/nSteps^2);
if (test == 1)
    fselect = (1:nSteps)' / nSteps;
    fselect = round(nFrames*(fselect.^2));
else % test > 1
    fselect = zeros(nSteps,1);
    fselect(2:nSteps) = (1:(nSteps-1))' / nSteps;
    fselect = round(nFrames*(fselect.^2));
    fselect(1) = 1;
end
clear test;

A = imread([filepath filename(1).name]);
fmHeight = size(A,1); fmWidth = size(A,2);

%% Define regions of interest (if required)

% ROI used to generate R,G,B,L plots
good_chitrkloc1 = exist('chitrkloc1','var') && ...
    (size(chitrkloc1,1) == 1) && (size(chitrkloc1,2) == 4) && ...
    (min(chitrkloc1(:)) >= 0) && ...
    (max(chitrkloc1(1)) <= fmWidth) && ...   % Ensure x,width<=fmWidth
    (max(chitrkloc1(3)) <= fmWidth) && ...
    (max(chitrkloc1(2)) <= fmHeight) && ...  % Ensure y,Height<=fmHeight
    (max(chitrkloc1(4)) <= fmHeight);
if ~good_chitrkloc1
    disp(' ')
    disp('No valid region of interest (''chitrkloc1'') row vector')
    disp('defined. Crop area to be used to find front position. Recommend')
    disp('cropping centre two tracks and regions to left/right of these.')
    disp('tracks.')
    clf;
    [~,chitrkloc1] = imcrop(A);
end
% Correct chitrkloc1 if required (ensure it covers full frame height)
chitrkloc1 = round(chitrkloc1);
chitrkloc1(2) = 0; chitrkloc1(4) = fmHeight;

% ROI used to in track image preview above plots.
good_chitrkloc2 = exist('chitrkloc2','var') && ...
    (size(chitrkloc2,1) == 1) && (size(chitrkloc2,2) == 4) && ...
    (max(chitrkloc2(2)) <= fmHeight) && ...  % Ensure y,Height<=fmHeight
    (max(chitrkloc2(4)) <= fmHeight);
    % Note: if x,width are invalid, they will be refined below. Permit
    % negative x-coordinates
if ~good_chitrkloc2
    disp(' ')
    disp('No valid region of interest (''chitrkloc2'') row vector ')
    disp('defined. This zone will be shown with plotted signals.')
    disp('Option A: Crop image to be used to display above plots. ')
    disp('    Note: If necessary, crop may be enlarged to a region ')
    disp('    that encloses ''chitrkloc1'' in its entirety.')
    disp('Option B: Use ''chitrkloc1'' as ''chitrkloc2''.')
    chitrkloc2 = input('    Select option A or B (a/b)?: ','s');
    if strcmp(chitrkloc2,'a')
        clf;
        [~,chitrkloc2] = imcrop(A);
    else
        chitrkloc2 = chitrkloc1;
        disp('   ''chitrkloc2'' set to ''chitrkloc1''')
    end
end

% Correct chitrkloc2 if required (ensure it encloses chitrkloc1 and covers
% full frame height)
chitrkloc2 = round(chitrkloc2);
chitrkloc2_Test = chitrkloc2;

chitrkloc1_xL = chitrkloc1(1);  % left,right edges of chitrkloc1
chitrkloc1_xR = chitrkloc1(1) + chitrkloc1(3) - 1;
chitrkloc2_xL = chitrkloc2(1);  % left,right edges of chitrkloc2
chitrkloc2_xR = chitrkloc2(1) + chitrkloc2(3) - 1;
chitrkloc2_xL = max(0, min(chitrkloc1_xL,chitrkloc2_xL) );
chitrkloc2_xR = min(fmWidth, max(chitrkloc1_xR,chitrkloc2_xR) );
chitrkloc2_w = chitrkloc2_xR - chitrkloc2_xL + 1;

chitrkloc2 = [chitrkloc2_xL 0 chitrkloc2_w fmHeight];

if max(max(chitrkloc2 ~= chitrkloc2_Test))
    disp(' ')
    disp('Original ''chitrkloc2'' region: ')
    disp(chitrkloc2_Test)
    disp('Modified to:')
    disp(chitrkloc2)
    disp('To fit within frame and enclose ''chitrkloc1'' region: ')
    disp(chitrkloc1)
end
clear chitrkloc2_Test;

%% Ask about image enhancements performed on track preview (chitrkloc2)
if ~exist('enh','var') || (~strcmp(enh,'y') && ~strcmp(enh,'n'))
    disp(' ')
    disp('Perform R,G,B histogram equalization on previewed region')
    enh = input('defined by ''chitrkloc2'' (y/n)?: ','s');
end

%% Measure front position in selected images
zf_measured = zeros(nSteps,1);	% Initialize zf_measured vector
disp(' ')
disp('Measure migration front positions.  If unable to identify, select')
disp('a point outside left or outside right of the image/plot.  (ie. ')
disp(['select a position < 0 [px] or > ' num2str(fmHeight) ' [px].)'])
disp(' ')
for n = 1:nSteps
    % Import and crop image to chitrkloc1 - Used to generate R,G,B,L curves
    img0 = imread([filepath filename(fselect(n)).name]);
    A = imcrop(img0, chitrkloc1);
    
    % Enhance track image preview (optional) and crop preview to display
    % above plotted curves (using chitrkloc2)
    if ~strcmp(enh,'n')
        img0(:,:,1) = histeq(img0(:,:,1),256);
        img0(:,:,2) = histeq(img0(:,:,2),256);
        img0(:,:,3) = histeq(img0(:,:,3),256);
    end    
    A2 = imcrop(img0, chitrkloc2);
    
    % Display rotated track image and title
    fig = figure(1);
    set(fig,'Position', [100 100 1100 800]);
    clf; sp = zeros(1,2);   % Initialize subplot handles
    sp(1) = subplot(2,1,1);
    % Use image() instead of imshow() since it permits non-square pixels.
    % This allows the image to stretch to fill the subplot and match the
    % axis of the subplot(2,1,2) below
    image(imrotate(A2,90)); axis on;
    set(sp(1),'Position',[0.1 0.6 0.85 0.35]);
    title(['Measurement ' num2str(n) ' of ' num2str(nSteps) ...
        ' cropped from frame#: ' num2str(fselect(n)) ': ' ...
        filename(fselect(n)).name ' (rotated)']);
    % Superimpose rectangle representing track on image
    trkrect = zeros(1,4);   % Coordinates of chitrkloc1 rectangle mapped 
                            % onto image cropped by chitrkloc2.
    trkrect(2) = chitrkloc2_xR - chitrkloc1_xR + 1;
    trkrect(4) = chitrkloc1(3);
    trkrect(3) = fmHeight;
    hold on;
    rectangle('Position',trkrect,'EdgeColor','r');  % Plot rectangle
    
    % Calculate and plot lightness [0,255]
    L = 0.5 * (double(max(A,[],3)) + double(min(A,[],3)) );
    sp(2) = subplot(2,1,2); hold on;
    set(sp(2),'Position',[0.1 0.1 0.85 0.5]);
    plot(sum(A(:,:,1),2),'-r');     % Add RED signal
    plot(sum(A(:,:,2),2),'-g');     % Add GREEN signal
    plot(sum(A(:,:,3),2),'-b');     % Add BLUE signal
    plot(sum(L,2),'-k');            % Add LIGHTNESS signal
    xlabel('Position [px]'); ylabel('Summed R,G,B,L [0,255] values');
    legend('RED','GREEN','BLUE','LIGHTNESS', 'Location', 'NorthEast');
    title('Curves produced across track region (red rectangle above)')
    hold off;
    linkaxes(sp,'x'); xlim([0 fmHeight]);
    
    % Click on the front position to retrieve x-position.  Note that ginput
    % returns x,y positions within the subplot in which the user clicks.
    % Outside of the subplots, ginput returns positions that would be
    % measured on the current axes (ie. those of subplot 2 above). However,
    % since the axes are linked, the user may click anywhere within the
    % figure to retreive the correct x-position.
    [zf_measured(n),~] = ginput(1);
    disp(['Measured front position (frame: ' num2str(fselect(n)) ...
        ') = ' num2str(zf_measured(n)) ' px'])
    % Discard value if it is negative, greater than the frame length, or
    % lower than in previous frames (ie. not the largest value)
    if (zf_measured(n) < 0) || (zf_measured(n) > fmHeight) ...
            || (zf_measured(n) ~= max(zf_measured))
        disp('    Invalid:  Could not identify front position.')
        disp('    Set value to ''NaN'' and exclude from fit.')
        zf_measured(n) = NaN;
    end
end

%% Fit the chi, zStart, and tStart values
zf_measured(zf_measured == 0) = NaN;
time = fselect - 1; % in units of "frames"

% Edit erroneous data (NaN and <0) out of data used in fits
gooddata = (time >= 0) & (zf_measured >= 0);
zf_measuredFIT = zf_measured(gooddata);
timeFIT = time(gooddata);

% Define guesses for fitting (chi_guess is the average of measured values;
% zStart_guess is the initial spot position; tStart_guess is the migration
% time before the first frame (in "frames")
if ~exist('tStart_guess','var') || (tStart_guess < 1)
    tStart_guess = 1;   % A non-zero guess at the elapsed time
end
if ~exist('zStart_guess','var')
    zStart_guess = zf_measuredFIT(1); % A non-zero guess at the zStart
end
chi_guess = mean( (zf_measuredFIT - zStart_guess).^2 ./ ...
    (timeFIT + tStart_guess) );
guess = [chi_guess zStart_guess tStart_guess];

% Fit measured data to extract chi, zStart, tStart scalar values.  Note
% that abs(tStart) ensures that only positive tStarts are returned.
% (tStart < 0 is nonsensical; it suggests that the elution has not started
% yet.)
migration = @(p,x) sqrt(p(1) .* (x + abs(p(3)))) + p(2);
output = nlinfit(timeFIT,zf_measuredFIT,migration,guess);
chi = output(1);
zStart = output(2);
tStart = abs(output(3));    % Ensure positive

%% Plot measured and fitted values
t = (1:nFrames)' - 1;
zf0 = migration([chi zStart tStart],t);

figure(2); clf; hold on;
plot(timeFIT,zf_measuredFIT,'rx');  % Measurements
plot(t,zf0,'b-');                   % Fitted equation
title('Fitted migration front versus measurements');
xlabel('Frames'); ylabel('Front position wrt top of frame  [px]')
hold off;

clearvars -except chi zStart tStart zf_measured time;
end