function [params,roi,res,resStdev,status] = ...
    iterativeGaussFit(x,y,roi0,iter,crit)

% Written by S.R. Jim, (C) October 2013
% Glancing Angle Deposition (GLAD) Laboratory, PI: M.J. Brett
% University of Alberta, Edmonton, Canada
% 
% Purpose:  This script is used to fit a Gaussian peak to a region of an
% input data vector.  It returns numerical parameters for the Gaussian fit
% if the following criteria are satisfied:
%   1. The fitted amplitude is more than 6 times greater than the standard
%   deviation of the residuals (calculated over fitting region).
%   2. The fitting region is wide enough to ensure that, with 99% (3 
%   sigma) certainty, at least the top half of the fitted peak is within 
%   the fitting region.  ie.  The FWHM is contained within the fitting 
%   region.  (The standard deviation of the residuals are used to estimate
%   this error, sigma.)
% If the criteria are not satisfied, a new broader fitting region is
% defined (based on the old fitted parameters) and fitting is attempted
% again over this region.  The difference between subsequently fitted peak
% standard deviations is used as a convergence criteria.
% 
% Inputs:
%   x    : Input position column vector
%   y    : Raw signal column vector to be fit
%   roi0 : Row vector defining inclusive x-bounds on y over which the first
%       fitting iteration is performed.
%   iter : Maxiumum number of fitting iterations
%   crit : Covergence criteria for the fitted peak standard deviations 
%       (crit = abs( ( stdev_N - stdev_(N-1) ) / stdev_(N-1) )
%       ex. crit = 0.01 means that the current and previous stdevs differ
%       by less than 1%.
% 
% Outputs:
%   params : Array containing fitted values for each iteration.  Columns
%       represent fitted Gaussian parameters [mu stdev amp].  One row for
%       each fitting iteration (ordered final 1, 2, ..., N, ..., iter) so
%       that the final *non-zero* row contains the final fitted values:
%       [stdev mu amp].  1 row padded with zeros for each incomplete
%       iteration exists below the final fitted values.
%   roi : Array defining the inclusive x-bounds on y over which fitting was
%       performed (1 row for each iteration like for params). If
%       successful, the last non-zero row is also the region over which the 
%       standard deviation in the residuals is calculated.
%   res : Residuals (fit - raw) calculated over ENTIRE signal.  Reported
%       only for the final fit.  (No NaNs present.)
%   resStdev : Standard deviation in residuals (fit - raw) calculated ONLY
%       over the fitting region.  Reported only for final fit.
%   status : A cell array containing strings to report status of iterative
%       fitting.  Rows contain:
%           'fail 1: insufficent data within roi0' : Cannot be fit, too 
%               little data in roi0.
%           'fail 2: fitted peak centre outside x' : Nonsensical fitted 
%               peak centre that would cause divergence of subsequent 
%               fitting attempts.
%           'fail 3: amp<#*stdev' : Unacceptable fit, too noisy.
%           'fail 4: roi too small' : Unacceptable fit, roi too small
%               for reliable fit.
%           'fail 1-4: repeat of failed iteration' : Previous fitting
%           region limits were tried already.
%           'iter # failed, revert to last acceptable iter #' : Use last
%           acceptable iteration.
%           'acceptable, continue iter' : Fit statisfies both acceptance
%               criteria but continue improving fitted values.
%           'stop: iter = max iter' : Stop fitting, reached max iterations.
%           'stop: conv crit satisfied' : Stop iteration; fitted values
%               stable.

%% Iteration parameters
smoothSpan = 5;     % Smooth y only for initial guess
h = 2;              % Perform calculations using points at half height
N_stdev0 = 3;       % Number of guessed stdev0 away from mu0 each bound on 
                    % roi should be set initially
N_res_sig = 3;      % Number of resStdev used when calculating tolerance in
                    % checking if enough of peak is in fitting region

%% Error checking
% Check position and raw signal vectors
if ~exist('x','var') || ~exist('y','var') || (size(x,2) ~= 1) || ...
        (size(y,2) ~= 1) || (length(x) ~= length(y)) 
    error('Position (x) and/or signal (y) column vectors are invalid.')
end

% Ensure x is monotonic
if ~all(diff(x)>=0)
    tempXY = [x y];
    tempXY = sortrows(tempXY,1);
    x = tempXY(:,1); y = tempXY(:,2);
    disp('Sorted x-positions to ensure that they are monotonic.')
end
clear tempXY;

% Check roi0 and assume full length of vector if invalid
if ~exist('roi0','var') || (size(roi0,1) ~= 1) || (size(roi0,2) ~= 2) ...
        || (min(roi0) < min(x)) || (max(roi0) > max(x))
    disp('Invalid ''roi0'' defined:')
    disp(roi0)
    disp('Redefine ''roi0'' as:')
    roi0 = [max(min(roi0),min(x)) min(max(roi0),max(x))];
    disp(roi0)
end
roi0 = [min(roi0) max(roi0)];

% Insufficient data within roi0 (at least 4 data points are required for
% fitting of 3-parameter function)
if sum((x>=roi0(1)) & (x<=roi0(2))) < 4
    params = [NaN NaN NaN];
    roi = roi0;
    res = NaN .* ones(size(y));
    resStdev = NaN;
    status = cellstr('fail 1: insufficent data within roi');
    return;
end

% Check input iteration
if ~exist('iter','var') || (length(iter) ~= 1) || (iter < 1) || ...
        (floor(iter) ~= ceil(iter))
    disp(['Invalid ''iter'' defined:' num2str(iter)])
    iter = max(1,ceil(iter));
    disp(['Redefine ''iter'' as:' num2str(iter)])
end

% Check input convergence criteria
if ~exist('crit','var') || (length(crit) ~= 1) || (crit <= 0) || ...
        (crit >= 1)
    disp(['Invalid ''crit'' defined:' num2str(crit)])
    disp('Clear ''crit''/''iter'' and perform only one fitting iteration.')
    clear iter crit;
    iter = 1;
end

%% Isolate region of signals defined by roi0 and generate initial guesses
% Base guesses on smoothed signal to reduce chance of centering about a
% noisy spike or discontinuity.  Isolate roi0 by masking parts of signal
% outside of roi0 with NaNs
y1 = smooth(y,smoothSpan);
y1(x < roi0(1)) = NaN;
y1(x > roi0(2)) = NaN;

% Guess peak centre (mu) and height (amp); define signals to the left and
% right of the peak
[amp,ind] = max(y1);    % maximum and index of maxium value
mu = x(ind);            % x-position of y-maximum
y1_L = y1;              % sub-region of y1 with values right of max = NaN
y1_L(x > mu) = NaN;
y1_R = y1;              % sub-region of y1 with values left of max = NaN
y1_R(x < mu) = NaN;

% Points used to find FWHM guess
ind_h_L = find((y1_L < amp/h),1,'last');
ind_h_R = find((y1_R < amp/h),1,'first');

% Use selected points to guess standard deviation
if  ~isempty(ind_h_L) && ~isempty(ind_h_R)      % guess from FWHM
    fwhm0 = x(ind_h_R) - x(ind_h_L);
    stdev = fwhm0 / (2*sqrt(2*log(h)));
elseif ~isempty(ind_h_L) && isempty(ind_h_R)    % guess using only left pt
    fwhm0 = 2 * (mu-x(ind_h_L));
    stdev = fwhm0 / (2*sqrt(2*log(h)));
elseif ~isempty(ind_h_R) && isempty(ind_h_L)    % guess using only right pt
    fwhm0 = 2 * (x(ind_h_R)-mu);
    stdev = fwhm0 / (2*sqrt(2*log(h)));
else                                    % guess roi0 as full width of roi0
    stdev = roi0(2) - roi0(1); 
end

% Guess initial bounds for roi
roi = zeros(iter,2);   % initialize
roi(1,1) = max(min(x), (mu - N_stdev0*stdev));
roi(1,2) = min(max(x), (mu + N_stdev0*stdev));

clear y1* ind* fwhm0;

%% Perform iterative gaussian fitting
n = 1;          % iteration number

% Initialize initial guesses
params = zeros(iter,3);     % Used to hold iterated fitted values
status = cell(iter,1);      % Used to hold status of fit
% Cell array temporarily holds residuals (column 1: over entire signal;
% column 2: inside fitting region, with NaNs outside of fitting region)
res = cell(iter,2);    

% Gaussian function definition: p(1) peak centre, p(2) standard deviation,
% p(3) amplitude.
gaussian = @(p,z) p(3)*exp(-0.5*(z-p(1)).^2./(p(2)^2));

while n <= iter
    % Insufficient data within roi (at least 4 data points are required for
    % fitting of 3-parameter function)
    if sum((x>=roi(n,1)) & (x<=roi(n,2))) < 4
        params(n,:) = [NaN NaN NaN];
        res = NaN .* ones(size(y));
        resStdev = NaN;
        status{n} = 'fail 1: insufficent data within roi';
        return;
    end
    
    % Verify that this roi has not been tried before and found to fail.
    % Search previous failed iterations in which the roi limits are
    % identical to the current limits.  If identical, the fit will never
    % converge so exit the while loop.
    roiTest1 = roi(:,1) == roi(n,1);
    roiTest2 = roi(:,2) == roi(n,2);
    roiTest = roiTest1 & roiTest2; % Has 1s where both roi values repeat
    roiTest(n) = 0; % Ignore current iteration
    roiTest = roiTest & ~strcmp(status,'acceptable, continue iter');
    if (n > 1) && any(roiTest)
        status{n} = 'fail 1-4: repeat of failed iteration';
        n = n+1;    % Increment for when previous accepted fit is taken
        break;
    end
    
    % Isolate fitting region from y
    y1 = y;
    y1(x < roi(n,1)) = NaN;
    y1(x > roi(n,2)) = NaN;
    
    % Surpress warnings that will produce bad fits (rejected anyways)
    warning('off','stats:nlinfit:IllConditionedJacobian');
    warning('off','stats:nlinfit:IterationLimitExceeded');
    warning('off','stats:nlinfit:ModelConstantWRTParam');
    warning('off','MATLAB:rankDeficientMatrix');
    
    % Perform fit
    guess = [mu stdev amp];
    params(n,:) = nlinfit(x,y1,gaussian,guess);
    params(n,2) = abs(params(n,2)); % Correct for fits with stdev < 0
    mu = params(n,1);               % Fitted peak centre
    stdev = params(n,2);            % Fitted peak standard deviation
    amp = params(n,3);              % Fitted peak amplitude
    
    % Restore warnings
    warning('on','stats:nlinfit:IllConditionedJacobian');
    warning('on','stats:nlinfit:IterationLimitExceeded');
    warning('on','stats:nlinfit:ModelConstantWRTParam');
    warning('on','MATLAB:rankDeficientMatrix');
    
    % Calculate residuals
    yFit = gaussian([mu stdev amp],x);
    res{n,1} = yFit - y;     % Residuals over ENTIRE signal
    res{n,2} = yFit - y1;    % Residuals over fitting region ONLY
    resStdev = nanstd(res{n,2},0);    % Standard deviation of residuals
    
    % Verify fitted peak centre is inside of x vector; otherwise diverges
    if (mu <= min(x)) || (mu >= max(x))
        params(n,:) = [NaN NaN NaN];
        roi(n,:) = [NaN NaN];
        res = NaN .* ones(size(y));
        resStdev = NaN;
        status{n} = 'fail 2: fitted peak centre outside x';
        return;
    end
    
    % Calculations used to determine acceptance and convergence
    % Minimum amplitude
    good_amp = abs(amp) > (N_res_sig * h * resStdev);
    % Improved distance between peak centre and edge of roi
    w1 = N_stdev0*stdev;
    % Minimum distance between peak centre and edge of roi
    w2 = sqrt(2*log(h*amp/(amp-N_res_sig*h*resStdev))) * stdev;
    goodL = roi(n,1) <= (mu - w2);
    goodR = roi(n,2) >= (mu + w2);
    % Convergence criteria
    err = abs( (stdev - guess(2)) / guess(2) );
    
    if ~good_amp                % Amplitude too small
        status{n} = ['fail 3: abs(amp)<' num2str(N_res_sig * h) ...
            '*stdev'];
        w = w1;                 % Cannot use w2 since it is imaginary
    elseif ~goodL || ~goodR     % roi is too small
        status{n} = 'fail 4: roi too small';
        w = max(w1,w2);
    elseif err <= crit          % Convergence criteria met
        status{n} = 'stop: conv crit satisfied';
        res = res{n,1};
        return;
    elseif n == iter            % Last iteration
        status{n} = 'stop: max iter';
        res = res{n,1};
        return;
    else
        status{n} = 'acceptable, continue iter';
        w = max(w1,w2);
    end    
    
    % Redefine roi for next iteration, if required/possible
    if n < iter
        roiL = find(x < (mu - w),1,'last');
        if isempty(roiL)
            roiL = 1;
        end
        roi(n+1,1) = max(min(x),x(roiL));     % New left limit of roi
        roiR = find(x > (mu + w),1,'first'); 
        if isempty(roiR)
            roiR = length(x);
        end
        roi(n+1,2) = min(max(x),x(roiR));     % New right limit of roi
    end
    
    % Increment iteraton number and continue
    n = n+1;
end

% The final fitting attempt failed.  (Other conditions would have already
% been returned.)  Substitute a previous acceptable fit if possible.
ok_iter = strcmp(status,'acceptable, continue iter');
if any(ok_iter)
    % Find index of last acceptable iteration
    ok_iter = find(ok_iter,1,'last');
    % Save fit status
    status{n-1} = ['iter ' num2str(n-1) ...
        ' failed, revert to last acceptable iter ' ...
        num2str(ok_iter)];
    % Copy parameters and recalculate residuals, stdev of residuals for
    % last acceptable iteration
    params(n-1,:) = params(ok_iter,:);
    resStdev = nanstd(res{ok_iter,2},0);    % Standard deviaton over roi
    res = res{ok_iter,1};                   % Residuals over entire signal
else
    % Invalidate fitted values but still report the most recent roi, res,
    % resStdev values
    params(n-1,:) = [NaN NaN NaN];
    res = res{n-1,1};                       % Residuals over entire signal
end

end