function mainUTLC(spec,inFile,outFile)

% Written by S.R. Jim, (C) October 2013
% Glancing Angle Deposition (GLAD) Laboratory, PI: M.J. Brett
% University of Alberta, Edmonton, Canada
% 
% Purpose:  This script is used to analyze a directory of frame *.jpg or
% *.tif files extracted from a UTLC development video.  It is used to
% extract colour-filtered chromatograms from tracks defined on each frame
% in the directory.  This script can be called by batchAnalyzeVideoUTLC.m.
%
% Input:  A recipe file prepared using createRecipeUTLC.m is selected.
% This contains all variables that may be relevant for this analysis.  The
% filepath to the directory of images is assumed to be the same as that
% identified in the recipefile (filepath).
% 
% Optional Inputs (those from batchAnalyzeVideoUTLC.m):
%   - spec:  A structure containing predefined values for inputs required
%   by this script.
%   - inFile:  Full filepath and name of the input recipe file.
%   - outFile:  Full filepath and name of the output main file.
%
% Outputs:  This script produces one output file.  This contains all data
% produced in this script and that included in the recipe file.
% 
% Subfunctions:  filteredDensitogram.m, iterativeGaussFit.m
% 
% Notes:
%   - Based on mainUTLCv7.mat written by A.J. Oko, September 2011
%   - Saving the Densitogram and fitDensitogram signals is optional. These
%   output signals may have lower sampling frequency than the frame rate.
%   (Ex. Only data from every 5 frames is selected, etc.)
%   - This script can also be used to save the raw blank pixel signals for
%   subsequent analysis (ex. histograms, etc.)
%   - Calculated blank track saturation value means and standard deviations
%   are calculated using filteredDensitogram.m using two different methods
%   (below).  They may be used in future system noise and LOD calculations.
%       - Consider all pixels in blank track.  Exclude pixels with hue
%       outside of the given hue filter.  Calculate mean and standard
%       deviation in remaining pixels' staturation values.
%       - Calculate a hue-filtered densitogram for the blank track as
%       usual.  Calculate the mean and standard deviation in this resultant
%       signal.

%% Set file directories
close all;
disp(' ')
disp('PART 0:  SET INPUT RECIPE AND OUTPUT DATA DIRECTORIES/FILES')
disp(' ')

% Select and load recipe file (May be defined in function argument)
if exist('inFile','var') && logical(exist(inFile,'file'))
    [ipRecipePath0,ipRecipe0,ext] = fileparts(inFile);
    ipRecipe0 = [ipRecipe0 ext];
    clear inFile ext;
else
    disp('Select input recipe file.')
    [ipRecipe0,ipRecipePath0] = uigetfile('*.mat', ...
        'Select input recipe file');
end
ipRecipePath0 = [ipRecipePath0 '\'];
load([ipRecipePath0 ipRecipe0]);
ipRecipe = ipRecipe0; ipRecipePath = ipRecipePath0;
clear ipRecipe0 ipRecipePath0;
disp(['Loaded input recipe file ''' ipRecipe '''.'])

% Specify output file name if not defined as a string in the function
% argument.  (Note the file itself doesn't need to exist yet.)
disp(' ')
if exist('outFile','var')
    [opFilePath,opFile,ext] = fileparts(outFile);
    opFilePath = [opFilePath '\'];
    opFile = [opFile ext];
    clear outFile ext;
    disp(['Specified output data file ''' opFile '''.'])
else
    disp('Define location and name of output file.  This contains data')
    disp('from input recipe file and calculations from this script.')
    opFilePath = uigetdir(ipRecipePath,['Select directory in which to '...
        'place output file.']);
    opFilePath = [opFilePath '\'];
    opFile = input('Enter output data file name (name.mat): ','s');
end

% Variable to record the script name and revision
script_mainUTLC = 'mainUTLC.m rev October 2013';

% Initialize output recipe file (verify that save is permitted)
save([opFilePath opFile]);

% Unpack predefined values for inputs stored in spec
if exist('spec','var') && ~isempty(spec)
    sVect = spec.sVect; % Analyte spots on each track to analyze
    tVect = spec.tVect; % Separation tracks to analyze
    tbVect = spec.tbVect; % Blank tracks to analyze
    saveData = spec.saveData; % Save also densitograms, blank data (y/n)?
    sampleFreq = spec.sampleFreq;     % Save data for 1 out of __ frames
    saveDensitogram = spec.saveDensitogram; % Raw/fitted densitograms?
    saveblankRaw = spec.saveblankRaw; % Blanks together as 1 signal
    saveblankRawTrk = spec.saveblankRawTrk; % Blanks as separate signals
end

% Set directory of UTLC video frames to analyze
disp(' ')
if ~logical(exist([filepath filename(1).name],'file')) || ...
        ~exist('spec','var')    % filepath/name don't exist, spec not input
    disp(['CONFIRM directory of frame *' img_ext ...
        ' images extracted from video'])
    % Assume location to be same as defined when creating recipe
    [img_ext,filepath] = uigetfile(...
        {'*.jpg;*.tif', '*.jpg or *.tif images'; ...
        '*.jpg', '*.jpg images'; ...
        '*.tif', '*.tif images'}, ...
        'Select a frame (*.jpg or *.tif) extracted from video.', ...
        [filepath filename(1).name]);
    [~,~,img_ext] = fileparts(img_ext);     % Find file extension
    filename = dir([filepath '*' img_ext]); % Filenames of given format
end

% Initialize relevant filename, image arrays, etc.
nFrames = length(filename);
disp(['Selected directory contains ' num2str(nFrames) ' frames.'])
disp(['The first frame is: ''' filename(1).name ''''])
disp(['The last frame is: ''' filename(nFrames).name ''''])

%% Display possible and set desired types of analyses
disp(' ')
disp('PART 1:  DEFINE SUBSET OF AVAILABLE DATA TO ANALYZE')
disp(' ')
nRows = size(lastFrame,1);
disp(['Input file ''' ipRecipe ''''])
disp(['contains data for ' num2str(nTracks) ' separation and ' ...
    num2str(nBlankTracks) ' blank tracks.'])
disp(['Defined hue ranges for ' num2str(nSpots) ' analytes in ' ...
    'descending hRF are:'])
disp(hueRangeLabel)

% Check validity of subsets defined in spec
good_sVect = exist('sVect','var') && ( strcmp(sVect,'all') || ...
    (isvector(sVect) && min(floor(sVect) == ceil(sVect)) && ...
    (min(sVect) >= 1) && (max(sVect) <= nSpots) && ...
    (length(sVect) == length(unique(sVect)))) );
good_tVect = exist('tVect','var') && ( strcmp(tVect,'all') || ...
    (isvector(tVect) && min(floor(tVect) == ceil(tVect)) && ...
    (min(tVect) >= 1) && (max(tVect) <= nTracks) && ...
    (length(tVect) == length(unique(tVect)))) );
good_tbVect = exist('tbVect','var') && ( strcmp(tbVect,'all') || ...
    (isvector(tbVect) && min(floor(tbVect) == ceil(tbVect)) && ...
    (min(tbVect) >= 1) && (max(tbVect) <= nBlankTracks) && ...
    (length(tbVect) == length(unique(tbVect)))) );

% Specify subset of data if desired and not already defined in spec
if ~good_sVect || ~good_tVect || ~good_tbVect
    confirm = 'n';
    while ~strcmp(confirm,'y')
        % Preview predefined separation and blank tracks
        disp('Preview separation and blank track regions on last frame.')
        clf; imshow(lastFrame); axis on; hold on;
        title('Predefined separation (black) and blank (blue) tracks')
        for i = 1:nTracks   % Overlay separation tracks and applied spots
            rectangle('Position', defineTrack(i,:),'EdgeColor','k');
            plot(spotLocationX(i),spotLocationY(i),'or');
        end
        for j = 1:nBlankTracks  % Overlay blank tracks
            rectangle('Position', defineBlank(j,:,nFrames), ...
                'EdgeColor','b');
        end
        hold off;

        % Define subset of separation tracks and hue ranges
        disp('Select a subset of tracks and hue ranges if desired.')
        disp('Enter row vectors with elements sorted in ascending order.')
        sVect = ...
            sort(input('    Analyte spots on each track to analyze: '));
        tVect = sort(input('    Separation tracks to analyze: '));
        tbVect = sort(input('    Blank tracks to analyze: '));

        % Select subset of relevant variables
        nTracks1 = length(tVect);
        defineTrack1 = defineTrack(tVect,:);
        spotLocationX1 = spotLocationX(tVect);
        spotLocationY1 = spotLocationY(tVect);
        zf1 = zf(:,tVect);
        nSpots1 = length(sVect);
        hueRange1 = hueRange(sVect,:);
        hueRangeLabel1 = hueRangeLabel(sVect);
        rfLimits1 = rfLimits(sVect,tVect,:);
        nBlankTracks1 = length(tbVect);
        defineBlank1 = defineBlank(tbVect,:,:);

        % Preview selected separation and blank tracks
        disp(' ')
        disp(['Selected hue ranges for ' num2str(nSpots1) ...
            ' analytes in descending hRF are:'])
        disp(hueRangeLabel1)
        disp('Preview separation and blank track regions on last frame.')
        clf; imshow(lastFrame); axis on; hold on;
        title('Selected separation (black) and blank (blue) tracks')
        for i = 1:nTracks1  % Overlay separation tracks and applied spots
            rectangle('Position', defineTrack1(i,:),'EdgeColor','k');
            plot(spotLocationX1(i),spotLocationY1(i),'or');
        end
        for j = 1:nBlankTracks1 % Overlay blank tracks
            rectangle('Position', defineBlank1(j,:,nFrames), ...
                'EdgeColor','b');
        end
        hold off;

        % Confirm values
        disp(' ')
        confirm = input('Confirm these values (y/n): ','s');
        if ~strcmp(confirm,'y')   % Clear selections and redefine
            clear sVect tVect tbVect;
            disp(' ')
        end 
    end
else    % Data defined in spec structure
    % Correct for when sVect, tVect, and/or tbVect = 'all'
    if strcmp(sVect,'all')  % sVect = 'all' case
        sVect = 1:nSpots;
    end
    if strcmp(tVect,'all')  % tVect = 'all' case
        tVect = 1:nTracks;
    end
    if strcmp(tbVect,'all') % tbVect = 'all' case
        tbVect = 1:nBlankTracks;
    end
    % Select subset of other relevant variables
    nTracks1 = length(tVect);
    defineTrack1 = defineTrack(tVect,:);
    spotLocationX1 = spotLocationX(tVect);
    spotLocationY1 = spotLocationY(tVect);
    zf1 = zf(:,tVect);
    nSpots1 = length(sVect);
    hueRange1 = hueRange(sVect,:);
    hueRangeLabel1 = hueRangeLabel(sVect);
    rfLimits1 = rfLimits(sVect,tVect,:);
    nBlankTracks1 = length(tbVect);
    defineBlank1 = defineBlank(tbVect,:,:);
    % Display values
    disp(['Selected hue ranges for ' num2str(nSpots1) ...
        ' analytes in descending hRF are:'])
    disp(hueRangeLabel1)
    disp('Selected separation tracks to analyze:')
    disp(tVect)
    disp('Selected blank tracks to analyze:')
    disp(tbVect)
end
close all;
clear good_sVect good_tVect good_tbVect confirm;

% Select subset of spot loading information if it exists
if exist('spotVolume','var')
    spotVolume1 = spotVolume(tVect);
else
    disp('''spotVolume'' data does not exist.')
end
if exist('spotMass','var')
    spotMass1 = spotMass(sVect,tVect);
else
    disp('''spotMass'' data does not exist.')
end

%% Initialize variables
x = (1:nRows)';             % Vector for position along tracks

% Raw and fitted densitogram data (temporary variables)
Densitogram0 = zeros(nRows,nSpots1);
fitDensitogram0 = zeros(nRows,nSpots1);

% Temporary row vectors used to store fitted parameters for spots on a
% given track in a given frame; also other fitting/calculated values
sigma = zeros(1,nSpots1);   % fitted peak stdev
mu = zeros(1,nSpots1);      % fitted peak centre
amp = zeros(1,nSpots1);     % fitted peak amplitude
zs = zeros(1,nSpots1);      % fitted peak centre wrt applied spot

% Arrays used to store relevant fitted peak data
stdSpot = zeros(nFrames,nSpots1,nTracks1);  % Width of peak (one sigma)
zSpot = zeros(nFrames,nSpots1,nTracks1);    % Peak centre wrt appllied spot
ampSpot = zeros(nFrames,nSpots1,nTracks1);  % Amplitude of fitted peak
roiSpot = zeros(nFrames,nSpots1,nTracks1,2);% Final ROI for fitting

% Standard deviation in fit-raw residuals.
stdResiduals = zeros(nFrames,nSpots1,nTracks1);

% Arrays to contain mean and standard deviaton of hue-filtered saturation
% values on each blank track and for all pixels extracted from blank tracks
avgBlankTrk = zeros(nFrames,nSpots1,nBlankTracks1);
stdevBlankTrk = zeros(nFrames,nSpots1,nBlankTracks1);
avgBlank = zeros(nFrames,nSpots1);
stdevBlank = zeros(nFrames,nSpots1);

% Similar arrays to contain mean and standard deviaton of hue-filtered
% saturation values on each blank track and for all pixels extracted from
% blank tracks.  However, these values are calculated from the extracted
% densitograms
avgBlankTrkD = zeros(nFrames,nSpots1,nBlankTracks1);
stdevBlankTrkD = zeros(nFrames,nSpots1,nBlankTracks1);
avgBlankD = zeros(nFrames,nSpots1);
stdevBlankD = zeros(nFrames,nSpots1);

disp(' ')
disp('Subsets defined, required variables initialized.')

%% Specify saving of Densitogram/fitDensitogram and/or raw blank pixel data
disp(' ')
disp('PART 2:  SPECIFY SAVING OF RAW/FITTED DENSITOGRAM SIGNALS')
disp(' ')

% Save raw data
if exist('saveData','var') && strcmp(saveData,'y')
    disp('Some or all raw/fitted signals will be saved')
elseif exist('saveData','var') && ~strcmp(saveData,'y')
    disp('None of the raw/fitted signals will be saved.')
    sampleFreq = NaN;
    saveDensitogram = 'n';
    saveblankRaw = 'n';
    saveblankRawTrk = 'n';
else
    disp('Save at least one of ''Densitogram''+''fitDensitogram'' and/or')
    disp('''blankRaw'' and/or ''blankRawTrk'' signals (y/n)? (Warning:');
    disp('These data arrays may require very large amounts of')
    saveData = input('memory!): ','s');
end

if strcmp(saveData,'y') && ( ~exist('sampleFreq','var') || ...
        ~isnumeric(sampleFreq) || isnan(sampleFreq) || ...
        ~exist('saveDensitogram','var') || ...
        ~exist('saveblankRaw','var') || ...
        ~exist('saveblankRawTrk','var') )
    % Data reduction
    disp(' ')
    disp('Sampling frequency (data reduction).  This data set contains')
    disp([num2str(nFrames) ' frames, extracted at a rate of ' ...
        num2str(frameRate) ' fps.'])
    disp(['This is a ' num2str((nFrames-1)/frameRate) ' s separation.'])
    disp('How should this data be sampled?  Input sampling rate.')
    disp('Ex. "Save one out of every 5 frames" saves 1, 6, 11, etc.')
    confirm = 'n';
    while ~strcmp(confirm,'y')
        sampleFreq = input('    "Save one out of every ___ frames": ');
        samples = (1:sampleFreq:nFrames);
        nSamples = length(samples);
        disp(['    Raw signals for ' num2str(nSamples) ...
            ' frames will be saved.'])
        % Ensure that sampleFreq is an integer (ceiling = floor)
        if  (ceil(sampleFreq) == floor(sampleFreq)) && ...
                (sampleFreq >= 1) && (sampleFreq <= nFrames)
            confirm = input('    Confirm selection (y/n)?: ','s');
        else
            disp('Invalid ''sampleFreq''.')
        end
    end
    
    % Specify which signals to save
    disp(' ')
    disp('Specify which signals should be saved at above ''sampleFreq''')
    confirm = 'n';
    while ~strcmp(confirm,'y')
        disp('    Save BOTH of the reduced ''Densitogram'' and')
        saveDensitogram = ...
            input('    ''fitDensitogram'' signals (y/n)?: ','s');
        disp(' ')
        disp('    COMBINE pixels from all blank tracks in a frame and ')
        disp('    produce a single signal for each hue range? (Use ')
        disp('    this when producing a single histogram for the entire ')
        saveblankRaw = input(['    population of blank pixels in a ' ...
            'given frame.) (y/n): '],'s');
        disp(' ')
        disp('    Produce one signal for each hue range on EACH SEPARATE ')
        disp('    BLANK TRACK?  (Use this when producing separate ')
        saveblankRawTrk = input(['    histograms for each blank track.)'...
            ' (y/n): '],'s');
        disp(' ')
        confirm = input('    Confirm selections (y/n)?: ','s');
        disp(' ')
    end
    clear confirm;
elseif strcmp(saveData,'y') && exist('sampleFreq','var') && ...
        exist('saveDensitogram','var') && ...
        exist('saveblankRaw','var') && exist('saveblankRawTrk','var')
    samples = (1:sampleFreq:nFrames);
    nSamples = length(samples);
    disp(['One set of raw/fitted signals will be saved every ' ...
        num2str(sampleFreq) ' frames.'])
end

if strcmp(saveData,'y')
    % Initialize required variables (cell arrays for blankRaw and
    % blankRawTrk because number of pixels depends on frame)
    if strcmp(saveDensitogram,'y')
        Densitogram = zeros(nRows,nSpots1,nSamples,nTracks1);
        fitDensitogram = zeros(nRows,nSpots1,nSamples,nTracks1);
        disp('    Initialized ''Densitogram''/''fitDensitogram'' arrays.')
    end
    if strcmp(saveblankRaw,'y')
        blankRaw = cell(nSamples,1);
        disp(['    Initialized (nSamples x 1 = ' num2str(nSamples) ...
            ' x 1) ''blankRaw'' cell array.'])
    end
    if strcmp(saveblankRawTrk,'y')
        blankRawTrk = cell(nSamples,nBlankTracks1);
        disp(['    Initialized (nSamples x nBlankTracks1 = ' ...
            num2str(nSamples) ' x ' num2str(nBlankTracks1) ...
            ') ''blankRaw'' cell array.'])
    end
else
    % Don't save any signals
    sampleFreq = NaN;
    saveDensitogram = 'n';
    saveblankRaw = 'n';
    saveblankRawTrk = 'n';
end

%% Confirm start and begin analysis
disp(' ')
disp('PART 3:  PROCESS FRAMES EXTRACTED FROM UTLC DEVELOPMENT VIDEO')
disp(' ')

% Automatically start if spec is defined
if ~exist('spec','var') || isempty(spec)
    process = input('Start processing data or abort (y/n)?: ','s');
    if ~strcmp(process,'y')
        error('Script aborted by Silly user... :-(');
    end
    clear process;
end

%% Begin analysis
tic; tstart = tic;             % Start timer

for f = 1:nFrames
    disp(['Now analyzing frame ' num2str(f) ' of ' num2str(nFrames)])
    rgbFrame = imread([filepath filename(f).name]);
    
    %% Extract colour-filtered densitograms and fit with gaussian
    
    for t = 1:nTracks1
        imgT = imcrop(rgbFrame,defineTrack1(t,:));

        % Calculate densitograms:  [D,avg_px,stdev_px,avg_D,stdev_D] = ...
        %   filteredDensitogram(img,hueRange)
        [Densitogram0(:,:),~,~,~,~] = filteredDensitogram(imgT,hueRange1);
        
        % Fit densitogram to gaussian function and numerically calculate
        % peak information
        for s = 1:nSpots1
            % GAUSSIAN PEAK FITTING
            % Define initial ROI for fit using rfLimits1
            roi0 = zeros(1,2);
            roi0(1) = spotLocationY1(t) + zf1(f,t)*rfLimits1(s,t,1);
            roi0(2) = spotLocationY1(t) + zf1(f,t)*rfLimits1(s,t,2);
            
            % [params,roi,res,resStdev,status] = ...
            %    iterativeGaussFit(x,y,roi0,iter,crit)
            % 10 iterations maxiumum, 1% convergence criteria
            [params,roiFitOutput,~,stdResiduals(f,s,t),~] = ...
                iterativeGaussFit(x,Densitogram0(:,s),roi0,10,0.01);
            
            ind = sum(params,2); % Vector to select final iteration values
            ind(ind==0) = [];    % Remove empty values
            ind = length(ind);   % Index of final iteration
            mu(s) = params(ind,1);
            zs(s) = mu(s) - spotLocationY1(t);
            sigma(s) = params(ind,2);
            amp(s) = params(ind,3);
            roiSpot(f,s,t,1) = roiFitOutput(ind,1);
            roiSpot(f,s,t,2) = roiFitOutput(ind,2);
            % Calculate fitDensitogram0 signal
            fitDensitogram0(:,s) = amp(s) * exp( -(x-mu(s)).^2 / ...
                (2*sigma(s)^2) );   % Equation for gaussian distribution
            clear ind params roi0 roiFitOutput;            
        end
        % Store the calculated values (fits and numerically calculated
        % values)
        stdSpot(f,:,t) = sigma;
        zSpot(f,:,t) = zs;
        ampSpot(f,:,t) = amp;
        
        % Save Densitogram0 and fitDensitogram0 signals to arrays if
        % desired and if the current frame is one that should be sampled
        if strcmp(saveDensitogram,'y') && max(samples == f)
            Densitogram(:,:,(samples == f),t) = Densitogram0;
            fitDensitogram(:,:,(samples == f),t) = fitDensitogram0;
        end
    end
    
    %% Consideration of blank pixels, noise (stdev) and average
    
    % 1 px wide image to hold all pixels extracted from the blank tracks
    blankpx = uint8( zeros(...
        dot((defineBlank1(:,3,f)+1),(defineBlank1(:,4,f)+1)), 1, 3) );
    
    % Analyze each blank track sepately and add all pixels to blankpx
    ind1 = 1;                               % Start insertion index
    for tb = 1:nBlankTracks1
        imgB = imcrop(rgbFrame,defineBlank1(tb,:,f));
        % Analyze each blank track's pixels separately; save signal if
        % desired. Note that filteredDensitogram.m allows calculation of
        % the average and standard deviation in two ways: From individual
        % pixels and from calculated densitogram
        % [D,avg_px,stdev_px,avg_D,stdev_D] = ...
        %   filteredDensitogram(img,hueRange)
        if strcmp(saveblankRawTrk,'y') && max(samples == f)
            [blankRawTrk{(samples == f),tb},...
                avgBlankTrk(f,:,tb),stdevBlankTrk(f,:,tb),...
                avgBlankTrkD(f,:,tb),stdevBlankTrkD(f,:,tb)] = ...
                filteredDensitogram(imgB,hueRange1);
        else
            [~,avgBlankTrk(f,:,tb),stdevBlankTrk(f,:,tb),...
                avgBlankTrkD(f,:,tb),stdevBlankTrkD(f,:,tb)] = ...
                filteredDensitogram(imgB,hueRange1);
        end
        % Reshape into 1 px wide image and append cropped blank pixels to
        % blankpx array
        imgB = reshape(imgB,[],1,3);
        ind2 = ind1 + size(imgB,1) - 1;     % Calculate end index
        blankpx(ind1:ind2,1,:) = imgB;      % Insert all of imgB pixels
        ind1 = ind2 + 1;                    % Increment start index
    end
    
    % Analyze entire set of blank track pixels together.  Save densitograms
    % if desired.  
    % [D,avg_px,stdev_px,avg_D,stdev_D] = filteredDensitogram(img,hueRange)
    if strcmp(saveblankRaw,'y') && max(samples == f)
        [blankRaw{(samples == f),1}, ...
            avgBlank(f,:),stdevBlank(f,:), ...
            avgBlankD(f,:),stdevBlankD(f,:)] = ...
            filteredDensitogram(blankpx,hueRange1);
    else
        [~,avgBlank(f,:),stdevBlank(f,:), ...
            avgBlankD(f,:),stdevBlankD(f,:)] = ...
            filteredDensitogram(blankpx,hueRange1);
    end
    
    clear imgT imgB ind1 ind2 blankpx;
    % Estimate time remaining
    testimate = toc(tstart) * (nFrames-f) / f;
    disp(['    Elapsed time is ' num2str(fix(toc)) ' seconds.'])
    disp(['    Estimated time remaining:   ' num2str(fix(testimate)) ...
        ' seconds.'])
end
clear i j f t s tb zs sigma amp mu residuals;
clear tstart testimate confirm;
clear leftLimit rightLimit Densitogram0 fitDensitogram0;
clear inFile outFile spec;

% Save calculated data
disp(' ')
runDate_MainUTLC = datestr(clock, 'yyyy-mm-dd, HH:MM');
disp(['Time stamp: ' runDate_MainUTLC])
disp(['Saving output file as: ''' opFile ''''])
save([opFilePath opFile]);   % Save changes
disp('***SUCCESS SCRIPT FINALLY DONE!!!***')

end