function [spotLoc,satSum] = sampleSpots(refFrame,spotLoc,...
    plot_data,exp_dir,exp_data,exp_plots,expName)

% Written by S.R. Jim, (C) March 2014
% Glancing Angle Deposition (GLAD) Laboratory, PI: M.J. Brett
% University of Alberta, Edmonton, Canada
% 
% This script can be used to identify the spot locations and hue ranges
% required during colour-filtered chromatogram extraction in the subsequent
% scripts:  createRecipeUTLC.m, mainUTLC.m, (fomUTLC.m)
% It is also a subfunction of singleImageUTLC.m
% 
% INPUTS:
%   - refFrame:  Reference image array of RGB (uint8) values.
%   - spotLoc:  Array used to define the regions from which spot pixels
%   should be cropped.
%   - plot_data:  Should data be displayed on plots ('y','n')?  (Must =
%   'y' in order to export plots below.)
%   - exp_dir:  Directory into which data *.mat and/or *.fig/*.png figures
%   will be exported.  (Skip by setting to 'n'.)
%   - exp_data:  Should data be exported as *.mat ('y','n')?
%   - exp_plots:  Should plots be exported as *.fig and *.png ('y','n')?
%   - expName:  Prefix of exported files (a string).

%% Load image file
disp('PART 0 : SELECT REFERENCE IMAGE AND PLOTTING/EXPORT OPTIONS')
disp(' ')
% Select reference image
if ~exist('refFrame','var') || ~isa(refFrame,'uint8') || ...
        (size(refFrame,3) ~= 3)
    disp('Select reference image used to identify hue ranges (*.jpg/.tif)')
    [imagename,path] = uigetfile(...
        {'*.jpg;*.tif', '*.jpg or *.tif images'; ...
        '*.jpg', '*.jpg images'; ...
        '*.tif', '*.tif images'}, ...
        'Select reference image used to identify hue ranges (*.jpg/.tif)');
    if imagename == 0
        error('Invalid reference image file, Silly!');
    end
    % Load/preview frame
    refFrame = imread([path imagename]);        % Load file
    disp(['Input reference image is: ' imagename])
end
fmHeight = size(refFrame,1);
fmWidth = size(refFrame,2);

% Preview reference frame
figure(100); clf; imshow(refFrame);
title('Reference frame image (to be sampled)');

% Plotting and export options
if exist('plot_data','var') && strcmp(plot_data,'y')
    disp('Hue measurement data will be plotted.')
elseif exist('plot_data','var') && ~strcmp(plot_data,'y')
    disp('Hue measurement data will not be plotted (or exported).')
else
    disp('Plot hue measurement results (y/n)?  (*Note that if no plots')
    plot_data = input('are generated, none will be exported.): ','s');
end
if ~strcmp(plot_data,'y')
    exp_plots = 'n';        % Force if no plots are generated
end
    
% Export data to what directory?
if exist('exp_dir','var') && strcmp(exp_dir,'n')   % Skip export
    exp_data = 'n';
    exp_plots = 'n';
    disp('No data or plots will be exported.')
end

% Export data?
if exist('exp_data','var') && strcmp(exp_data,'y')
    disp('Generated data will be exported.')
elseif exist('exp_data','var') && strcmp(exp_data,'n')
    disp('Generated data will not be exported.')
else
    exp_data = input('Export generated data (y/n)?: ','s');
end

% Export plots?
if exist('exp_plots','var') && strcmp(exp_plots,'y')
    disp('Plots will be exported.')
elseif exist('exp_plots','var') && strcmp(exp_plots,'n')
    disp('No plots will be exported.')
else
    exp_plots = input('Export generated plots (y/n)?: ','s');
end

% Set output file path if required
if ( strcmp(exp_data,'y') || strcmp(exp_plots,'y') ) && ...
        ( ~exist('exp_dir','var') || isempty(exp_dir) || ~ischar(exp_dir) )
    disp('Define location in which to export *.fig, *.png, *.mat')
    disp('files.  (*.mat contains ''spotLoc'', and ''satSum''.)')
    exp_dir = uigetdir(path,['Select directory in which to '...
        'place output figures and data files.']);
    exp_dir = [exp_dir '\'];
end

% Define expName (prefix of exported files) if required
if ( strcmp(exp_data,'y') || strcmp(exp_plots,'y') ) && ...
        ( ~exist('expName','var') || isempty(expName) || ~ischar(expName) )
    expName = input('Enter prefix for exported files: ','s');
end

% Colours used when displaying rectangles
dataColour = 'rgbcmyrgbcmy';

%% How many tracks and spots should be considered?
disp(' ')
disp('PART 1 : DETERMINE ISOLATED SPOT LOCATIONS')
disp(' ')

% Image enhancement of reference image?
spot_select_frame = refFrame;
disp('Perform histogram equalization image enhancement to ease spot')
disp('identification (y/n)?  (Samples are taken from unenhanced')
enh = input('image regardless): ','s');
if strcmp(enh,'y')
    spot_select_frame(:,:,1) = histeq(spot_select_frame(:,:,1),256);
    spot_select_frame(:,:,2) = histeq(spot_select_frame(:,:,2),256);
    spot_select_frame(:,:,3) = histeq(spot_select_frame(:,:,3),256);
end

% Crop spot pixels
confirm = 'n';
while ~strcmp(confirm,'y')
    disp(' ')
    % Show image for spot selection
    figure(101); clf; imshow(spot_select_frame);
    title('Image for spot selection only (not sampled directly)');
    
    % Show existing spotLoc array.  If there isn't an exisiting valid
    % spotLoc input array, import or define a new one
    if exist('spotLoc','var') && iscell(spotLoc)
        nTracks = size(spotLoc,1);
        nSpots = size(spotLoc,2);
        disp('Current ''spotLoc'' cell array is valid...')
        disp(['    nTracks = ' num2str(nTracks)])
        disp(['    nSpots = ' num2str(nSpots)])
        % Preview spotLoc array
        for n = 1:nSpots
            for m = 1:nTracks
                if ~isempty(spotLoc{m,n}) && isvector(spotLoc{m,n}) && ...
                        ( length(spotLoc{m,n}) == 4 )
                    rectangle('Position', spotLoc{m,n}, 'EdgeColor',...
                        dataColour(n));
                end
            end
        end
    else
        disp('No valid ''spotLoc'' position definition array.')
        import_spotLoc = ...
            input('Define (d) or import (i) a new cell array?: ','s');
        disp(' ')
    end
    
    % Import or define new spotLoc array
    if exist('import_spotLoc','var') && strcmp(import_spotLoc,'i')
        disp('Select a *.mat file with valid ''spotLoc'' array.')
        [import_spotLoc,import_spotLoc_path] = uigetfile('*.mat', ...
        'Select predefined ''spotLoc'' array');
        load([import_spotLoc_path import_spotLoc], 'spotLoc');
        nTracks = size(spotLoc,1);
        nSpots = size(spotLoc,2);
        % Preview spotLoc array
        for n = 1:nSpots
            for m = 1:nTracks
                rectangle('Position', spotLoc{m,n}, 'EdgeColor',...
                    dataColour(n));
            end
        end
    elseif exist('import_spotLoc','var') && ~strcmp(import_spotLoc,'y')
        nTracks = input(['Number of tracks from which spots will ' ...
            'be isolated: ']);
        disp('Number of analyte spots to analyze from each track.  (Enter')
        nSpots = input('''nSpots''+1 to include background: '); 
        spotLoc = cell(nTracks,nSpots);
    end
    
    % Confirmation
    confirm = input('Confirm inputs (y/n)?: ','s');
    if ~strcmp(confirm,'y')
        clear nTracks nSpots spotLoc import_spotLoc import_spotLoc_path;
    end
end
clear confirm;

%% Crop spots and identify hue ranges
disp(' ')
disp('PART 2 : ISOLATE AND ANALYZE SPOTS')

% Define crops
confirm = 'n';
while ~strcmp(confirm,'y')
    % Spot (selecting and) cropping
    disp(' ')
    disp('For each analyte spot (in descending hRF), crop analyte ')
    disp('spots from left to right.  (If analyzing background, these')
    disp('crops will be considered as the "last spot".)')
    
    % Sample spots
    for n = 1:nSpots
        disp(['Consider spot ' num2str(n) ' of ' ...
            num2str(nSpots) '...'])
        for m = 1:nTracks
            if isempty(spotLoc{m,n})
                disp(['    Crop from track ' num2str(m) ' of ' ...
                    num2str(nTracks)])
                [~, spotLoc{m,n}] = imcrop(spot_select_frame);
            end
        end
    end
    
    % Preview final spotLoc array
    dataColour = 'rgbcmyrgbcmy';    % Plotting colours
    figure(101); clf; imshow(spot_select_frame);
    title('Image for spot selection only (not sampled directly)');
    for n = 1:nSpots
        for m = 1:nTracks
            rectangle('Position', spotLoc{m,n}, 'EdgeColor',...
                dataColour(n));
        end
    end
    
    % Confirm
    disp(' ')
    disp('Confirm all (y), redefine all (a), or edit individually (e)')
    confirm = input(' cropped spots?: ','s');
    % If confirm isn't 'y', 'a', or 'e'; reiterate until confirm
    if strcmp(confirm,'y')
        break;  % Leave while loop
    elseif strcmp(confirm,'a')
        clear spotLoc spot import_spotLoc import_spotLoc_path;
        spotLoc = cell(nTracks,nSpots);     % Reinitialize spot array
    elseif strcmp(confirm,'e')
        disp('Edit individual spot crops:')
        goEdit = 'y';
        while ~strcmp(goEdit,'n')
            disp('    Re-crop which spot [track# spot#]?  (Enter ''c''')
            spotEdit = input('    (with quotes) to cancel edit): ');
            good_spotEdit = ~isa(spotEdit,'char') && isvector(spotEdit) ...
                && (length(spotEdit) == 2) && ...
                (spotEdit(1) >= 1) && (spotEdit(1) <= nTracks) && ...
                (spotEdit(2) >= 1) && (spotEdit(2) <= nSpots);
            if good_spotEdit
                figure(102); clf;	% Open a new figure for this cropping
                [~, spotLoc{spotEdit(1),spotEdit(2)}] = ... % Define crop
                    imcrop(spot_select_frame);
            end
            goEdit = input('    Continue editing spot crops (y/n)?: ','s');
        end
        clear spotEdit goEdit good_spotEdit import_spotLoc ...
            import_spotLoc_path;
        close 102;
    end
end
clear confirm;

% Initialize cell array. Rows correspond to track number (left to right);
% columns correspond to spot number (descending hRF); layers correspond to
% cropped image RGB array (3D), HSV array (3D), and array of H,S
% (hue,saturation) ordered pairs (2D).
spot = cell(nTracks,nSpots,3);

% Perform crops and calculations
disp(' ')
disp('Cropping defined spot pixels from image and processing...')
for n = 1:nSpots
    for m = 1:nTracks
        % Crop spot
        spot{m,n,1} = imcrop(refFrame, spotLoc{m,n});
        % Dimensions of crop
        cropHeight = size(spot{m,n,1},1);
        cropWidth = size(spot{m,n,1},2);
        spot{m,n,2} = rgb2hsv(spot{m,n,1});     % Convert (RGB to HSV)
        % Extract relevant H,S values and reshape into 2D array
        spot{m,n,3} = spot{m,n,2}(:,:,1:2);
        spot{m,n,3} = reshape(spot{m,n,3}, cropHeight*cropWidth, 2, 1);
        % Ensure H [0,360)
        spot{m,n,3}(:,1) = round( 360*spot{m,n,3}(:,1) );
        spot{m,n,3}(spot{m,n,3} == 360) = 0;    % Adjust for H = 360 
        clear cropHeight cropWidth;
    end
end
disp('Completed cropping and processing.')

%% Calculate saturation sums for each analyte spot on each track
disp(' ')
disp('PART 3 : CALCULATE HUE-FILTERED SATURATION SUMS FOR EACH SPOT')
disp(' ')

% Cell array to hold calculated saturation sums.  Each cell array column
% represents a different analyte spot.  First cell array row contains 2D
% arrays (360 x nTracks) whose columns represent the saturation sums of a
% given analyte spot on different tracks.  Second row contains similar
% arrays with sums scaled by the total in a given column (prevents
% domination by most intense spots)
satSum = cell(2,nSpots);

close all;
for n = 1:nSpots
    disp(['Consider spot ' num2str(n) ' of ' num2str(nSpots) '...'])
    for m = 1:nTracks
        disp(['    Processing spot ' num2str(n) ' from track ' ...
            num2str(m) ' of ' num2str(nTracks)])
        H = spot{m,n,3}(:,1);
        S = spot{m,n,3}(:,2);
        % Hues h over [0,359]: Consider saturaton
        for h = 1:360
            satSum{1,n}(h,m) = sum( S( H == (h-1) ) );
        end
        clear H S;
    end
    
    % Scale values so that sum of scaled saturation sums for all hues = 1
    disp('    Also calculating scaled saturation sums')
    satSum{2,n} = satSum{1,n} ./ repmat(sum(satSum{1,n},1),360,1);
    
    % Plot bar graphs for given analyte spot
    if strcmp(plot_data,'y')
        disp('    Generating bar plots of saturation sums')
        fig = figure(n);
        ax(1) = subplot(2,2,1);
            bar([0:359]',satSum{1,n});
            title('Saturation sums vs. hue angle [SEPARATE TRACKS]');
            xlim([0 360]);
        ax(2) = subplot(2,2,2);
            bar([0:359]',satSum{2,n});
            title(['Scaled saturation sums vs. hue angle ' ...
                '[SEPARATE TRACKS]']);
            xlim([0 360]);
        ax(3) = subplot(2,2,3);
            bar([0:359]',sum(satSum{1,n},2));
            title('Saturation sums vs. hue angle [SUM OVER ALL TRACKS]');
            xlim([0 360]);
        ax(4) = subplot(2,2,4);
            bar([0:359]',sum(satSum{2,n},2));
            title(['Scaled saturation sums vs. hue angle ' ...
                '[SUM OVER ALL TRACKS]']);
            xlim([0 360]);
        linkaxes(ax,'x');
    end
    
    % Export figures to MATLAB *.fig and *.png to ensure that these do not
    % interfere with *.jpg or *.tif processing
    if strcmp(exp_plots,'y')
        saveas(fig,[exp_dir expName '_hfilter' num2str(n) '.fig']);
        saveas(fig,[exp_dir expName '_hfilter' num2str(n) '.png']);
        disp('    Figure exported to *.fig/*.png files with prefix:')
        disp(['    ' expName '_hfilter' num2str(n)])
    end
end

%% Save data
% Clean-up
clear ax m n;

% Optional data export
if strcmp(exp_data,'y')
    disp(' ')
    clearvars -except spotLoc satSum path expName exp_dir;
    % Variable to record the script name, revision, and processing date
    script_checkHueRange = 'sampleSpots.m rev March 2014';
    runDate_checkHueRange = datestr(clock, 'yyyy-mm-dd, HH:MM');
    save([exp_dir expName '.mat'])
    disp(['Data exported to ' expName '.mat'])
end

%% End
disp(' ')
disp('***COMPLETED DEFINING SPOT LOCATIONS AND MEASURING HUE RANGE***')
end