function singleImageUTLC

% Written by S.R. Jim, (C) March 2014
% Glancing Angle Deposition (GLAD) Laboratory, PI: M.J. Brett
% University of Alberta, Edmonton, Canada
% 
% Purpose:  This script is designed to analyze a single image of a UTLC
% plate to produce chromatograms and relevant figures-of-merit.  It
% incorporates many of the same TR-UTLC functions in createRecipeUTLC.m,
% mainUTLC.m, filteredChromaPlotVideo.m, and fomUTLC.m.  Before analysis,
% it also allows for calibration of the input image (rotation and
% cropping).
% 
% Subfunctions:  sampleSpots.m
% 
% Inputs/outputs:  Data file (*.mat) that stores configurations and output
% data.
% 
% Exports:
%   - Rotated and cropped version of the UTLC plate image (*.jpg)
%   - Figures demonstrating hue measurement/selection (*.fig/*.jpg) and
%   *.mat data file containing this data.
%   - Chromatogram figures (isolated track, raw/fitted peaks)
%   (*.fig,*.jpg)
%   - Tables of calculated FOMs (*.csv)
%   - Data file with all configuration/calculated data

%% Filenames and Image Selection
clear all; close all;
disp(' ')
disp('PART 0: FILE DIRECTORIES AND IMAGE SELECTION')
disp(' ')

oldDataFile0 = input('Load old UTLC data file (y/n): ','s');
if strcmp(oldDataFile0,'y')
    [oldDataFile0,oldDataFilePath0] = ...
        uigetfile('*.mat','Select old data file');
    oldDataFilePath0 = [oldDataFilePath0 '\'];
    load([oldDataFilePath0 oldDataFile0]);
    oldDataFile = oldDataFile0; oldDataFilePath = oldDataFilePath0;
    disp(['Loaded UTLC data file ''' oldDataFile '''.'])
else
    oldDataFile = ''; oldDataFilePath = '';
    disp('Creating entirely new UTLC data file.')
end
clear oldDataFile0 oldDataFilePath0;

% Specify output datafile path and name
disp('Define location and name of new data files.')
dataPath = uigetdir(oldDataFilePath,['Select directory in which to '...
    'place new UTLC data file.']);
dataPath = [dataPath '\'];
opPrefix = input('Enter output UTLC file prefix: ','s');
dataFile = [opPrefix '.mat'];

% Other files exported into subdirectory (create if it doesn't exist)
dataPathEXP = [dataPath opPrefix '_exp\'];
if ~exist([dataPath opPrefix '_exp\'],'dir')
    mkdir([dataPath opPrefix '_exp\']);
    disp('* Created new directory for *.csv files.')
end
disp('Files other than main *.mat data file will be exported to: ')
disp(['    ' dataPathEXP])

% Variable to record the script name and revision
script_singleImageUTLC = 'singleImageUTLC.m rev March 2014';

% Initialize output recipe file (verify that save is permitted)
save([dataPath dataFile]);
disp('Verfied *.mat file save permitted.')

% Set UTLC plate image to analyze
disp(' ')
disp('Choose the single (unenhanced) image of the developed UTLC plate.')
disp('Select frame of the correct file format: ')
disp('8-bits/channel RGB *.jpg or *.tif.')
if ~exist('img0Path','var')
    img0Path = '';      % Initialize if it doesn't already exist
end
if ~exist('img0File','var')
    img0File = '';      % Initialize if it doesn't already exist
end
[img0File,img0Path] = uigetfile(...
    {'*.jpg;*.tif', '*.jpg or *.tif images'; ...
    '*.jpg', '*.jpg images'; ...
    '*.tif', '*.tif images'}, ...
    'Select a UTLC plate image (*.jpg or *.tif).', ...
    [img0Path img0File]);

% Import image
img0 = imread([img0Path img0File]);         % Original image
img0H = size(img0,1); img0W = size(img0,2); % Height and width of img0
disp(['Imported selected image: ' img0File])
disp(['with dimensions (h x w, px): ' num2str(img0H) ' x ' num2str(img0W)])

save([dataPath dataFile]); % Save changes

%% Image calibration
disp(' ')
disp('PART 1: CALIBRATE SELECTED IMAGE (ROTATION, RESOLUTION, CROPPING')
disp(' ')

% Give option to skip if all calibration data already exists
good_img_calib = ...
    exist('define_refrect','var') && exist('refrect','var') && ...
    exist('refrectlengths','var') && exist('refrectsize','var') && ...
    exist('imgRes','var') && exist('correctTilt','var') && ...
    exist('img0_rotate','var') && exist('imgCrop','var') && ...
    exist('img','var') && isa(img,'uint8') && (size(img,3) == 3);
if good_img_calib
    figure(1); clf; imshow(img0); title(['Original: ''' img0File ''''])
    figure(2); clf; imshow(img); title('Calibrated and cropped image')
    disp('Sufficent calibration data already exists:')
    disp(['    - Image resolution: ''imgRes'' = ' num2str(imgRes) ...
        ' (px/mm).'])
    disp(['    - Image tilt correction: ''img0_rotate'' = ' ...
        num2str(img0_rotate) ' (deg, CCW).'])
    disp('    - Tilt-corrected image cropped:  ''imgCrop'':')
    disp(imgCrop)
    good_img_calib = ...
        input('Accept current calibration (y) or recalibrate (n)?: ','s');
    good_img_calib = logical(strcmp(good_img_calib,'y'));
    if good_img_calib
        disp('Image will not be recalibrated.  (PARTS 1A-1D skipped.)')
        disp('Subsequent analysis performed on corrected ''img''.')
    else
        disp('Review, edit, and complete image calibration data.')
    end
    disp(' ')
end

% Define reference rectangle or skip
disp('PART 1A: REFERENCE RECTANGLE')
confirm = 'n';
while ~strcmp(confirm,'y') && ~good_img_calib
    figure(1); clf; imshow(img0); axis on; hold on;
    disp(' ')
    if ~exist('define_refrect','var') || ~exist('refrect','var') || ...
            (size(refrect,1) ~= 4) || (size(refrect,2) ~= 2)
        define_refrect = ...
            input('Define new reference rectangle (y/n): ','s');
        if strcmp(define_refrect,'y')
            disp('Specify 4 corners of rectangular reference region ')
            disp('(ie. UTLC plate) used to calculate image resolution ')
            disp('(px/mm) and calculate required rotation.  Select ')
            disp('corners CW from top left (of plate).  Reminder!: ')
            disp('script designed for image of plate developed downwards.')
            refrect = zeros(4,2);
            [refrect(:,1),refrect(:,2)] = ginput(4); % 4 corners of rect.
            refrect2 = zeros(4,2);  % refrect with rows rearranged: 2341
            refrect2(1:3,:) = refrect(2:4,:); refrect2(4,:) = refrect(1,:); 
            refrect2 = refrect2 - refrect; % vector from 1-2, 2-3, 3-4, 4-1
            refrectlengths = [norm(refrect2(1,:)), norm(refrect2(2,:)), ...
                norm(refrect2(3,:)), norm(refrect2(4,:))];
            % Diagonals 1-3 and 2-4
            refrectlengths(5) = norm(refrect(3,:) - refrect(1,:));
            refrectlengths(6) = norm(refrect(4,:) - refrect(2,:));
        else
            refrect = NaN .* ones(4,2);         % Array of NaNs
            refrectlengths = NaN .* ones(1,6);  % Array of NaNs
        end
    end
    % Plot reference rectangle corners
    if strcmp(define_refrect,'y')
        for j = 1:4     % Plot reference rectangle corners
            plot(refrect(j,1),refrect(j,2),'or');
        end
        clear j;
        disp(' ')
        disp('Reference rectangle has measured pixel dimensions of ')
        disp('top, right, bottom, left edges and TL-BR, TR-BL diagonals:')
        disp(refrectlengths)
        hold off;
    else
        disp('Skip defintion of reference rectangle.  Enter ')
        disp('image resolution and rotation manually.')
    end
    % Confirm
    confirm = input('Confirm these definitions (y/n)?: ','s');
    if strcmp(confirm,'y')
        save([dataPath dataFile]); % Save changes
    else
        clear refrect*;
    end
end
clear confirm;
    
% Image resolution
disp(' ')
disp('PART 1B: CALCULATE OR ENTER IMAGE RESOLUTION (px/mm)')
confirm = 'n';
while ~strcmp(confirm,'y') && ~good_img_calib
    disp(' ')
    % Was this calculated using input mm dimensions of refrect?
    if exist('calc_imgRes','var') && strcmp(calc_imgRes,'y') && ...
            exist('refrectlengths','var') && exist('refrectsize','var')
        disp('Image resolution calculated assuming reference ')
        disp(['rectangle has real dimensions of ' ...
            num2str(refrectsize(1)) ' x ' num2str(refrectsize(2)) ...
            ' (mm)'])
        imgRes1 = sum(refrectlengths(1:4)) / ( 2*sum(refrectsize) );
        if imgRes ~= imgRes1    % Verify this value is unchanged
            disp('*WARNING:  Image resolution recalculated.')
            disp(['    (Old value was: ' num2str(imgRes) ' (px/mm))'])
            imgRes = imgRes1;
        end
        clear imgRes1;
    end
    
    % Calculate or Set imgRes value
    if ~exist('imgRes','var') && strcmp(define_refrect,'y')
        disp('Calculate ''imgRes'' using ''refrect'' (y) or enter value ')
        calc_imgRes = input('directly in (px/mm) (n)?: ','s');
        if strcmp(calc_imgRes,'y')
            disp('Input real dimensions (mm) of reference rectangle as ')
            refrectsize = input('row vector [width height]: ');
            imgRes = sum(refrectlengths(1:4)) / ( 2*sum(refrectsize) );
        else
            imgRes = input('    Enter frame resolution (px/mm): ');
            refrectsize = [NaN NaN];
        end
    elseif ~exist('imgRes','var') && ~strcmp(define_refrect,'y')
        calc_imgRes = 'n';
        imgRes = input('    Enter frame resolution (px/mm): ');
        refrectsize = [NaN NaN];
    end
    
    % Confirm
    disp(['    Image resolution is: ' num2str(imgRes) ' (px/mm)'])
    confirm = input('Confirm image resolution (y/n)?: ','s');
    if strcmp(confirm,'y')
        save([dataPath dataFile]); % Save changes
    else
        clear imgRes calc_imgRes refrectsize;
    end
end
clear confirm;

% Image tilt correction
disp(' ')
disp('PART 1C: IMAGE TILT CORRECTION')

% Measure tilt angle from reference rectangle, if possible
if strcmp(define_refrect,'y') && exist('refrect2','var') && ...
        exist('refrectlengths','var') && ~good_img_calib
    disp(' ')
    disp('Use reference rectangle to determine by how much the ')
    disp('UTLC plate is rotated within the image.')
    % Find equivalent coordinates of vector when +y direction is up
    refrect3 = refrect2; refrect3(:,2) = -refrect3(:,2);
    
    % Rotate 4 edge vectors so that tilt can be measured wrt +x
    refrect4 = [refrect3(1,1) refrect3(1,2); ...
        -refrect3(2,2) refrect3(2,1);
        -refrect3(3,1) -refrect3(3,2);
        refrect3(4,2) -refrect3(4,1)];
    cos_tilt = refrect4(:,1) ./ refrectlengths(1:4)';
    
    % Calculate tilt angle (+CCW in degrees)
    tilt = zeros(1,4);
    for j = 1:4
        tilt(j) = 180/pi * acos(cos_tilt(j));
        if refrect4(j,2) < 0
            tilt(j) = -tilt(j);    % Ensure -y value indicates tilt < 0
        end
    end
    clear j;
    
    % Display measured angles
    disp('Tilt angles measured using top, right, bottom, and left ')
    disp('edges of reference rectangle (+CCW, deg)')
    disp(tilt)
    tilt_avg = mean(tilt);
    disp(['Average tilt angle: ' num2str(tilt_avg) ' deg.'])
end
clear refrect2 refrect3 refrect4 cos_tilt;

% Set tilt angle correction if any
confirm = 'n';
while ~strcmp(confirm,'y') && ~good_img_calib
    disp(' ')
    % Define tilt angle correction
    correctTilt = input('Perform tilt correction (y/n)?: ','s');
    if strcmp(correctTilt,'y') && exist('img0_rotate','var')
        disp(['Current tilt angle correction is ' num2str(img0_rotate) ...
            ' deg (CCW).'])
    elseif strcmp(correctTilt,'y') && ~exist('img0_rotate','var') && ...
            exist('tilt_avg','var')
        disp('Automatically correct using measured tilt angle')
        correctTilt = input(['''tilt_avg'' = ' num2str(tilt_avg) ...
            ' deg (a) or enter manually (m)?: '],'s');
        if strcmp(correctTilt,'a')
            img0_rotate = - tilt_avg;
        else
            disp('Input angle by which UTLC plate image should be ')
            img0_rotate = input('tilted (CCW, deg): ');
        end
    elseif strcmp(correctTilt,'y') && ~exist('img0_rotate','var') && ...
            ~exist('tilt_avg','var')
        disp('Input angle by which UTLC plate image should be ')
        img0_rotate = input('rotated (CCW, deg): ');
    else
        img0_rotate = 0;
        disp('No tilt angle correction will be performed.')
    end
    
    % Perform image rotation
    img1 = imrotate(img0,img0_rotate,'bicubic');
    figure(2); clf; imshow(img1); axis on;
    title('Rotated UTLC plate image, ''img1''')
    disp(['Rotated UTLC plate image by ' num2str(img0_rotate) ...
        ' deg (CCW).'])
    
    % Confirm
    confirm = input('Confirm this tilt correction (y/n)?: ','s');
    if strcmp(confirm,'y')
        % Export file and save changes
        % imwrite(img1,[dataPathEXP opPrefix '-rotate.tif'],...
        %   'Compression','lzw');
        imwrite(img1,[dataPathEXP opPrefix '-rotate.jpg'],...
            'Quality',100);
        disp(['Exported rotated image to: ' opPrefix '-rotate.jpg'])
        save([dataPath dataFile]); % Save changes
    else
        clear correctTilt img0_rotate;
    end
end
clear confirm;

% Image tilt correction
disp(' ')
disp('PART 1D: IMAGE CROPPING')

% Set image cropping if required
confirm = 'n';
while ~strcmp(confirm,'y') && ~good_img_calib
    disp(' ')
    if ~exist('img','var') || ~exist('imgCrop','var')
        disp('Crop UTLC plate from calibrated tilt-corrected image.')
        disp('This selection will be used for all subsequent analysis.')
        % Crop rotated image
        [img,imgCrop] = imcrop(img1);
    else
        [img,imgCrop] = imcrop(img1,imgCrop);   % Use defined imgCrop
        disp('Image crop location already defined.')
    end
    figure(1); clf; imshow(img); axis on;
    disp('Cropped UTLC plate from calibrated tilt-corrrected image.')
    disp('Cropping coordinates are:')
    disp(imgCrop)
    % Confirm
    confirm = input('Confirm this crop (y/n)?: ','s');
    if ~strcmp(confirm,'y')
        clear img imgCrop;
    end
end
clear confirm;

%% Preparation for further UTLC plate definitions/analysis

% Clear img0; img1; export img; save data
disp(' ')
clear img0 img1 good_img_calib;
disp('Cleared original image ''img0'' and tilt-corrected image ''img1''')
% imwrite(img,[dataPathEXP opPrefix '-correct.tif'],'Compression','lzw')
imwrite(img,[dataPathEXP opPrefix '-correct.jpg'],'Quality',100)
disp(['Exported corrected image to: ' opPrefix '-correct.jpg'])

% Create enhanced version of image
imgH = size(img,1); imgW = size(img,2);     % Height, width of img
% Perform enhancement (histogram equalization of each RGB channel)
clear img_enh;  % Clear if it existed in old data file
for j = 1:3
    img_enh(:,:,j) = histeq(img(:,:,j),256);
end
clear j;
disp('Created also enhanced version of this image (not exported).')
close all;                                  % Close all open figures
save([dataPath dataFile]); % Save changes
disp('PART 1 definitions saved.')

%% Get initial spot location for each track
disp(' ')
disp('PART 2: SPECIFY NUMBER AND INITIAL LOCATIONS OF SPOTS')

% Display working image
figure(1); clf; imshow(img); axis on;

confirm = 'n';
while ~strcmp(confirm,'y')
    % Select (un)enhanced images for this section
    disp(' ')
    disp('Use enhanced UTLC plate image in this section?  Enter: ')
    disp('    (c) for inverted image (ie. complement);')
    disp('    (e) for image enhanced using histogram equalization;')
    disp('    (n) for no image enhancement')
    enh = input('Type option here: ','s');
    if strcmp(enh,'c')
        imgDefSpt = imcomplement(img);
    elseif strcmp(enh,'e')
        imgDefSpt = img_enh;
    else
        imgDefSpt = img;
    end
    clear enh;

    % Preview track locations if available
    figure(1); clf; imshow(imgDefSpt); axis on; hold on;
    if exist('defineTrack','var')
        for i = 1:size(defineTrack,1)
            % TOP left corner coordinates and dimensions (TOP because of
            % inverted image y-axis. Note: At this point, nTracks doesn't
            % need to equal size(defineTrack,1) )
            rectangle('Position', defineTrack(i,:),'EdgeColor','b');
        end
    end
    % Specify number of tracks
    if ~exist('nTracks','var') && exist('spotLocationX','var') && ...
            exist('spotLocationY','var') && ...
            (length(spotLocationX) == length(spotLocationY))
        nTracks = length(spotLocationY);    % Reset nTracks if lost
    elseif ~exist('nTracks','var') || ~exist('spotLocationX','var') || ...
            ~exist('spotLocationY','var')
        nTracks = input('Enter number of tracks that can be analyzed: ');
    end
    % Input spot locations if required
    if ~exist('spotLocationX','var') || ~exist('spotLocationY','var') ...
            || (length(spotLocationX) ~= length(spotLocationY))
        disp('Locate each initial applied spot position (left to right)')
        [spotLocationX,spotLocationY] = ginput(nTracks);
    end
    % Correct imported location vectors if required
    spotLocationTEST1 = [spotLocationX,spotLocationY];
    spotLocationTEST2 = sortrows(spotLocationTEST1,1);
    if max(max( spotLocationTEST1 ~= spotLocationTEST2 ))
        spotLocationX = spotLocationTEST2(:,1);
        spotLocationY = spotLocationTEST2(:,2);
        disp('''spotLocationX'' and ''spotLocationY'' vectors updated so ')
        disp('that all applid spot locations are sorted left to right.')
    end
    clear spotLocationTEST1 spotLocationTEST2;
    % Preview spot positions and track locations
    disp('Initial spot x,y positions (left to right) defined as [px]:')
    disp(' ')
    disp([spotLocationX spotLocationY])
    for i = 1:nTracks   % Add locations of each spot to figure
        plot(spotLocationX(i),spotLocationY(i),'or');
    end
    hold off;
    confirm = input('Confirm these values (y/n): ','s');
    % Redefine spot locations if necessary
    if ~strcmp(confirm,'y')
        redefineSpotLoc1 = ...
            input('    Redefine single/all spot locations (s/a)?: ','s');
    end
    % Clear all spot location definitions or edit single spot
    if exist('redefineSpotLoc1','var') && strcmp(redefineSpotLoc1,'a')
        clear spotLocationX spotLocationY;
        disp(' ')
        redefineSpotLoc2 = ...
            input('    Redefine graphically or numerically (g/n)?: ','s');
        if ~strcmp(redefineSpotLoc2,'g') % Enter numerically
            disp('    Enter coordinates of all initial spots as an ')
            editSpot = input (['    ' num2str(nTracks) ...
                ' x 2 array.  One row for [x y] coordinates in px: ']);
            spotLocationX = editSpot(:,1);
            spotLocationY = editSpot(:,2);
        end
    end
    % Edit single spots graphically or numerically
    while exist('redefineSpotLoc1','var') && ~strcmp(redefineSpotLoc1,'a')
        disp(' ')
        spotNum = input(['    Edit which spot (1 - ' ...
            num2str(nTracks) ')?: ']);
        redefineSpotLoc2 = ...
            input('    Redefine graphically or numerically (g/n)?: ','s');
        if strcmp(redefineSpotLoc2,'g')
            disp(['    Locate initial spot ' num2str(spotNum)])
            [spotLocationX0,spotLocationY0] = ginput(1);
            spotLocationX(spotNum) = spotLocationX0;
            spotLocationY(spotNum) = spotLocationY0;
            clear spotLocationX0 spotLocationY0;
        else
            clf; imshow(imgDefSpt); axis on; hold on;
            disp(['    Enter coordinates of spot ' num2str(spotNum)])
            editSpot = input ('    as a row vector ([x y] in px): ');
            spotLocationX(spotNum) = editSpot(1);
            spotLocationY(spotNum) = editSpot(2);
        end
        % Display editted coordinates for given spot
        disp(['    Spot ' num2str(spotNum) ' has coordinates:'])
        disp(['    (x,y) = (' num2str(spotLocationX(spotNum)) ',' ...
            num2str(spotLocationY(spotNum)) ') px'])
        % Confirm and stop editing
        editSpot = ...
            input('    Continue editing spot locations (y/n)?: ','s');
        if strcmp(editSpot,'n')  % Exit while loop
            clear redefineSpotLoc*;
        end
    end
    clear redefineSpotLoc*;
    
end
clear confirm i spotNum redefineSpotLoc* editSpot imgDefSpt;
save([dataPath dataFile]); % Save changes

%% Define track locations
disp(' ')
disp('PART 3: SPECIFY LOCATION OF TRACKS')

confirm = 'n';
while ~strcmp(confirm,'y')
    % Select (un)enhanced images for this section
    disp(' ')
    disp('Use enhanced UTLC plate image in this section?  Enter: ')
    disp('    (c) for inverted image (ie. complement);')
    disp('    (e) for image enhanced using histogram equalization;')
    disp('    (n) for no image enhancement')
    enh = input('Type option here: ','s');
    if strcmp(enh,'c')
        imgDefTrk = imcomplement(img);
    elseif strcmp(enh,'e')
        imgDefTrk = img_enh;
    else
        imgDefTrk = img;
    end
    clear enh;

    % Define all tracks if required
    if ~exist('defineTrack','var') || (size(defineTrack,1) ~= nTracks)
        disp('No valid ''defineTrack'' array exists.')
        disp(['Define locations of ' num2str(nTracks) ...
            ' tracks graphically or '])
        make_defineTrack = ...
            input('numerically (g/n)?: ','s');
        disp(' ')
        defineTrack = zeros(nTracks,4);
        if strcmp(make_defineTrack,'g')
            disp('Graphically: Crop track locations. (Height of crop ')
            disp('will be forced to the full frame height.)')
            figure(2);
            for trackNum = 1:nTracks
                disp(['    Crop location of track ' num2str(trackNum) ...
                    ' of ' num2str(nTracks)])
                clf;
                [~,defineTrack(trackNum,:)] = imcrop(imgDefTrk);
            end
        else
            figure(2);  % Open a new window so that figure 1 remains
            clf; imshow(imgDefTrk); axis on; hold on;
            disp('Numerically: Enter left/right corner x-positions of ')
            disp(['each track together as a ' num2str(nTracks) ...
                ' x 2 matrix. (One row '])
            disp('for each track, columns for left/right x-positions.)')
            editTrk = input ('Enter in px: ');
            editTrk = [min(editTrk,[],2) max(editTrk,[],2)]; % Proper order
            defineTrack(:,1) = editTrk(:,1);    % left corner
            defineTrack(:,3) = editTrk(:,2) - editTrk(:,1) + 1; % width
        end
        % Ensure defineTrack selects full height of frame
        defineTrack = round(defineTrack);
        defineTrack(:,2) = 0; defineTrack(:,4) = size(img,1);
    end
    clear make_defineTrack editTrk;
    
    % Correct imported defineTrack array if required
    defineTrackTEST = round(defineTrack);
    defineTrackTEST(:,2) = 0; defineTrackTEST(:,4) = imgH;
    defineTrackTEST = sortrows(defineTrackTEST,1);
    if max(max( defineTrack ~= defineTrackTEST ))
        defineTrack = defineTrackTEST;
        disp(' ')
        disp('''defineTrack'' array updated so that all entries are')
        disp('integers, all tracks span full height of frame, and ')
        disp('all tracks are sorted left to right.')
    end
    clear defineTrackTEST;

    % Preview track locations
    disp(' ')
    disp('Current ''defineTrack'' (rows for [x y width height] in px):')
    disp(' ')
    disp(defineTrack);
    
    disp('The left, right edge coordinates for the tracks are (px):')
    disp(' ')
    disp([defineTrack(:,1) (defineTrack(:,1) + defineTrack(:,3) - 1)])
    
    % Check to see if any tracks overlap.  Calculate gap between tracks 
    % and see if any are < 0.
    defineTrackRIGHT = (defineTrack(1:(nTracks-1),1) + ...
        defineTrack(1:(nTracks-1),3) - 1);
    defineTrackLEFT = defineTrack(2:nTracks,1);
    defineTrackGAP = defineTrackLEFT - defineTrackRIGHT;
    if max(defineTrackGAP < 0)
        disp('Warning: Some of the tracks overlap!')
    end
    clear defineTrackRIGHT defineTrackLEFT defineTrackGAP;

    % Preview on unenhanced
    figure(1); clf; imshow(img); axis on; hold on;
    for i = 1:nTracks       % Indicate location of each track
        % TOP left corner coordinates and dimensions (TOP because of
        % inverted image y-axis)
        rectangle('Position', defineTrack(i,:),'EdgeColor','b');
        plot(spotLocationX(i),spotLocationY(i),'or');
    end
    title('Separation tracks on (unenhanced) ''img''');
    hold off;
        
    % Preview on enhanced
    figure(2); clf; imshow(img_enh); axis on; hold on;
    for i = 1:nTracks       % Indicate location of each track
        % TOP left corner coordinates and dimensions (TOP because of
        % inverted image y-axis)
        rectangle('Position', defineTrack(i,:),'EdgeColor','b');
        plot(spotLocationX(i),spotLocationY(i),'or');
    end
    title('Separation tracks on (enhanced) ''img''');
    hold off;
    
    % Confirm or edit track locations
    confirm = input('Confirm these track locations (y/n): ','s');
    if ~strcmp(confirm,'y')
        redefineTrack1 = ...
            input('    Redefine single/all tracks (s/a)?: ','s');
    end
    % Clear all track location definitions or edit single tracks
    if exist('redefineTrack1','var') && strcmp(redefineTrack1,'a')
        clear defineTrack;
        disp('    All existing separation track definitions deleted.')
    end
    % Edit single tracks graphically or numerically
    while exist('redefineTrack1','var') && ~strcmp(redefineTrack1,'a')
        disp(' ')
        trackNum = input(['    Edit which track (1 - ' ...
            num2str(nTracks) ')?: ']);
        redefineTrack2 = ...
            input('    Redefine graphically or numerically (g/n)?: ','s');
        if strcmp(redefineTrack2,'g')
            disp(['    Re-crop track ' num2str(trackNum)])
            figure(3);  % Open a new window so that figures 1-2 remain
            [~,defineTrack(trackNum,:)] = imcrop(imgDefTrk);
        else
            figure(3);  % Open a new window so that figures 1-2 remain
            clf; imshow(imgDefTrk); axis on; hold on;
            disp('    Enter left/right corner x-positions of ')
            editTrk = input (['    track ' num2str(trackNum) ...
                ' as a row vector ([left right] in px): ']);
            editTrk = [min(editTrk) max(editTrk)];  % Ensure propper order
            defineTrack(trackNum,1) = editTrk(1);
            defineTrack(trackNum,3) = editTrk(2) - editTrk(1) + 1;
        end
        % Ensure defineTrack selects full height of frame
        defineTrack = round(defineTrack);
        defineTrack(:,2) = 0; defineTrack(:,4) = imgH;
        
        % Confirm and stop editing
        editTrk = ...
            input('    Continue editing ''defineTrack'' (y/n)?: ','s');
        if strcmp(editTrk,'n')  % Exit while loop
            clear redefineTrack*;
            close 3;    % Close figure 3
        end
    end
    clear redefineTrack*;
end
close all;  % Close all figures (ie. figures 1-3)
clear confirm* i trackNum redefineTrack* editTrk;
clear enh imgDefTrk;
save([dataPath dataFile]); % Save changes

%% Identify migration front location
disp(' ')
disp('PART 4: MIGRATION FRONT LOCATION')

confirm = 'n';
while ~strcmp(confirm,'y')
    % Select (un)enhanced images for this section
    disp(' ')
    disp('Use enhanced UTLC plate image in this section?  Enter: ')
    disp('    (c) for inverted image (ie. complement);')
    disp('    (e) for image enhanced using histogram equalization;')
    disp('    (n) for no image enhancement')
    enh = input('Type option here: ','s');
    if strcmp(enh,'c')
        imgDefFront = imcomplement(img);
    elseif strcmp(enh,'e')
        imgDefFront = img_enh;
    else
        imgDefFront = img;
    end
    clear enh;
    
    % Preview track locations
    figure(1); clf; imshow(imgDefFront); axis on; hold on;
    for i = 1:nTracks       % Indicate location of each track
        % TOP left corner coordinates and dimensions (TOP because of
        % inverted image y-axis)
        rectangle('Position', defineTrack(i,:));
        plot(spotLocationX(i),spotLocationY(i),'or');
    end
    clear i;
    title('Separation tracks on (unenhanced) ''img''');
    hold off;
    
    % Input front locations if required
    if ~exist('zf0','var')
        disp('Locate each migration front y-position (left to right)')
        [~,zf0] = ginput(nTracks);
        zf0 = zf0'; % Convert into row vector
    end
    
    % Calculate front position wrt zStart
    zf = zf0 - spotLocationY';
        
    % Overlaid fronts on on preview image
    frontpos = defineTrack;
    frontpos(:,2) = spotLocationY;  % Top of rectangle
    frontpos(:,4) = zf' + 1;        % Height of rectangle
    hold on;
    if max(frontpos(:,4) <= 0)  % Check if invalid zf0 calculated.
        frontpos((frontpos(:,4) <= 0),4) = 1;   % Dummy value!
        disp('    Calculated zf0 invalid.  Displaying "dummy" fronts.')
    end
    for i = 1:nTracks
        rectangle('Position', frontpos(i,:),'EdgeColor','r');
    end
    hold off;
    title('Migration fronts on (unenhanced) ''img'' ');
    
    % Display migration front positions
    disp('Measured migration front positions.  (Top row ''zf0'' wrt top')
    disp('of frame; bottom row ''zf'' wrt initial spot location; in px.)')
    disp([zf0; zf])
    
    % Confirm
    confirm = input('Confirm these values (y/n): ','s');
    
    % Redefine spot locations if necessary
    if ~strcmp(confirm,'y')
        clear zf*;
        disp('    Redefine front locations numerically as a row vector')
        disp('    (''zf0'' positions measured wrt top of frame in px;')
        zf0 = input('    define for tracks from left to right): ');
    end
end
clear confirm i frontpos imgDefFront img_enh;

save([dataPath dataFile]); % Save changes

%% Locate spots
% Uses functions incorporated into sampleSpots.m.
disp(' ')
disp('PART 5: LOCATE SPOTS AND MEASURE HUES')
disp(' ')

% Define the number of available spots
confirm = 'n';
while ~strcmp(confirm,'y')
    if exist('nSpots','var')
        disp([num2str(nSpots) ' analyte spots will be analyzed.'])
    else
        nSpots = input('Enter number of analyte spots to be analyzed: ');
    end
    confirm = input('Confirm this value (y/n)?: ','s');
    if ~strcmp(confirm,'y')
        clear nSpots;
    end
end
clear confirm;

% Check validity of existing spotLoc array.  Initialize a new one if
% required.
good_spotLoc = exist('spotLoc','var') && (size(spotLoc,1) == nTracks) ...
    && (size(spotLoc,2) >= nSpots);
if ~good_spotLoc
    disp('No valid ''spotLoc'' spot position array.')
    disp(['Initializing a new ' num2str(nTracks) ' x ' num2str(nSpots+1)])
    disp('''spotLoc'' array to store spot position data for all')
    disp([num2str(nSpots) ' spots + 1 background "spot" on all ' ...
        num2str(nTracks) ' tracks.'])
    spotLoc = cell(nTracks, (nSpots+1) );
end
clear good_spotLoc;

% Perform analysis (with exported plots)
disp('Define or verify spot locations and measure hues. Note that the ')
disp('last "spot" corresponds to regions cropped from the background.')
disp(' ')
disp('Perform analysis using ''sampleSpots'' subfunction...')
disp(' ')
disp('START ..................................................')
disp(' ')
% [spotLoc,satSum] = sampleSpots(refFrame,spotLoc,plot_data,exp_dir,...
%     exp_data,exp_plots,expName)
[spotLoc,~] = sampleSpots(img,spotLoc,'y',dataPathEXP,'y','y',...
    [opPrefix '_checkHue']);
% Note: Plots provided to assist in hue filter definition; spotLoc used
% also to define initial windows over which iterative peak fitting should
% occur.
disp(' ')
disp('END   ..................................................')
disp(' ')
disp('''spotLoc'' and hue measurements obtained using sampleSpots.m')
save([dataPath dataFile]); % Save changes

%% Name spots (ie. label) and define hue filters
disp(' ')
disp('PART 6: ANALYTE SPOT LABELS AND HUE FILTERS')
disp(' ')

% Continue?
goProcess = input('Continue to set-up definitions (y/n)?: ','s');
if ~strcmp(goProcess,'y')
    disp('    User stopped process.')
    return;
end
clear goProcess;

% Figure 100 to allow Fig 1, 2, etc. from above to stay open
figure(100); clf; imshow(img); axis on;
confirm = 'n';
while ~strcmp(confirm,'y')
    disp(' ')
    % Define hueRange array
    if ~exist('hueRange','var') || (min(hueRange(:)) < 0) || ...
            (max(hueRange(:)) > 360) || (size(hueRange,1) ~= nSpots)
        hueRange = zeros(nSpots,2); % Intitialize hueRange array
        disp('Enter analyte hue ranges in decreasing hRF as row vectors')
        for i = 1:nSpots
            hueRange(i,:) = input(['    Enter hue range for ' ...
                'analyte ' num2str(i) ': ']);
        end
    end
    hueRange(hueRange == 360) = 0;  % Check for 360 == 0 discontinuity
    % Define hueRangeLabels
    if ~exist('hueRangeLabel','var') || (size(hueRangeLabel,1) ~= nSpots)
        % Display and label selected hueRange
        disp('Label analyte hue ranges')
        hueRangeLabel = cell(nSpots,1);
        for i = 1:nSpots
            templabel = input(['    Name analyte ' num2str(i) ...
                ': H = [' num2str(hueRange(i,1)) ':' ...
                num2str(hueRange(i,2)) ']: '],'s');
            hueRangeLabel{i} = [templabel ': H = [' ...
                num2str(hueRange(i,1)) ':' num2str(hueRange(i,2)) ']'];
        end
    end
    % Display current hueRange and hueRangeLabel and confirm
    disp('Current analyte hue ranges in descending hRF are:')
    disp(hueRange)
    disp('Current analyte hue ranges labels in descending hRF are:')
    disp(hueRangeLabel)
    confirm = input('Confirm these values (y/n): ','s');
    if ~strcmp(confirm,'y')   % Clear hueRange and/or hueRangeLabel
        redohueRangeLabel = ...
            input('    Redefine only ''hueRangeLabel'' (y/n)?: ','s');
        redohueRange = input(['    Redefine ''hueRange'' and ' ...
            '''hueRangeLabel'' (y/n)?: '],'s');
        if ~strcmp(redohueRange,'n')
            clear hueRange hueRangeLabel;
        end
        if ~strcmp(redohueRangeLabel,'n')
            clear hueRangeLabel;
        end
        if strcmp(redohueRange,'n') && strcmp(redohueRangeLabel,'n')
            disp('    No values changed.  Current values confirmed.')
            confirm = 'y';
        end
    end
    clear i templabel redohueRange redohueRangeLabel;
end
clear confirm;
save([dataPath dataFile]); % Save changes

%% Information about spot loading
disp(' ')
disp('PART 7: SPOT LOADING INFORMATION')
disp(' ')

% Undiluted dye concentration
disp('PART 7A: Undiluted dye concentration')
disp([num2str(nSpots) ' analyte hue filters are defined.'])
if exist('undilutedConc','var')  % Check to see if appropriate data exists
    disp([num2str(length(undilutedConc)) ' undiluted concentrations ' ...
        'have been defined.'])
end
goPart = input('Proceed with this part (y/n)?: ','s');
confirm = 'n';
while ~strcmp(confirm,'y') && strcmp(goPart,'y')
    disp(' ')
    if ~exist('undilutedConc','var')
        disp('Specify the concentraton of each analyte for which a hue')
        disp('filter was defined. Define as a row vector of form')
        undilutedConc = input('[conc1 conc2 conc3...] in units (g/L): ');
        undilutedConc = undilutedConc'; % Convert to column vector
    else
        disp('Current undiluted analyte conc. (g/L) is (descending hRF):')
        disp(undilutedConc)
    end
    if length(undilutedConc) == nSpots
        confirm = input('Confirm these values (y/n): ','s');
    end
    if ~strcmp(confirm,'y')   % Clear hueRange and redefine
        clear undilutedConc;
    end
end
clear confirm goPart;
save([dataPath dataFile]); % Save changes

% Applied spot mass loading
disp(' ')
disp('PART 7B: Applied spot mass loading')
disp([num2str(nTracks) ' separation tracks/ initial spots were defined.'])
if exist('spotVolume','var')    % Check to see if appropriate data exists
    disp([num2str(length(spotVolume)) ' applied spot volumes '  ...
        'have been defined.'])
end
goPart = input('Proceed with this part (y/n)?: ','s');
confirm = 'n';
while ~strcmp(confirm,'y') && strcmp(goPart,'y')
    disp(' ')
    if ~exist('relConc','var') || ~exist('spotVolume','var') || ...
            ~exist('spotMass','var')
        relConc = input(['Specify relative sample concentration ' ...
            '([0,1]; ex. 0.2 for 20%): ']);
        % Applied spot volumes (row vector: 1 x nTracks)
        disp('Specify applied spot volumes (nL) for each track.')
        spotVolume = input('Enter row vector: ');
    else
        disp(['The relative sample concentration is: ' num2str(relConc)])
        disp('The applied spot volumes are (left to right):')
        disp(spotVolume)
    end
    if exist('undilutedConc','var')
        spotMass = relConc .* (undilutedConc * spotVolume);
        disp('Calculated mass (ng) of each analyte (rows) in spot')
        disp('applied to each track (columns):')
        disp(spotMass)
    else
        disp('Mass of each analyte in spots applied to each track')
        disp('cannot be calculated because ''undilutedConc'' does not')
        disp('exist.  Forced to skip mass calculation.')
        clear spotMass;
    end
    confirm = input('Confirm these values (y/n): ','s');
    if ~strcmp(confirm,'y')   % Clear hueRange and redefine
        clear relConc spotVolume spotMass;
    end
end
clear confirm goPart;
save([dataPath dataFile]); % Save changes

%% Generate inital guesses and fitting windows for analyte peaks
disp(' ')
disp('PART 8: CALCULATE ANALYTE PEAK FITTING WINDOWS')
disp('(Use ''spotLoc'' array from PART 6 to generate.)')
disp(' ')

% Extract relevant y-position data from spotLoc; consider only the first
% nSpot number of samples spots (ie. exclude background).  First level of
% roi array for lower limit; second level for upper limit of window (in
% px; wrt top of frame).
roi = zeros(nTracks,nSpots,2);
for m = 1:nTracks
    for n = 1:nSpots
        roi(m,n,1) = spotLoc{m,n}(2);   % Upper edge of spotLoc rectangle
        roi(m,n,2) = spotLoc{m,n}(2) + spotLoc{m,n}(4); % Bottom edge
    end
end
clear m n;
disp('Fitting windows calculated')

%% Chromatogram extraction and iterative gaussian fitting
disp(' ')
disp('PART 9: CHROMATOGRAM EXTRACTION AND GAUSSIAN PEAK FITTING')
disp(' ')

% Continue?
goProcess = input('Continue processing (y/n)?: ','s');
if ~strcmp(goProcess,'y')
    disp('    User stopped process.')
    return;
end
clear goProcess;

% Initialize output variables
Densitogram = zeros(imgH,nSpots,nTracks);       % Raw Densitograms
fitDensitogram = zeros(imgH,nSpots,nTracks);    % Fitted Densitograms
mu = zeros(nTracks,nSpots);         % Fitted peak centre wrt top of frame
stdSpot = zeros(nTracks,nSpots);    % Fitted peak stdev
ampSpot = zeros(nTracks,nSpots);    % Fitted peak amplitude
zSpot = zeros(nTracks,nSpots);      % Fitted peak centre wrt applied spot
roiSpot = zeros(nTracks,nSpots,2);  % Final fitting window roi
stdResiduals = zeros(nTracks,nSpots);  % stdev of residuals
stopFitStatus = cell(nTracks,nSpots);  % Holds strings for stop fit status

% Vector of y-positions
x = (1:imgH)';

close all;
for t = 1:nTracks
    disp(' ')
    disp(['Processing track ' num2str(t) ' of ' num2str(nTracks) '...'])
    imgT = imcrop(img,defineTrack(t,:));    % Cropped track image
    
    % Initialize densitogram arrays
    Densitogram0 = zeros(imgH,nSpots);
    fitDensitogram0 = zeros(imgH,nSpots);
    
    % [D,avg_px,stdev_px,avg_D,stdev_D] = ...
    %   filteredDensitogram(img,hueRange)
    [Densitogram0(:,:),~,~,~,~] = filteredDensitogram(imgT,hueRange);
    
    % Fit gaussian function to densitogram and numerically calculate
    % peak information
    for s = 1:nSpots
        % [params,roi0,res,resStdev,status] = ...
        %    iterativeGaussFit(x,y,roi0,iter,crit): 10 iterations, 1%
        %    criteria
        [params,roiFitOutput,~,stdResiduals(t,s),fitStatus] = ...
            iterativeGaussFit(x,Densitogram0(:,s),...
            [roi(t,s,1) roi(t,s,2)],10,0.01);
        ind = sum(params,2); % Vector to select final iteration values
        ind(ind==0) = [];    % Remove empty values
        ind = length(ind);   % Index of final iteration (scalar)
        % Save relevant values
        mu(t,s) = params(ind,1);
        zSpot(t,s) = mu(t,s) - spotLocationY(t);
        stdSpot(t,s) = params(ind,2);
        ampSpot(t,s) = params(ind,3);
        roiSpot(t,s,1) = roiFitOutput(ind,1);
        roiSpot(t,s,2) = roiFitOutput(ind,2);
        % Stop after what iteration?  Why?
        stopFitStatus{t,s} = [num2str(ind) ': ' fitStatus{ind}];
        % Calculate fitDensitogram0 signal
        fitDensitogram0(:,s) = ampSpot(t,s) * exp( -(x-mu(t,s)).^2 / ...
            (2*stdSpot(t,s)^2) );   % Equation for gaussian distribution
        clear ind params roiFitOutput fitStatus; 
    end
    
    % Save raw and fitted densitograms
    Densitogram(:,:,t) = Densitogram0;          % Raw densitogram
    fitDensitogram(:,:,t) = fitDensitogram0;    % Fitted densitogram
    
    % Generate a figure to show Densitogram/fitDensitogram:
    disp('    Generating figure.')
    fig = figure(t); clf;
    set(fig,'Position',[50 50 800 600]);    % Figure size = 800 x 600
    set(fig,'Color',[1 1 1]);
    % Display rotated track image and title
    sp(1) = subplot(2,1,1);
    imshow(imrotate(imgT,90));
    set(sp(1),'Position',[0.1 0.7 0.85 0.35]);
    title(['Track ' num2str(t) ' cropped from: ' opPrefix ...
        '-correct.tif (rotated 90 deg CCW)']);
    xlim([0 imgH]);
    
    % Prepare plot for colour-filtered densitogram data:  Plotting options
    plotUnfilteredSignal = 'y';
    plotsignalFit = 'y';
    plotInitialSpotLocY = 'y';
    plotZfront = 'y';
    filterLabel = hueRangeLabel;
    dataColour = 'rgbcmyrgbcmy';    % Plotting colours
    
    % Generate subplot
    sp(2) = subplot(2,1,2); hold on;
    set(sp(2),'Position',[0.1 0.15 0.85 0.6]);
    xlabel('Position [px]'); ylabel('Filtered saturation sum [a.u.]');
    title(['Track loc. [x y w h] = [' num2str(defineTrack(t,1)) ' ' ...
        num2str(defineTrack(t,2)) ' ' num2str(defineTrack(t,3)) ' ' ...
        num2str(defineTrack(t,4)) ']']);
    
    % Add imported signal but not fitted signal for each filter
    for h = 1:nSpots             % For all hue filters
        plot(1:imgH,Densitogram(:,h,t),['.' dataColour(h)]);
    end
    
    % Add calculated unfiltered summed saturation Densitogram as grey
    % line if desired. Create unfiltered Densitogram with added offset
    % (10% of peak)
    if strcmp(plotUnfilteredSignal,'y')
        trk = rgb2hsv(imgT);
        trk(:,:,1) = 360 .* trk(:,:,1);
        % Unfiltered Densitogram with 10% offset
        UnfilteredSignal = ...
            sum(trk(:,:,2),2) + max(sum(trk(:,:,2),2))/10;
        plot(1:imgH,UnfilteredSignal,'Color',[0.5 0.5 0.5]);
        filterLabel{nSpots+1} = 'UNFILTERED, offset';
        clear trk UnfilteredSignal;
    end

    % Add legend for above signals
    legend(filterLabel, 'Location', 'NorthEast');
    
    % Add imported fitted Densitogram for each filter if desired
    if strcmp(plotsignalFit,'y')
        for h = 1:nSpots             % For all hue filters
            plot(1:imgH,fitDensitogram(:,h,t),['-' dataColour(h)]);
        end
    end
    
    % Add initial spot location if desired
    if strcmp(plotInitialSpotLocY,'y')
        % Vertical line at initilSpotLocY
        ylims = get(sp(2),'ylim');
        plot([spotLocationY(t);spotLocationY(t)],ylims,'--k');
    end
    
    % Add migration front position if desired
    if strcmp(plotZfront,'y')
        % Add initialSpotLocY back into zf1
        Zfront = zf(t) + spotLocationY(t);
        ylims = get(sp(2),'ylim');
        plot([Zfront;Zfront],ylims,'--k');   % Vertical line at Zfront
        clear Zfront;
    end
    linkaxes(sp,'x');
    xlim([0 imgH]);
    hold off;
    clear plotUnfilteredSignal plotUnfilteredSignal plotsignalFit;
    clear plotInitialSpotLocY plotZfront h sp;
    
    % Save figure
    saveas(fig,[dataPathEXP opPrefix '_trk' num2str(t) '.fig']);
    saveas(fig,[dataPathEXP opPrefix '_trk' num2str(t) '.png']);
    disp('    Figure exported to *.fig/*.png files with prefix:')
    disp(['    ' opPrefix '_trk' num2str(t)])
    
    % Analysis for this track complete
    disp(['    Completed analysis of track ' num2str(t) ' of ' ...
        num2str(nTracks)])
end
clear t s dataColour fig imgT Densitogram0 fitDensitogram0;

save([dataPath dataFile]); % Save changes

%% Figures-of-merit
disp(' ')
disp('PART 10: CALCULATE FIGURES-OF-MERIT')
disp(' ')

% Continue?
goProcess = input('Continue processing (y/n)?: ','s');
if ~strcmp(goProcess,'y')
    disp('    User stopped process.')
    return;
end
clear goProcess;

% hRF calculation
zf_2D = repmat(zf',[1 nSpots]);
spotHRF = 100 * zSpot ./ zf_2D;    % 100 * retention factor (RF)

% Peak width (FWHM)
spotFWHM = stdSpot*sqrt(8*log(2));

% Theoretical plate number and plate height
spotPN = (zSpot./stdSpot).^2;
spotPH = (stdSpot.^2) ./ zSpot;   % = migration distance / plate number

clear zf_2D;
disp('Calculated hRF, FWHM, PN, PH')

% Resolution
% Generate arrays to calculate resSpot(analyte1,analyte2,track) Example:
% resSpot(2,3,5) is the resolution between analytes 2 and 4 (=3+1) on track
% 5. Introduce a 3rd dimension (analyte2) to arrays with original
% dimensions (analyte x track).

% Square base of "array1"s has ROWS that correspond to analyte1 number
% (analytes 1:(nSpots-1)) and identical COLUMNS.
zSpot1 = permute(zSpot,[2 3 1]);  % dim1 corresponds to analyte1 number
zSpot1 = repmat(zSpot1(1:(nSpots-1),1,:),[1 (nSpots-1) 1]);
stdSpot1 = permute(stdSpot,[2 3 1]);    % Similar to above
stdSpot1 = repmat(stdSpot1(1:(nSpots-1),1,:),[1 (nSpots-1) 1]);

% Square base of "array2"s has COLUMNS that correspond to analyte2 number
% (analytes 2:nSpots) and identical ROWS.
zSpot2 = permute(zSpot,[3 2 1]);  % dim2 corresponds to analyte2 number - 1
zSpot2 = repmat(zSpot2(1,2:nSpots,:),[(nSpots-1) 1 1]);
stdSpot2 = permute(stdSpot,[3 2 1]);    % Similar to above
stdSpot2 = repmat(stdSpot2(1,2:nSpots,:),[(nSpots-1) 1 1]);

% Calculate resolution
resScale = 0.5; % Scaling factor depends on res criteria and spot width
resSpot = resScale * (zSpot1 - zSpot2) ./ (stdSpot1 + stdSpot2);

% Set all nonsensical and redundant resolution values to NaN 
%   Ex: resSpot(2,1,trk) = NaN since this considers spot 2 vs spot 2
%   Ex: resSpot(3,1,trk) = NaN since this considers spot 3 vs spot 2 and
%	resSpot(2,2,trk) already considers spot 2 vs spot 3.
% Use a lower triangular matrix of NaN values
resRedundant = repmat(tril(NaN*ones(nSpots-1),-1),[1 1 nTracks]);
resSpot = resSpot + resRedundant;
clear zSpot1 zSpot2 stdSpot1 stdSpot2 resScale resRedundant;
disp(['Calculated resolution between ' num2str(nSpots) ' spots.'])

save([dataPath dataFile]); % Save changes

%% FOM calculation and export; writing of summary file
disp(' ')
disp('PART 11: GENERATE SUMMARY INFO FILE AND EXPORT FIGURES-OF-MERIT')
disp(' ')

% Continue?
goProcess = input('Continue processing (y/n)?: ','s');
if ~strcmp(goProcess,'y')
    disp('    User stopped process.')
    return;
end
clear goProcess;

% Import all calculated FOMs into a structure:
disp('Importing all FOMs into a structure for convenient export.')
fom = load([dataPath dataFile], ...
    'spotVolume','spotMass', ...  % Spot loading informaton
    'RealPH','RealPN','SN','zf', ... % 1D arrays then 2D arrays (below)
    'ampSpot','sigSpot','spotFWHM','spotHRF','spotLOD0', 'spotLOD', ...
    'spotPH','spotPN','spotSNR','stdevBlankScaled','stdResiduals', ...
    'stdSpot','zSpot', ...
    'resSpot'); % 3D array

% Turn on diary file (delete old version first if it exists)
disp(' ')
if exist([dataPathEXP opPrefix '-info.txt'],'file')
    diary off;
    delete([dataPathEXP opPrefix '-info.txt']);
    disp(['* Deleted previous version of ' opPrefix '-info.txt'])
    disp(' ')
end
diary([dataPathEXP opPrefix '-info.txt']);
disp(['Begin generating info file: ' opPrefix '-info.txt'])
disp(' ')

% Information about image
disp('UTLC Image Analysis Information')
disp(['Data set name: ' opPrefix])
disp('DATA FILE INFORMATION ........................................')
disp(['    Data saved in directory: ' dataPath])
disp(['    File name: ' dataFile])
disp(['    Other files exported to: ' dataPathEXP])
disp(' ')

disp('BASIC INFORMATION ........................................')
disp(['    Original image name: ' img0File])
disp(['    Original image pixel dimensions (H x W): ' ...
    num2str(img0H) ' x ' num2str(img0W) ' px'])
if exist('tilt_avg','var')
    disp(['    Measured average tilt angle: ' num2str(tilt_avg) ...
        ' deg (CCW)'])
else
    disp('    Tilt angle not measured.')
end
disp(['    Corrected by rotating : ' num2str(img0_rotate) ' deg (CCW)'])
disp(['        - Tilt-corrected image exported: ' opPrefix '-rotate.jpg'])
disp('    The cropped image on which all analysis was performed:')
disp(['        - Has image pixel dimensions (H x W): ' ...
    num2str(imgH) ' x ' num2str(imgW) ' px'])
disp(['        - Was exported as: ' opPrefix '-correct.jpg'])
disp(['    Assumed frame resolution: ' num2str(imgRes) ' px/mm'])
disp(' ')
disp(['Number of separation tracks: ' num2str(nTracks)])
disp([num2str(nSpots) ' analyte colour filters were analyzed:'])
disp(hueRangeLabel)
disp('These were selected using hue measurements taken using nested')
disp('function ''sampleSpots.m'' and exported into:')
disp([opPrefix '_checkHue.mat'])
disp(' ')

disp('Migration distances (''zSpot'') and front positions (''zf'')')
disp('calculated with respect to the initial spot position.')
disp(['''zf'' measured on ' num2str(nTracks) ' (in px and in mm):'])
disp([zf; zf/imgRes])
disp(' ')

% Spot loading information
if exist('spotVolume','var') || exist('spotMass','var')
    disp('SPOT LOADING INFORMATION .................................')
    if exist('spotVolume','var')
        disp(['Spot volumes applied to ' num2str(nTracks) ...
            ' tracks are (nL):'])
        disp(spotVolume)
    end
    if exist('undilutedConc','var') && exist('relConc','var')
        disp(['Applied sample contained analytes with undiluted ' ...
            'concentrations (ng/nL):'])
        disp(undilutedConc)
        disp(['This was then diluted to ' num2str(100*relConc) '%.'])
    end
    if exist('spotMass','var')
        disp(['The applied masses for all analytes (rows) and tracks ' ...
            '(columns) are (ng):'])
        disp(spotMass)
    end
end

% Chromatogram extraction and fitting
disp(' ')
disp('CHROMATOGRAM EXTRACTION AND GAUSSIAN PEAK FITTING ........')
disp([num2str(nTracks) ' figures of colour filtered chromatograms'])
disp('generated and exported to *.png and MATLAB *.fig.  The last ')
disp(['has file names: ' opPrefix '_trk' num2str(nTracks) '(.png/.mat).'])

% File exporting
disp(' ')
disp('FIGURE-OF-MERIT DATA EXPORTED TO *.csv FILES .............')
disp('(Dimensions of original 1D and 2D arrays listed in file names.)')
disp(' ')

% List of 1D and 2D FOMs to consider
fomList1D = {'RealPH'; 'RealPN'; 'SN'; 'spotVolume'; 'zf'};
fomList1D_mm = {'RealPH'; 'zf'};
fomList2D = {'ampSpot'; 'sigSpot'; 'spotFWHM'; 'spotHRF'; 'spotLOD0'; ...
    'spotLOD'; 'spotPH'; 'spotPN'; 'spotSNR'; 'stdevBlankScaled'; ...
    'stdResiduals'; 'stdSpot'; 'zSpot'};
fomList2D_mm = {'spotFWHM'; 'stdSpot'; 'zSpot'};

% Export fomList1D
disp(' ')
disp(['1D arrays of dimension ' num2str(nTracks) ' tracks x 1 exported:'])
for j = 1:length(fomList1D)
    if isfield(fom,fomList1D{j})
        csvwrite([dataPathEXP opPrefix '_' fomList1D{j} '-' ...
            num2str(nTracks) '.csv'], fom.(fomList1D{j}));
        disp(['''' fomList1D{j} ''''])
    end
end
clear j;

% Export fomList3D
disp(' ')
disp(['2D arrays of dimension ' num2str(nTracks) ' tracks x ' ...
    num2str(nSpots) ' spots exported:'])
for j = 1:length(fomList2D)
    if isfield(fom,fomList2D{j})
        csvwrite([dataPathEXP opPrefix '_' fomList2D{j} '-' ...
            num2str(nTracks) '-' num2str(nSpots) '.csv'], ...
            fom.(fomList2D{j}));
        disp(['''' fomList2D{j} ''''])
    end
end
clear j;
% Migration distances and Plate height in microns
disp(' ')
disp('Export some figures-of-merit in real units (mm, um).')
for j = 1:length(fomList1D_mm)   % FOMs in mm
    if isfield(fom,fomList1D_mm{j})
        csvwrite([dataPathEXP opPrefix '_' fomList1D_mm{j} '_mm-' ...
            num2str(nTracks) '.csv'], ...
            (fom.(fomList1D_mm{j}) ./ imgRes) );  % Convert to mm
        disp(['''' fomList1D_mm{j} '_mm'''])
    end
end
clear j;
for j = 1:length(fomList2D_mm)   % FOMs in mm
    if isfield(fom,fomList2D_mm{j})
        csvwrite([dataPathEXP opPrefix '_' fomList2D_mm{j} '_mm-' ...
            num2str(nTracks) '-' num2str(nSpots) '.csv'], ...
            (fom.(fomList2D_mm{j}) ./ imgRes) );  % Convert to mm
        disp(['''' fomList2D_mm{j} '_mm'''])
    end
end
clear j;
% Plate height in um
if isfield(fom,'spotPH')
    csvwrite([dataPathEXP opPrefix '_spotPH_um-' num2str(nTracks) '-' ...
        num2str(nSpots) '.csv'], ...
        (fom.spotPH * 1000 ./ imgRes) );   % Convert to um
    disp('''spotPH_um''')
end

% Resolution
if isfield(fom,'resSpot')
    % Convert 3D array into 2D array
    % Original coordinates: spot1,(spot2-1),tracks
    % New coordinates: pair of spots1&2, tracks where pairing
    % gives columns for:
    %   1vs2, 1vs3, ..., 2vs2, 2vs3, ..., 3vs2, 3vs3, ..., (N-1)vsN
    % (Note some nonsensical and redundant pairings still exist have all
    % values = NaN)
    resSpotCSV = permute(fom.resSpot,[2 1 3]);
    resSpotCSV = reshape(resSpotCSV, ...
        [1 (nSpots-1)^2 nTracks]);
    resSpotCSV = permute(resSpotCSV,[3 2 1]); % nTracks x nPairs
    
    % Remove nonsensical and redunant pairings (all values = NaN)
    % Define a vector with 1's where columns should be removed
    % Lower triangular matrix similar to NaN-ed areas at base of original
    % 4D resSpot array
    redundant = tril(ones(nSpots-1,nSpots-1),-1)';
    redundant = reshape(redundant,[1 (nSpots-1)^2]);
    redundant = logical(redundant);
    resSpotCSV(:,redundant) = []; % Delete these columns from 2D array
    % Write to *.csv file
    csvwrite([dataPathEXP opPrefix '_resSpot-' num2str(nTracks) '-' ...
        num2str(size(resSpotCSV,2)) '.csv'], resSpotCSV);
    % Label columns in diary file
    % Spot #1: ex: [1 1 1 2 2 2 3 3 3]
    spotLabel1 = repmat(1:(nSpots-1), [(nSpots-1) 1]);
    spotLabel1 = reshape(spotLabel1,[1 (nSpots-1)^2]);
    % Spot #2: ex: [2 3 4 2 3 4 2 3 4]
    spotLabel2 = repmat(2:nSpots, [1 (nSpots-1)]);
    spotLabel = [spotLabel1; spotLabel2];
    spotLabel(:,redundant) = [];
    % Display exported information
    disp(' ')
    disp('''resSpot'' exported.  This 2D array has dimensions:')
    disp(['    ' num2str(nTracks) ' tracks x ' ...
        num2str(size(spotLabel,2)) ' spot pairings.'])
    disp('Columns correspond to pairings between the spot listed in the ')
    disp('    top row with that in the bottom row:')
    disp(spotLabel)
    disp('    ex: Column 1 of exported ''resSpot'' array denote ')
    disp('    between Spot 1 and Spot 2.')
    clear spotLabel* resSpotCSV;
end

%% Clean up, conclusion and time stamping
clear confirm* i j trackNum tempFrame dummyImage fom;
disp(' ')
runDate_singleImageUTLC = datestr(clock, 'yyyy-mm-dd, HH:MM');
disp(['Time stamp: ' runDate_singleImageUTLC])
disp(['Saving data file as: ''' dataFile ''''])
save([dataPath dataFile]); % Save changes
diary off;  % Stop recording output to info file.
disp('***SUCCESS SCRIPT FINALLY DONE!!!***')
end